//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//

//******************************************************************************
//*
//* Source File Name = TCallbackDES.CPP
//*
//* TCallbackDES - an class that represents a callback
//*               for DES
//*      
//*****************************************************************************
#include "TCallbackDES.hpp"
#include "DKDatastoreDES.hpp"
#include "DKBlobDES.hpp"
#include <stdio.h>
#include <stdlib.h>

// constructors

TCallbackDES::TCallbackDES()
{
 _threshold = 5;
 _cancel = FALSE;
 _isDone = FALSE;
 _numCalled = 0;
 _exc = NULL;
  cout << "*** callback created " << endl;
}

// destructor
TCallbackDES::~TCallbackDES()
{
    if (_exc != NULL)
    {
     delete _exc;
    }

}

//******************************************************************************
//* setThreshold - Sets the number of returned hits for each data callback
//*****************************************************************************

void TCallbackDES::setThreshold(long nValue)
{
 _threshold = nValue;
}

//******************************************************************************
//* getThreshold - Gets the number of returned hits that the server returns
//******************************************************************************

long TCallbackDES::getThreshold()
{
 return _threshold;
}

//******************************************************************************
//* cancelQuery - Indicates user has canceled the query
//******************************************************************************

void TCallbackDES::cancelQuery()
{
 _cancel = TRUE;
}

//******************************************************************************
//* dataCallback - Return nHits from the backend datastore
//******************************************************************************

DKBoolean TCallbackDES::dataCallback(dkCollection* ddoList, DKBoolean isDone)
{
 DKBoolean fDone = FALSE;
 _numCalled++;
 if (_cancel == TRUE || isDone == TRUE)
 {
  fDone = TRUE;
  _isDone = TRUE;
  processResults(ddoList);
 }
 else
 {
  cout << "*** " << _numCalled << " time called ***" << endl;
  processResults(ddoList);
 }
 delete ddoList;

 return fDone;
}

/******************************************************************************
 * reportException - Report an exception from the backend datastore
 *****************************************************************************/

void TCallbackDES::reportException(DKException* exc)
{
    if (_exc != NULL)
    {
     delete _exc;
    }

    _exc = exc;
}

/******************************************************************************
 * getException - Get an exception from the backend datastore
 *****************************************************************************/

DKException* TCallbackDES::getException()
{
 return _exc;
}

/******************************************************************************
 * resetException - Call after the exception has been deleted, this method
 * will clear the exception variable.
 *****************************************************************************/

void TCallbackDES::resetException()
{
  _exc = NULL;
}

//******************************************************************************
//* isDone - 
//******************************************************************************
DKBoolean TCallbackDES::isDone()
{
 return _isDone;
}


//******************************************************************************
//* processResults - 
//******************************************************************************
void TCallbackDES::processResults(dkCollection* results)
{
   unsigned short itemType;
   DKDDO* item;
   unsigned short i;
   DKString dataName;
   DKAny a;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   cout << " cardinality of results " << results->cardinality() << endl;
   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
      itemType = item->getPropertyByName(DK_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_DOCUMENT:
             cout << " Item is a document, " 
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << " data-item[" << i << "] = "
                      << dataName  << ", \t value "
                      << item->getData(i) << endl;
				a = item->getData(i);
				if (a.typeCode() == DKAny::tc_dobase)
				{
					dkDataObjectBase* pDOBase = a;
					dkXDO* pXDO = (dkXDO*)pDOBase;
					((DKBlobDES*)pXDO)->getContentToClientFile("c:\\temp\\temp.html", 1);
				}
				ushort usCount = item->dataPropertyCount(i);
				for (ushort k = 1; k <= usCount; k++)
				{
					DKAny a = item->getDataProperty(i, k);
					//cout << " property " << k << " " << a << endl;
				}	
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }
	  delete item;
	  item = 0;
   }
   delete iter;
}
