/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Attribute Group Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute Group Definition because it was not defined.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Attribute Group Definitions, Insurance Model Attribute Group
 *                    Definitions, and Other Sample Attribute Groups
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Attribute Group Definitions
 *                    Deleting Insurance Model Attribute Group Definitions
 *                    Deletes all Attribute Groups Defined in SAttributeGroupDefCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeGroupDefDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attribute Groups used here should be already defined.  If not, Sample
 *                    SAttributeGroupDefCreationICM may be run.  Sample will complete whether
 *                    the attribute groups exist or not.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefDeletionICM_launch.cpp
 *                    SAttributeGroupDefDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeGroupDefDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                           << endl;
            cout << "  SAttributeGroupDefDeletionICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."            << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "=============================================="     << endl;
        cout << "IBM Information Integrator for Content    v" << ver << endl;
        cout << "Sample Program:  SAttributeGroupDefDeletionICM"     << endl;
        cout << "----------------------------------------------"     << endl;
        cout << " Database: " << database                            << endl;
        cout << " UserName: " << userName                            << endl;
        cout << "==============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Retrieving an Attribute Group Definition
            //--------------------------------------------------------------
            // Retrieving the "S_carAccident" attribute Group.
            //
            // For more information on Attribute Group Retrieval, please consult
            // the Sample SAttributeGroupDefRetrievalICM.
            cout << "Retrieving Attribute Group 'S_carAccident'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM *) dsDefICM->retrieveAttrGroup("S_carAccident"); // Retrieve the definition.

            cout << "Retrieved Attribute Group 'S_carAccident'." << endl;

            //--------------------------------------------------------------
            // Delete Attribute Group Definition in Datastore Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's del() method can be called
            // as well as the Datastore Definition's del(attrGroupDef) method.
            //
            // This example will use the Attribute Group's del() method.
            cout << "Deleting Attribute Group Definition 'S_carAccident' in Datastore..." << endl;

                if(attrGroupDef!=NULL){    // if it is not already deleted...
                    attrGroupDef->del();   // Delete Attribute Group Definition in Datastore Definition
                    delete(attrGroupDef);  // Free Memory
            }

            cout << "Deleted Attribute Group Definition 'S_carAccident' in Datastore." << endl;

            //-------------------------------------------------------------
            // Retrieve & Delete "Address"
            //-------------------------------------------------------------
            // This method uses the Datastore Definition's del(attrGroup) method.
            cout << "Retrieving & Deleting Attribute Group Definition 'S_address'..." << endl;

                attrGroupDef = (DKAttrGroupDefICM *) dsDefICM->retrieveAttrGroup("S_address"); // Retrieve the definition.

                if(attrGroupDef!=NULL){           // if it is not already deleted...
                    dsDefICM->del(attrGroupDef);  // Delete Attribute Group Definition in Datastore Definition
                    delete(attrGroupDef);         // Free Memory
                }

            cout << "Deleted Attribute Group Definition 'S_address'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Attribute Group Definition
    * @param dsICM          Connected DKDatastoreICM object.
    * @param attrGroupName  Attribute Group Name for the Attribute Group Definition to Delete.
    **/
    void SAttributeGroupDefDeletionICM::deleteAttrGroupDef(DKDatastoreICM * dsICM, char * attrGroupName){
        cout << "Deleting the Attribute Group Definition '" << attrGroupName << "'..." << endl;

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM *) dsDefICM->retrieveAttrGroup(attrGroupName); // Retrieve the definition.
            if(attrGroupDef!=NULL){    // if it is not already deleted...
                attrGroupDef->del();   // Delete the Attribute Group Definition
                delete(attrGroupDef);  // Free Memory
            }

        cout << "Deleted the Attribute Group Definition '" << attrGroupName << "'." << endl;
    }
