/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    This sample pulls together all concepts learned so far, using the IBM COntent Management
    Data Modeling Concepts in a business model example.  The Insurance Model is defined and
    explained in SItemTypeCreationICM.
    
    Minimum Required Reading & Samples Run
        SConnectDisconnectICM                   SFolderICM
        SAttributeDefinitionCreationICM         SLinksICM
        SAttributeGroupDefinitionCreationICM    SResourceMgrDefCreationICM
        SReferenceAttrDefCreationICM            SResourceMgrSetDefaultICM
        SItemTypeCreationICM                    SSMSCollectionDefCreationICM
        SItemCreationICM                        SSMSCollectionDefSetDefaultICM
        SItemRetrievalICM                       SResourceItemCreaitonICM
        SItemUpdateICM                          SResourceItemRetrievalICM
        SItemDeletionICM

This sample shows how to simply delete each created in the creation sample.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemDeletionICM.hpp"
#include "SInsuranceModelItemCreationICM.hpp"
#include "SInsuranceModelItemDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SInsuranceModelItemDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deletes all Items that fit the Insurance Model.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes Items created for the Insurance Model.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SInsuranceModelItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeRetrievalICM.hpp/cpp          (Needed by SItemDeletionICM)
 *                    SItemCreationICM.hpp/cpp               (Needed by SItemDeletionICM)
 *                    SItemDeletionICM.hpp/cpp
 *                    SItemRetrievalICM.hpp/cpp              (Needed by SInsuranceModelItemCreationICM)
 *                    SLinksICM.hpp/cpp                      (Needed by SInsuranceModelItemCreationICM)
 *                    SInsuranceModelItemCreationICM.hpp/cpp
 *                    SInsuranceModelItemDeletionICM_launch.cpp
 *                    SInsuranceModelItemDeletionICM.hpp
 *                    SInsuranceModelICM_AccidenReport_0120200130.tif
 *                    SInsuranceModelICM_AccidenReport_0506200110.tif
 *                    SInsuranceModelICM_AccidenReport_729200173.tif
 *                    SInsuranceModelICM_Adjustment_4-852369.tif
 *                    SInsuranceModelICM_Adjustment_5-654123.tif
 *                    SInsuranceModelICM_Adjustment_6-987654.tif
 *                    SInsuranceModelICM_Claim_4-852369.tif
 *                    SInsuranceModelICM_Claim_5-654123.tif
 *                    SInsuranceModelICM_Claim_6-987654.tif
 *                    SInsuranceModelICM_Photo_1ABC089.jpg
 *                    SInsuranceModelICM_Photo_4IBM134-1.tif
 *                    SInsuranceModelICM_Photo_4IBM134-2.tif
 *                    SInsuranceModelICM_Photo_6usa676-1.tif
 *                    SInsuranceModelICM_Photo_6usa676-2.tif
 *                    SInsuranceModelICM_Policy_10285265410.tif
 *                    SInsuranceModelICM_Policy_10315975308.tif
 *                    SInsuranceModelICM_Policy_10452136412.tif
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SInsuranceModelItemDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                             << endl;
            cout << "  SInsuranceModelItemDeletionICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."              << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="     << endl;
        cout << "IBM Information Integrator for Content     v" << ver << endl;
        cout << "Sample Program:  SInsuranceModelItemDeletionICM"     << endl;
        cout << "-----------------------------------------------"     << endl;
        cout << " Database: " << database                             << endl;
        cout << " UserName: " << userName                             << endl;
        cout << "===============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Uses Function provided in the creation sample.
            cout << "Creating Insurance Model Items..." << endl;

                dkCollection* ddoColl = SInsuranceModelItemCreationICM::createInsuranceModelItems(dsICM);  // Uses Function provided in the creation sample.
                
                dkIterator* iter = ddoColl->createIterator();  // Create iterator to go through the DDOs returned.

                DKLobICM* autoClaim1      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* autoClaim2      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* autoClaim3      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo1a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo1b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo2        = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo3a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo3b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport1 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport2 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport3 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport1   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport2   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport3   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport4   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy1         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy2         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy3         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();

            cout << "Created Insurance Model Items." << endl;

            //-------------------------------------------------------------
            // Deleting All Items Created Above
            //-------------------------------------------------------------
            cout << "Deleting All Items Created Above..." << endl;

                autoClaim1->del();
                autoClaim2->del();
                autoClaim3->del();
                photo1a->del();
                photo1b->del();
                photo2->del();
                photo3a->del();
                photo3b->del();
                adjusterReport1->del();
                adjusterReport2->del();
                adjusterReport3->del();
                policeReport1->del();
                policeReport2->del();
                policeReport3->del();
                policeReport4->del();
                policy1->del();
                policy2->del();
                policy3->del();

                // Clean up Memory
                SInsuranceModelItemCreationICM::deleteMemoryObjectsReturnedByCreateInsuranceModelItems(ddoColl);

            cout << "Deleted All Items Created Above." << endl;

            //-------------------------------------------------------------
            // Delete all Items of the Insurance Model Item Types.
            //-------------------------------------------------------------
            cout << "Deleting all Items of the Insurance Model Item Types..." << endl;

                SItemDeletionICM::deleteAllItems(dsICM,"S_autoClaim");
                SItemDeletionICM::deleteAllItems(dsICM,"S_adjustrReport");
                SItemDeletionICM::deleteAllItems(dsICM,"S_policeReport");
                SItemDeletionICM::deleteAllItems(dsICM,"S_insurePolicy");
                SItemDeletionICM::deleteAllItems(dsICM,"S_autoPhoto");

            cout << "Deleted all Items of the Insurance Model Item Types." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
