/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating Items / Check Out / Locking
    In order to update an item, it must be checked out.  As mentioned in SItemRetrievalICM,
    an option can be specified during item retrieval to check out the item.  If it is not
    check out during retrieval, it simply needs to be explicitly checked out using the 
    DKDatastoreICM.checkOut(ddo) method.
    
    Child Components / Multivalue Attributes
        In order to update a child component or multivalue attribute, the child must be
        accessed through the root node and the update() method called on the root.
        Although child component DDOs may be retrieved, they can not be updated individually.

    Once the item is checked out, any DDO in the hierarchial tree structure may be modified
    the same way they were read in SItemRetrievalICM and set in SItemCreationICM.

    Update Options:

    Option       Contant                       Description
    -----------  ----------------------------  ------------------------------------------------
    New Version  DK_CM_VERSION_NEW             If Version Control is set to "By Application",
                                               for this Item Type, this option explicitly 
                                               specifies that it should create a new version 
                                               instead of updating the specific version checked
                                               out.  For more informaiton in Version Control
                                               options, refer to SItemTypeCreationICM Sample.
    ...........................................................................................
    Check In     DK_CM_CHECKIN                 The Item will be checked in / unlocked.
    (Default)
    ...........................................................................................
    Check Out    DK_CM_CHECKOUT                The Item will be checked out / locked.
    (Default)
    -------------------------------------------------------------------------------------------    
    Note:  Constants shown above are defined in DKConstant.h

Checking In / Unlocking
    An Item will remain checked out / locked until it is explicitly checked
    in either through Update Options or through the DKDatstoreICM.checkIn()
    function.  Please refer to the Update Options for more information.

Updating Child Components / Multivalue Attributes
    Multivalue attributes must be modified, added, and removed/deleted through
    the parent.  All operations must be made persistent through the parent DDO's
    DKDDO::add() or DKDDO::update() functions.
    
    Incremental Changes Tracked
        All changes to each individual DKChildCollection are tracked internally.  
        Each incremental change will be packaged up and sent to the Library Server
        in single call when the root component's DKDDO.update() function is called.
        It is important to note that the incremental changes are tracked and it is
        not the end result that is made persistent.  Optimal modifications rely on
        the fewest changes that result in the same end result.
    
    Updating a Child Component
        Updating a child component is performed simply by finding the desired
        DDO within the parent's DKChildCollection, modifying it, and then call
        DKDDO::update() on the root component.  
        
        The desired DDO must be located in the parent's DKChildCollection by
        creating a dkIterator from the collection, and searching for it based
        on PID String, Component ID, or based on attributes.
    
    Adding a Child Component
        As shown in SItemCreationICM, additin a child coimponent is performed
        simply adding a new child DDO to the parent's DKChildCollection, and 
        then calling DKDDO::update() on the root component.
        
    Removing/Deleting a Child Component
        Deleting a child component is performed through simply removing the
        undesired DKDDO object from the parent's DKChildCollection, and then
        calling DKDDO::update() on the root component.
        
        The desired DDO must be located in the parent's DKChildCollection by
        creating a dkIterator from the collection, and searching for it based
        on PID String, Component ID, or based on attributes.  The 
        DKChildCollection::removeElementAt(dkIterator) function may be used
        to remove the undesired DDO.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreExtICM.hpp>
#include <DKChildCollection.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemCreationICM.hpp"
#include "SItemRetrievalICM.hpp"
#include "SItemUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemUpdateICM.hpp/cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, Checking Out / Locking, Checking In / Unlocking Items,
 *                    Update Options
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Check Out / Locking
 *                    Check In / Unlocking
 *                    Modifying Items
 *                    Updating / Saving Items
 *                    Update Options
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SItemRetrievalICM.hpp/cpp
 *                    SItemUpdateICM_launch.cpp
 *                    SItemUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        const char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        const char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        const char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemUpdateICM <database> <userName> <password>"        << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        const char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemUpdateICM"                 << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of semantic type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            cout << "Creating & Saving Sample Items..." << endl;

                DKDDO* ddoDocument = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_DOCUMENT);
	            DKDDO* ddoFolder   = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_FOLDER);   
	            DKDDO* ddoItem     = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_ITEM);     

            cout << "Created & Saved Sample Items." << endl;

            //-------------------------------------------------------------
            // Check Out / Lock
            //-------------------------------------------------------------
            // Items may be checked out / locked explicitly using the 
            // DKDatastoreICM.checkOut() method, or by specifying the 
            // Check Out option during retrieval.
            cout << "Checking Out Items..." << endl;

                dsICM->checkOut(ddoDocument);
                dsICM->checkOut(ddoFolder);
                dsICM->checkOut(ddoItem);

            cout << "Checked Out Items." << endl;

            //-------------------------------------------------------------
            // Modify Item
            //-------------------------------------------------------------
            // Values may be read as explained in SItemRetrievalICM and may
            // be set as explained in SItemCreationICM.
            cout << "Modifying Item..." << endl;

                // Modify Attribute Values
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_varchar"), DKString("My modified string value")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_double"),  (double) 999); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_integer"), (long) 888); 

                ddoFolder->setData(ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,"S_varchar"), DKString("My modified string value")); 
                ddoFolder->setData(ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,"S_double"),  (double) 999); 
                ddoFolder->setData(ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,"S_integer"), (long) 888); 

                ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_varchar"), DKString("My modified string value")); 
                ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_double"),  (double) 999); 
                ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_integer"), (long) 888); 

            cout << "Modified Item." << endl;

            //-------------------------------------------------------------
            // Updating item in persistent datastore / Saving changes
            //-------------------------------------------------------------
            // Item will remain checked out / locked until it is explicitly
            // checked in either through Update Options or through the checkIn()
            // function.  Please refer to the Update Options specified in the
            // header documentation of this sample.
            cout << "Updating / Saving Item..." << endl;

                ddoDocument->update();             // Item will remain checked out.
                ddoFolder->update(DK_CM_CHECKIN);  // May specify Update Options
                ddoItem->update(DK_CM_VERSION_NEW + DK_CM_CHECKIN); // Save as new version.

            cout << "Updated / Saved Item." << endl;

            //-------------------------------------------------------------
            // Explicit Check In / Unlocking
            //-------------------------------------------------------------
            cout << "Explicitly Checking an item Out & In, Locking & Unlocking..." << endl;

                dsICM->checkOut(ddoFolder);
                
                cout << "Item Checked Out / Locked." << endl;
                
                dsICM->checkIn(ddoFolder);

            cout << "Item Checked In / Unlocked" << endl;

            //---------------------------------------------------------------
            // Determining if an Item is Locked / Checked Out & By Which User
            //---------------------------------------------------------------
            cout << "Determining if an Item is Locked / Checked Out & By Which User..." << endl;

                // Check Out Item
                cout << "Checking out item..." << endl;
                dsICM->checkOut(ddoFolder);
                
                // Create a new Datastore Extension object.
                DKDatastoreExtICM* dsExtICM = (DKDatastoreExtICM*) dsICM->getExtension(DK_CM_DATASTORE_EXT); // Get the Datastore Extension Object

                if(dsExtICM->isCheckedOut(ddoFolder)){
                    cout << "Item is Checked Out / Locked by Some User." << endl;
                    DKString checkedOutByStr = dsExtICM->checkedOutUserid(ddoFolder);
                    cout << "Item is Checked Out by '" << checkedOutByStr << "'." << endl;
                } else
                    cout << "Item is not Checked Out.  It is unlocked." << endl;
                
                cout << "Checking out item back in..." << endl;
                dsICM->checkIn(ddoFolder);

            cout << "Determined if an Item is Locked & By Which User." << endl;
            
            //-------------------------------------------------------------
            // Print Updated DDO 
            //-------------------------------------------------------------
            // Print a DDO using the wrapper from SItemRetrievalICM.            
            cout << "Printing DDO..." << endl;

                SItemRetrievalICM::printDDO(ddoDocument); // Use wrapper defined in SItemRetrievalICM.
            
            cout << "Printed DDO." << endl;

            //--------------------------------------------------------------
            // Add another value to a Multivalue Attribute / Child Component
            //--------------------------------------------------------------
            cout << "Adding another value to a Multivalue Attribute / Child Component..." << endl;

                cout << "Checking Out Item." << endl;  // Must check it out.
                dsICM->checkOut(ddoItem);
                
                cout << "Accessing Multivalue Attribute Named 'S_simpleChild' / Child Collection." << endl;
                // The name of the multivalue attribute is the object type of the child component defined in the Item Type.
                // SItemTypeCreationICM created the S_withChild Item Type with a child type / multivalue attribute of "S_simpleChild".
                short dataid = ddoItem->dataId(DK_CM_NAMESPACE_CHILD,"S_simpleChild");  // If the multivalue attribute doesn't exist, '0' will be returned.
                DKChildCollection* children = (DKChildCollection*)(dkCollection*) ddoItem->getData(dataid);
                if(children==NULL){ // if no collection exists, create one.
                    cout << "Creating New Child Collection..." << endl;
                    children = new DKChildCollection();
                    ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_CHILD, "S_simpleChild"), children);
                }
    
                cout << "Creating a new value / child ddo." << endl;
                DKDDO* child = dsICM->createChildDDO("S_withChild","S_simpleChild"); // Create a new child DDO.
                // ... Set any attributes or children of its own here.
                
                cout << "Adding the new value / child ddo to the parent's collection." << endl;
                children->addElement(child); // add children to the collection.
            
                cout << "Making changes persistent." << endl;
                ddoItem->update(DK_CM_CHECKIN);  // Explicitly unlocking item.

            cout << "Added another value to a Multivalue Attribute / Child Component." << endl;

            //--------------------------------------------------------------
            // Remove a value of a Multivalue Attribute / Child Component
            //--------------------------------------------------------------
            cout << "Removing a value of a Multivalue Attribute / Child Component..." << endl;

                cout << "Checking Out Item." << endl;  // Must check it out.
                dsICM->checkOut(ddoItem);
                
                cout << "Accessing Multivalue Attribute Named 'S_simpleChild' / Child Collection." << endl;
                // The name of the multivalue attribute is the object type of the child component defined in the Item Type.
                // SItemTypeCreationICM created the S_withChild Item Type with a child type / multivalue attribute of "S_simpleChild".
                dataid = ddoItem->dataId(DK_CM_NAMESPACE_CHILD,"S_simpleChild");  // If the multivalue attribute doesn't exist, '0' will be returned.
                children = (DKChildCollection*)(dkCollection*) ddoItem->getData(dataid);
    
                cout << "Searching for undesired child/value." << endl;
                // Let's say the component id is known.  We will then find it based on Component ID.
                DKString desiredComponentId = ((DKPidICM*)child->getPidObject())->getComponentId();
                dkIterator* iter = children->createIterator();
                bool found = false;  // Search until it is found.
                DKDDO* curChild = NULL;
                while(!found){ 
                    curChild = (DKDDO*)(dkDataObjectBase*) iter->next()->value(); // Get the current element and move pointer to next.
                    if(curChild==NULL) // if at end of collection, it was not found.
                        throw DKException("This child should exist in this sample since it was just added above.");
                    DKString foundComponentId = ((DKPidICM*)curChild->getPidObject())->getComponentId();
                    if(strcmp(foundComponentId,desiredComponentId)==0)
                        found = true;
                }
                
                cout << "Removing the value / child ddo from the parent's collection." << endl;
                children->removeElementAt(*iter); // Remove value / ddo.  Will remove last item returned by iter.next().
                delete curChild;                  //if you remove from childcollection, you must delete ddo.
                delete iter;

                cout << "Making changes persistent." << endl;
                ddoItem->update(DK_CM_CHECKIN);  // Explicitly unlocking item.

                delete(ddoDocument);                      // Free Memory
                delete(ddoFolder);
                delete(ddoItem); 

            cout << "Removed a value of a Multivalue Attribute / Child Component." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            exit(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
