/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Link Type Definitions, if it does not exist, null is returned.  
       Usage of the null object will result in a NullPointerException.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKLinkTypeDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SLinkTypeDefinitionRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving, Printing, & Listing Link Type Definitions in the System.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Link Type Definitions
 *                    Printing Link Type Definitions
 *                    Listing All Link Types Available in the System
 *                    Obtaining All Link Type Definitions in the System
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SLinkTypeDefinitionRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types must already be defined.  If not, please run Sample
 *                    SLinkTypeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SLinkTypeDefinitionRetrievalICM_launch.cpp
 *                    SLinkTypeDefinitionRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SLinkTypeDefinitionRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                              << endl;
            cout << "  SLinkTypeDefinitionRetrievalICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."               << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================"     << endl;
        cout << "IBM Information Integrator for Content      v" << ver << endl;
        cout << "Sample Program:  SLinkTypeDefinitionRetrievalICM"     << endl;
        cout << "------------------------------------------------"     << endl;
        cout << " Database: " << database                              << endl;
        cout << " UserName: " << userName                              << endl;
        cout << "================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Obtaining all Link Type Definitions in the System
            //----------------------------------------------------------------
            cout << "Obtaining All Link Type Definitions in the System..." << endl;
        
        	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        	    
        	    dkCollection* linkTypeColl = dsDefICM->listLinkTypes(); // Obtain all of the Link Type Definition objects in the system.
        	    // NOTE: dsDefICM.listLinkTypeName() could also be used to just obtain the names.

            cout << "Obtained All Link Type Definitions in the System." << endl;

            //----------------------------------------------------------------
            // List all Link Types Available in the System
            //----------------------------------------------------------------
            // Uses the link type definitions retrieved above.
            cout << "Listing All Link Types Available In System..." << endl;

        	    // Print each in the list retrieved above.
        	    cout << "All Link Types in System:  (" << linkTypeColl->cardinality() << ")" << endl;
        	    dkIterator* iter = linkTypeColl->createIterator(); // create an iterator go iterate through collection.
                while(iter->more()){                               // while there still more items in the colleciton, continue
        	        DKLinkTypeDefICM* linkTypeDef = (DKLinkTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
        	        cout << "     - " << linkTypeDef->getName() << ":  " << linkTypeDef->getDescription() << endl;
                    delete(linkTypeDef);                           // Free Memory.  Contents will not be deleted when collection deleted.
        	    }
                delete(iter);                                      // Free Memory.
                delete(linkTypeColl);                              // Free memory.  Contents are not deleted in this case.

            cout << "Listed All Link Types Available In System." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_policeReports Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined Link Types, please consult
            // the SLinkTypeDefinitionCreationICM sample.  For more information
            // on linking, please consult the SLinksICM Sample.
            //
            // NOTE:  When retrieving Link Type Definitions, if it does not exist, null is 
            //        returned.  Usage of the null object will result in a NullPointerException.
            cout << "Retrieving the Link Type Definition of 'S_policeReports'..." << endl;

                DKLinkTypeDefICM* linkTypeDef = (DKLinkTypeDefICM*) dsDefICM->retrieveLinkType("S_policeReports"); // Retrieve the definition.
                
            cout << "Retrieved the Link Type Definition of 'S_policeReports'." << endl;
            
            //---------------------------------------------------------------
            // Print the Link Type Definition
            //---------------------------------------------------------------
            cout << "Printing the Link Type Definition of 'S_policeReports'..." << endl;
                
                printLinkTypeDef(linkTypeDef); // Use wrapper funciton from this Sample.
                delete(linkTypeDef);           // Free Memory.

            cout << "Printed the Link Type Definition of 'S_policeReports'." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_otherPolicies Link Type Definition
            //----------------------------------------------------------------
            cout << "Retrieving the Link Type Definition of 'S_otherPolicies'..." << endl;

                linkTypeDef = (DKLinkTypeDefICM*) dsDefICM->retrieveLinkType("S_otherPolicies"); // Retrieve the definition.
                
            cout << "Retrieved the Link Type Definition of 'S_otherPolicies'." << endl;

            //---------------------------------------------------------------
            // Print the Link Type Definition
            //---------------------------------------------------------------
            cout << "Printing the Link Type Definition of 'S_otherPolicies'..." << endl;
                
                printLinkTypeDef(linkTypeDef); // Use wrapper funciton from this Sample.
                delete(linkTypeDef);           // Free Memory

            cout << "Printed the Link Type Definition of 'S_otherPolicies'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * List all Link Types available in the system.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SLinkTypeDefinitionRetrievalICM::listLinkTypes(DKDatastoreICM* dsICM){

        // Obtaining all Link Type Definitions in the System
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        dkCollection* linkTypeColl = dsDefICM->listLinkTypes(); // Obtain all of the Link Type Definition objects in the system.

        // List all Link Types Available in the System
        cout << "All Link Types in System:  (" << linkTypeColl->cardinality() << ")" << endl;
        dkIterator* iter = linkTypeColl->createIterator(); // create an iterator go iterate through collection.
        while(iter->more()){                               // while there still more items in the colleciton, continue
        	DKLinkTypeDefICM* linkTypeDef = (DKLinkTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
        	cout << "     - " << linkTypeDef->getName() << ":  " << linkTypeDef->getDescription() << endl;
            delete(linkTypeDef);                           // Free Memory.  Contents of collection will not be deleted when the colleciton is deleted.
        }
        delete(iter);                                      // Free Memory.
        delete(linkTypeColl);                              // Free Memory.  Contents will not be deleted in this call.
    }//end listLinkTypes

   /**
    * Print the specified Link Type Definition
    * @param linkTypeDef  Link Type Definition, DKLinkTypeDefICM.
    **/
    void SLinkTypeDefinitionRetrievalICM::printLinkTypeDef(DKLinkTypeDefICM* linkTypeDef){
        cout << "--------------------------------------------"        << endl;
        cout << "            Link Type Definition"                    << endl;
        cout << "---------------------------------------------"       << endl;
        cout << "           Name:  " << linkTypeDef->getName()        << endl;
        cout << "    Description:  " << linkTypeDef->getDescription() << endl;
        cout << "        ID Code:  " << linkTypeDef->getCode()        << endl;
        cout << "---------------------------------------------"       << endl;
    }

   /**
    * Print the specified Link Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param linkTypeName  Link Type Name.
    **/
    void SLinkTypeDefinitionRetrievalICM::printLinkTypeDef(DKDatastoreICM* dsICM, char* linkTypeName){
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKLinkTypeDefICM* linkTypeDef = (DKLinkTypeDefICM*) dsDefICM->retrieveLinkType(linkTypeName); // Retrieve the definition.
        printLinkTypeDef(linkTypeDef);  
        delete(linkTypeDef);                                        // Free Memory.
    }
