/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    Resource Items are an extension of non-resource Items.  The Item Samples cover the 
    core documentation.  Likewise, the Resource Item Samples will suppliment the 
    documentation and demonstration with material pertaining specifically to Resource
    Items.

Creating Resource Items
    Resource Items area created just as regular Items are.  XDOs extend DDOs.  Depending on
    the type of Resource Item, the XDO may be extended further.  
    
        Class Hierarchy
        
            Type     DDO       XDO       Extension
            -----   -----    --------    -----------
            Lob     DKDDO -> DKLobICM      
            Text    DKDDO -> DKLobICM -> DKTextICM
            Image   DKDDO -> DKLobICM -> DKImageICM
            Stream  DKDDO -> DKLobICM -> DKStreamICM
           
    Since each are still DKDDOs, as with Items, DKDDOs should always be created using the 
    DKDatastoreICM's createDDO() methods.  Using the DKDatastoreICM's createDDO methods
    allow the system to automatically setup important information in the DKDDO structure 
    and which allows for easier & greater functionality such as Resources, 
    CM Document Model, & Folders.  Creating DDOs through the DKDatastoreICM is the 
    official and recommended way to create Items.
    
    Items of type Resource returned from DKDatastoreICM.createDDO may be cast to the 
    correct XDO or Subclass depending on the XDO Classification.
    
    For more information on Creating Items in general and the DKDatastoreICM.createDDO()
    function, please refer to the SItemCreationICM Sample.

    Note:  The Image XDO does not currently implement any additional functionality.

MIME Type / Type of Content
    The MIME Type describes the type of content that is being stored, 
    such as "image/mpg".  System-defined MIME Types are already setup.  
    Additional User-Defined MIME Types may be created using the Mime Type 
    Management Object.
    
    For an extensive list of MIME Types, please refer to 
    SResourceItemMimeTypesICM.txt located in the 
	<CMBROOT>/Samples/java/icm directory.
    
    Below is a subset of SResourceItemMimeTypeICM.txt's list, displaying
    the msot commonly used MIME Types.  For information on programmatically
    listing the MIME Types available in the system, refer to the Application
    Programming Reference (API Javadoc).
    
    Most Commonly Used MIME Types   Commonly Associated File Extensions
    ------------------------------  -----------------------------------------
    application/msword              doc, dot, rtf
    application/octet-stream        bin, dms, lha, lzh, exe, class
    application/pdf                 pdf
    application/rtf                 rtf
    application/lotus-1-2-3         123, wk4, wk3, wk1, wks, wg1
    application/lotus-freelance     prz, pre
    application/lotus-wordpro       lwp, sam, mwp, smm
    application/vnd.ms-excel        xls, xlt, xlm, xld, xla, xlc, xlw, xll
    application/vnd.ms-powerpoint   ppt, pot, ppa, pps, pwz
    application/vnd.visio           vsd
    audio/basic                     au, snd, ulw
    audio/mpeg                      mpeg, mpg, m1s, m1a, mp2, mp3, mpm, mpa, mpga
    audio/x-aiff                    aif, aiff, aifc
    audio/x-midi                    midi, mid, smf, kar
    audio/x-wav                     wav
    image/bmp                       bmp, dib
    image/gif                       gif
    image/jpeg                      jpeg, jpg, jpe, jfif, pjpeg, pjp
    image/tiff                      tiff, tif
    text/html                       html, htm, shtml, plg
    text/plain                      txt, text
    text/xml                        xml, dtd
    video/mpeg                      mpeg, mpg, mpe, m1s, m1v, m1a, m75, m15, mp2
    video/quicktime                 mov, qt

Storing Content
    Content may be loaded & stored in the following ways:
    
    1. File loaded into XDO's local memory seperately such as when other
       attributes are being specified.  When all modification is complete
       the DDO/XDO may be added tot he persistent datastore.
            
          - DKLobICM::add()
     
    2. Directly from a file when adding to the persistent datastore.
    
          - DKLobICM::add(filename)
          
    3. Directly from a stream when adding to the persistent datastore.
       
          - DKLobICM::add(stream)

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceItemCreationICM.hpp"


/************************************************************************************************
 *          FILENAME: SResourceItemCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating Resource Items, Storing Resource Content, MIME Types, Creating
 *                    a Lob / Text / Image Resource Item, Storing Resource Content directly 
 *                    from a file / stream / local memory, & Loading a file into item's memory.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Resource Items
 *                    Storing Resource Content
 *                    MIME Types
 *                    Creating a Lob Resource Item
 *                    Creating a Text Resource Item
 *                    Creating an Image Resource Item
 *                    Storing Resource Content directly from a file
 *                    Storing Resource Content directly from a stream
 *                    Loading a file into a Resource Item's local memory
 *                    Storing Resource Content from Memory
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SResourceItemCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                        - SResourceMgrDefSetDefaultICM
 *                        - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceItemCreationICM_launch.cpp
 *                    SResourceItemCreationICM.hpp
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Document2.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Text2.txt
 *                    SResourceItemICM_Image1.bmp
 *                    SResourceItemICM_Image2.bmp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceItemCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SResourceItemCreationICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SResourceItemCreationICM"       << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Empty Resource Objects
            //-------------------------------------------------------------
            // Use the Datastore's createDDO() function to create the DDOs.
            // The returned DDO may be cast to the corresponding subclass
            // depending on the XDO Classification.
            //
            // Please refer to SItemCreationICM for all non-resource related
            // information.
            //
            // Remember that "S_lob", "S_text", & "S_image" are Item Types 
            // defined in SItemTypeCreationICM.
            cout << "Creating Resource Objects..." << endl;

                DKLobICM*     lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM*   text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM* image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

            cout << "Created Resource Objects." << endl;

            //-------------------------------------------------------------
            // Setting the MIME Type (type of content)
            //-------------------------------------------------------------
            // mimeType: The type of content, such as "image/mpg"
            //
            // For a list of common MIME Types, please refer to the 
            // header documentation and SResourceItemMimeTypesICM.txt
			// located in the <CMBROOT>/Samples/java/icm directory.
            cout << "Setting MIME Type / Type of Content..." << endl;

                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");

            cout << "Set MIME Type / Type of Content." << endl;

            //-------------------------------------------------------------
            // Store Content Directly From a File
            //-------------------------------------------------------------
            // In one operation, save the new item along with the specified conent.
            cout << "Adding Item, Storing Content Directly From File..." << endl;

                lob->add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text->add("SResourceItemICM_Text1.txt");     // storing the content from
                image->add("SResourceItemICM_Picture1.bmp"); // the specified file.

                delete(lob);                                 // Free Memory since these object will not be needed anymore in this sample.
                delete(text);
                delete(image);

            cout << "Added Item, Storing Content Directly From File." << endl;

            //-------------------------------------------------------------
            // Create Empty Resource Objects & Setting MIME Type
            //-------------------------------------------------------------
            // Create some new objects so that adding from a Stream can be
            // demonstrated.
            cout << "Creating Resource Objects & Setting MIME Type..." << endl;

                  lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                 text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

                // Setting the MIME Type (type of content)
                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");

            cout << "Created Resource Objects & Set MIME Type." << endl;

            //-------------------------------------------------------------
            // Create Streams
            //-------------------------------------------------------------
            // In order to store from a stream, we must first create a stream
            // for this sample to use.  The following will create streams from
            // the content files.
            cout << "Creating Streams..." << endl;

                // First, let's get a stream that we can use
                istream* iStream1 = new ifstream("SResourceItemICM_Document2.doc");
                istream* iStream2 = new ifstream("SResourceItemICM_Text2.txt");
                istream* iStream3 = new ifstream("SResourceItemICM_Picture2.bmp");

            cout << "Created Streams." << endl;
            
            //-------------------------------------------------------------
            // Store Content Directly From a Stream
            //-------------------------------------------------------------
            // In one operation, store content from a stream while saving 
            // to the persistent datastore
            cout << "Adding Item, Storing Content Directly From a Stream..." << endl;

                lob->add(iStream1,getFileSize("SResourceItemICM_Document2.doc"));    // Now add to datastore,
                text->add(iStream2,getFileSize("SResourceItemICM_Text2.txt"));   // storing the content from
                image->add(iStream3, getFileSize("SResourceItemICM_Picture2.bmp"));  // the specified stream

                delete(lob);           // Free Memory for object that are not needed anymore in this sample.
                delete(text);
                delete(image);
                delete iStream1;
                delete iStream2;
                delete iStream3;

                cout << "Added Item, Stored Content Directly From a Stream." << endl;

            //-------------------------------------------------------------
            // Create Empty Resource Objects & Setting MIME Type
            //-------------------------------------------------------------
            // Create some new objects so that adding from Memory can be
            // demonstrated.
            cout << "Creating Resource Objects & Setting MIME Type..." << endl;

                  lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                 text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

                // Setting the MIME Type (type of content)
                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");

            cout << "Created Resource Objects & Set MIME Type." << endl;

            //-------------------------------------------------------------
            // Load Content into Item's Local Memory.
            //-------------------------------------------------------------
            // Content may be loaded directly into the XDO's local memory.
            // When all modification is complete, it may then be added to
            // the datastore.
            cout << "Loading Content into Item's Local Memory..." << endl;
    
                lob->setContentFromClientFile("SResourceItemICM_Document1.doc"); // Load the file into memory.
                text->setContentFromClientFile("SResourceItemICM_Text1.txt");
                image->setContentFromClientFile("SResourceItemICM_Picture1.bmp");

            cout << "Loaded Content into Item's Local Memory." << endl;

            //-------------------------------------------------------------
            // Add to Datastore With Content Already in Memory.
            //-------------------------------------------------------------
            // When all modification is complete, it may be added to the
            // datastore.
            cout << "Adding to Datastore with Content Already in Memory..." << endl;

                lob->add();      // The content was already loaded into memory
                text->add();     // using the setContentFromClientFile(filename)
                image->add();    // function

                delete(lob);     // Free Memory for object no longer needed in this sample.
                delete(text);
                delete(image);

            cout << "Added to Datastore with Content Already in Memory." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.



long SResourceItemCreationICM::getFileSize(char * szFileName)
{
  long l,m;
  ifstream file (szFileName, ios::in|ios::binary);
  l = file.tellg();
  file.seekg (0, ios::end);
  m = file.tellg();
  file.close();
  return (m-l); 
  
}
