/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Deleting Resource Items
    Deleting Resource Items is exactly the same method as non-resource Item deletion,
    shown in SItemDeletionICM.  Please refer to SItemDeletionICM for complete deletion
    information.  As explained in the Resource Item Creation Sample, XDOs are DDOs.  
    Therefore all functionality of DDOs, including deletion, still applies.
    
    This sample will delete all items created by the Resource Item Samples.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemDeletionICM.hpp"
#include "SResourceItemDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceItemDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Resource Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Resource Items
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SResourceItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model should be defined.  Sample will still complete successfully
 *                    because it meets the desired end results.  For a demonstration, Items
 *                    should be created first.  If desired please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeRetrievalICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SItemDeletionICM.hpp/cpp
 *                    SResourceItemDeletionICM_launch.cpp
 *                    SResourceItemDeletionICM.hpp
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Image1.bmp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceItemDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SResourceItemDeletionICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SResourceItemDeletionICM"       << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Resource Object for Retrieval Demonstration
            //-------------------------------------------------------------
            // In order to demonstrate Updating of Resource Items, a few
            // Resource Items must first be created.
            cout << "Creating Resource Objects..." << endl;

                DKLobICM*     lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM*   text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM* image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

                // Set Mime Type / Type of Content
                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");
                
                // Store Resource Content & Save Resource Item
                lob->add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text->add("SResourceItemICM_Text1.txt");     // storing the content from
                image->add("SResourceItemICM_Picture1.bmp"); // the specified file.

            cout << "Created Resource Objects." << endl;

            //-------------------------------------------------------------
            // Deleting Resource Items
            //-------------------------------------------------------------
            // Resource Items are treated just as Non-Resource Items are in
            // regards to deletion.
            cout << "Deleting Resource Items..." << endl;

                lob->del();
                text->del();
                image->del();

                delete(lob);      // Free Memory for objects no longer needed in this sample.
                delete(text);
                delete(image);
        
            cout << "Deleted Resource Items." << endl;

            //-------------------------------------------------------------
            // Delete all items created by Resource Item Samples
            //-------------------------------------------------------------
            // Deleting all Items created by SResourceItemCreationICM,
            // SResourceItemRetrievalICM, & SResourceItemUpdateICM.
            //
            // Please refer to SItemDeletionICM for more informaiton.
            cout << "Deleting All Items Used in Resource Item Samples." << endl;

                SItemDeletionICM::deleteAllItems(dsICM,"S_lob");
                SItemDeletionICM::deleteAllItems(dsICM,"S_text");
                SItemDeletionICM::deleteAllItems(dsICM,"S_image");
            
            cout << "Deleted All Items Used in Resource Item Samples." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
