/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Resource Item Storage
    Resource Item's content is stored in a Resource Manager.  It is possible to specify
    which Resource Manager & which SMS Collection within the Resource Manager that each
    Resource Item is to be stored.  Multiple Resource Managers and multiple SMS 
    Collections may be available for use.
    
    Default Resource Manager & SMS Collection
        The Item Type Definition contains information that specifies the default
        Resource Manager and default SMS Collection that Items of that Item Type
        are to be stored if not otherwise specified.  Please refer to the corresponding
        wrapper function at the bottom of this sample, and to the Item Type Creation
        sample, SItemTypeCreationICM for more information and examples of how to 
        set and modify these properties.
        
        The Non-Media colleciton is created automatically with a Resource Manager
        Definition.

    The ICM Content Manager v8.2 install on Windows will create a Resource Manager object
    named "RMDB" by default and one SMS Collection, Non-Media by default.
    
    For more information on Resource Manager Definitions, please refer to the corresponding
    section below.
    
    For more information on SMS Collections, please refer to the SSMSCollectionDefICM Sample.

Setting Up a Resource Manager Definition
    A user-defined reference name is specified for future reference and use.  This only 
    needs to be done once per resource manager to be used with this datastore.

    Parameter Explanations:
        
        New User-Defined Reference Name
            The user-defined reference name to the one of the defined resource managers 
            in the Library Server.  This value can be any string with no spaces.
            Ex: "MY_RM".
            
            IMPORTANT NOTE:  The name will be shifted to all upper-case.  Please take note
                             of this for later retrieval or listing based on the Reference
                             Name.

            RESTRICTION:  In regular usage of the API, this may be any user-defined
                          name.  However, in some cases, such as if the migration tool
                          is to be used, the name must be the same as the database name.
                          Ex: "RMDB".
  
        RM Server INET Address          
            The address / name / hostname of the computer running the Resource Manager
            Server.
        
        RM Server Port
            Port at which Resource Manager is running on the server.  On Windows, this 
            is usually 80 or 9080.
            
        RM Platform
            Platform that the Resource Manager is running on.
            
            RM Platforms    Constant
            -------------   -----------------------------
             Windows (R)    RM_PLATFORM_WIN
             AIX     (R)    RM_PLATFORM_AIX
             SUN     (R)    RM_PLATFORM_SUN
             OS/390  (R)    RM_PLATFORM_390
            ---------------------------------------------
            Note: Constants shown above are defined in DKConstantICM.h
            Note: Windows(R) is a Registered Trademark of Microsoft Corporation.
                  AIX(R) is a Registered Trademark of IBM Corporation.
                  SUN(R) is a Registered Trademark of Sun Microsystems.
                  OS/390(R) is a Registered Trademark of IBM Corporation.
        
        RM Access Type
            Access type for the Resource Manager, such as HTTP, FTP, RTP, JDBC, LSRM, etc.

            RM Access Types   Constant
            ---------------   --------------------------------
                 HTTP         RM_ACCESSTYPE_HTTP 
                 FTP          RM_ACCESSTYPE_FTP
                 RTP          RM_ACCESSTYPE_RTP
                 JDBC         RM_ACCESSTYPE_JDBC
                 LSRM         RM_ACCESSTYPE_LSRM
                 HTTPS        RM_ACCESSTYPE_HTTPS
            --------------------------------------------------
            Note: Constants shown above are defined in DKConstantICM.h

        Access Data
            This is the default servelet to run.  This will be entered as a row in a 
            table in LS.  This is usually "/icmrm/ICMResourceManager".
            
        RM Token Duraction             
            How long an object will remain valid while retrieved and/or checked out.  
            In v8.2, tokens are not enforced, but will be in the future.  Default is 48 hours.
            If just starting, "0" is a common value to specify.
            
        RM User Name
            UserName to log onto the Resource Manager.
            
            NOTE: There appears to be two function that set and store the value seperately
                  that will need to be set, setUserid() & setRMUserid().

        RM Password
            Password for the RM User Name to log onto the Resource Manager.

        User Name (Not RM)
            Not to be set by User Application.  User Name that is used to 
            create the Resource Manager Definition.  This is not the RM 
            User Name.  This field will be set by the system.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SResourceMgrDefCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceMgrDefCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Resource Manager Definition, Explanation, & Setup / Configuration
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Accessing Datastore Administration
 *                    Accessing Configuration Management
 *                    Accessing RM Configuration Management
 *                    Setting up a Resource Manager Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SResourceMgrDefCreationICM 
 *                                         <New RM User-Defined Reference Name> 
 *                                         <RM Server INET Address>
 *                                         <RM Server Port (ex: 9080)>
 *                                         <RM Platform (0 - Win, 1 - AIX, 2 - Sun, 3 - 390)>
 *                                         <RM Access Type (1 - HTTP, 2 - FTP,  3 - RTP,
 *                                                          4 - JDBC, 5 - LSRM, 6 - HTTPS)>
 *                                         <RM Access Data (Ex: /icmrm/ICMResourceManager)>
 *                                         <RM Token Duration (Ex: 0)>
 *                                         <RM User Name>
 *                                         <RM Password>
 *                                         <database> <userName> <password>
 *                    NOTE: Please refer to "Parameter Explanations" in the header for 
 *                            explanations of each.
 *
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 * 
 *                    ex: SResourceMgrDefCreationICM MY_RM myRMComputerName.ibm.com 80 0 1 /icmrm/ICMResourceManager 0 icmadmin password
 *                    ex: SResourceMgrDefCreationICM MY_RM myRMComputerName.ibm.com 80 0 1 /icmrm/ICMResourceManager 0 icmadmin password icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Resource Manager Reference Name specified must not be defined.  If it is,
 *                    please remove / delete the SMS Collection & ResourceMgrDef first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SResourceMgrDefCreationICM_launch.cpp
 *                    SResourceMgrDefCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceMgrDefCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        char * rmServerInetAddress         = "NOT_SPECIFIED";
        short  rmServerPort                 = 0;
        short  dkConstantRM_PLATFORM        = 0;
        int    dkConstantRM_ACCESSTYPE        = 0;
        char * rmAccessData                = "NOT_SPECIFIED";
        int    rmTokenDuration                = 0;
        char * rmUserName                  = "NOT_SPECIFIED";
        char * rmPassword                  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 13) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                                  << endl;
            cout << "   SResourceMgrDefCreationICM"                                            << endl;
            cout << "                      <RM User-Defined Reference Name>"                   << endl;
            cout << "                      <RM Server INET Address>"                           << endl;
            cout << "                      <RM Server Port (ex: 9080)>"                        << endl;
            cout << "                      <RM Platform (0 - Win, 1 - AIX, 2 - Sun, 3 - 390)>" << endl;
            cout << "                      <RM Access Type (1 - HTTP, 2 - FTP,  3 - RTP,"      << endl;
            cout << "                                       4 - JDBC, 5 - LSRM, 6 - HTTPS)>"   << endl;
            cout << "                      <RM Access Data (Ex: /icmrm/ICMResourceManager)>"   << endl;
            cout << "                      <RM Token Duration (Ex: 0)>"                        << endl;
            cout << "                      <RM User Name>"                                     << endl;
            cout << "                      <RM Password>"                                      << endl;
            cout << "                      <database> <userName> <password>"                   << endl;
            cout << endl;
            cout << "  NOTE: Please refer to 'Parameter Explanations' in the header for "      << endl;
            cout << "        explanations of each."                                            << endl;
            cout << endl;
            cout << "  NOTE: Defaults will be used if the database, username, & password are not" << endl;
            cout << "        specified.  All others are required."                                << endl;
            cout << endl;
            cout << " ex: SResourceMgrDefCreationICM MY_RM myRMComputerName.ibm.com 80 0 1"    << endl;
            cout << "                                /icmrm/ICMResourceManager 0 icmadmin"     << endl;
            cout << " ex: SResourceMgrDefCreationICM MY_RM myRMComputerName.ibm.com 80 0 1"    << endl;
            cout << "                                /icmrm/ICMResourceManager 0 icmadmin"     << endl;
            cout << "                                password icmnlsdb icmadmin password"      << endl;
        }
        if(argc < 10){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 13)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];
        if (argc > 2) rmServerInetAddress        = argv[2];
        if (argc > 3) rmServerPort               = (short)atoi(argv[3]);
        if (argc > 4) dkConstantRM_PLATFORM      = (short)atoi(argv[4]);
        if (argc > 5) dkConstantRM_ACCESSTYPE    = (int)atoi(argv[5]);
        if (argc > 6) rmAccessData               = argv[6];
        if (argc > 7) rmTokenDuration            = (int)atoi(argv[7]);
        if (argc > 8) rmUserName                 = argv[8];
        if (argc > 9) rmPassword                 = argv[9];

        // Store optional commandline arguments.
        if (argc > 10) database = argv[10];
        if (argc > 11) userName = argv[11];
        if (argc > 12) password = argv[12];
        char * ver = SConnectDisconnectICM_EIP_VERSION;
            
        cout << "==========================================="           << endl;
        cout << "IBM Information Integrator for Content v" << ver       << endl;
        cout << "Sample Program:  SResourceMgrDefCreationICM"           << endl;
        cout << "-------------------------------------------"           << endl;
        cout << "           Database:  " << database                    << endl;
        cout << "           UserName:  " << userName                    << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName  << endl;
        cout << "RM Server INET Addr:  " << rmServerInetAddress         << endl;
        cout << "     RM Server Port:  " << rmServerPort                << endl;
        cout << "        RM Platform:  " << dkConstantRM_PLATFORM       << endl;
        cout << "     RM Access Type:  " << dkConstantRM_ACCESSTYPE     << endl;
        cout << "     RM Access Data:  " << rmAccessData                << endl;
        cout << "  RM Token Duration:  " << rmTokenDuration             << endl;
        cout << "        RM UserName:  " << rmUserName                  << endl;
        cout << "===========================================\n"         << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Obtaining the Configuration Management Objects
            //-------------------------------------------------------------
            // The sample code blow demonstrates how to obtain objects for
            // Datastore Definition, Datastore Administration, Configuration
            // Management, and RM Configuration Management.
            cout << "Obtaining Configuration Management Objects..." << endl;

                DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
                DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
                DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
                
            cout << "Obtained Configuration Management Objects." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will utilize the wrapper functions provided at 
            // the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Setup New RM Definition
            //-------------------------------------------------------------
            // Setup a new Resource Manager Definition.
            //
            // Please refer to SResourceMgrDefRetrievalICM for all listing,
            // printing, rmCodes, and more.
            cout << "Setting up RM Definition..." << endl;
            
                short rmCode = setupResourceManager(dsICM, rmUserDefinedReferenceName, rmServerInetAddress, rmServerPort, dkConstantRM_PLATFORM, dkConstantRM_ACCESSTYPE, rmAccessData, rmTokenDuration, rmUserName, rmPassword);

                cout << "RM Definition '" << rmUserDefinedReferenceName << "' setup with ID Code '" << rmCode << "'." << endl;

            cout << "Set up RM Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setupResourceManager
    //******************************************************************
   /**
    * Setup & Configure a Resource Manager for use with this datastore.
    * A reference name is specified for future reference and use.  This
    * only needs to be done once per resource manager to be used with this
    * datastore.
    * @param dsICM                        Connected DKDatastoreICM object.
    * @param newUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It will be shifted to all upper-case.
    * @param rmServerInetAddress          The address / name of the computer running the Resource Manager Server.
    * @param rmServerPort                 Port at which Resource Manager is running on the server.
    * @param dkConstantRM_PLATFORM        Platform that the Resource Manager is running on.  Available options are common/DKConstant.RM_PLATFORM_xxxxxx.
    * @param dkConstantRM_ACCESSTYPE      Access type for the Resource Manager (HTTP, FTP, RTP, JDBC, LSRM, etc.).  Available options are common/DKConstant.RM_ACCESSTYPE_xxxxxx.
    * @param rmAccessData                 This is the default servelet to run.  This will be entered as a row in a table in LS.
    * @param rmTokenDuraction             How long an object will remain valid while retrieved and/or checked out.  In v8.2, tokens are not enforced, but will be in the future.  Default is 48 hours.
    * @param rmUserName                   UserName to log onto the Resource Manager.
    * @param rmPassword                   Password for the rmUserName.
    * @return Returns the new RM ID Code in the datastore.
    **/
    short SResourceMgrDefCreationICM::setupResourceManager(DKDatastoreICM* dsICM, char* newUserDefinedReferenceName, char* rmServerInetAddress,short rmServerPort, short dkConstantRM_PLATFORM, int dkConstantRM_ACCESSTYPE, char* rmAccessData, int rmTokenDuration, char* rmUserName, char* rmPassword){
        cout << "Setting up Resource Manager..." << endl;
        cout << "     New User Defined Reference Name: " << newUserDefinedReferenceName  << endl;
        cout << "                 Server INET Address: " << rmServerInetAddress          << endl;    
        cout << "                                Port: " << rmServerPort                 << endl;
        if(dkConstantRM_PLATFORM==RM_PLATFORM_WIN)
            cout << "                            Platform: WIN (" << dkConstantRM_PLATFORM << ")" << endl;
        else if(dkConstantRM_PLATFORM==RM_PLATFORM_AIX)
            cout << "                            Platform: AIX (" << dkConstantRM_PLATFORM << ")" << endl;
        else if(dkConstantRM_PLATFORM==RM_PLATFORM_SUN)
            cout << "                            Platform: SUN (" << dkConstantRM_PLATFORM << ")" << endl;
        else if(dkConstantRM_PLATFORM==RM_PLATFORM_390)
            cout << "                            Platform: 390 (" << dkConstantRM_PLATFORM << ")" << endl;
        else
            throw DKException(DKString("Resource Manager Platform of '")+DKString(dkConstantRM_PLATFORM)+DKString("' is not supported."));
        if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_HTTP)
            cout << "                         Access Type: HTTP ("  << dkConstantRM_ACCESSTYPE << ")" << endl;
        else if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_FTP)
            cout << "                         Access Type: FTP ("   << dkConstantRM_ACCESSTYPE << ")" << endl;
        else if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_RTP)
            cout << "                         Access Type: RTP ("   << dkConstantRM_ACCESSTYPE << ")" << endl;
        else if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_JDBC)
            cout << "                         Access Type: JDBC ("  << dkConstantRM_ACCESSTYPE << ")" << endl;
        else if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_LSRM)
            cout << "                         Access Type: LSRM ("  << dkConstantRM_ACCESSTYPE << ")" << endl;
        else if(dkConstantRM_ACCESSTYPE==RM_ACCESSTYPE_HTTPS)
            cout << "                         Access Type: HTTPS ("  << dkConstantRM_ACCESSTYPE << ")" << endl;
        else
            throw DKException(DKString("Resource Manager Access Type of '")+DKString(dkConstantRM_ACCESSTYPE)+DKString("' is not supported."));

        // Make sure this reference name isn't already used.  See SResourceMgrDefRetrievalICM Sample for more information.
        if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM, newUserDefinedReferenceName))
            throw DKException(DKString("A Resource Manager is already defined using the specified user-defined reference name of '")+DKString(newUserDefinedReferenceName)+DKString("'."));

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Create Resource Manager Definition
        DKResourceMgrDefICM* rmDef = new DKResourceMgrDefICM();  // Create new Resource Manager Definition object.
        rmDef->setName(newUserDefinedReferenceName);             // Set the user-defined reference name to be used for future reference.
        rmDef->setPlatform(dkConstantRM_PLATFORM);               // Set the RM's platform.
        rmDef->setINETAddress(rmServerInetAddress);              // Set the Address / Name of the computer that the server is running on.

        // Define Access Type
        DKAccessTypeDef* accessTypeDef = new DKAccessTypeDef();  // Create a new Access Type for the data, to describe HTTP, FTP, etc.
        accessTypeDef->setAccessType(dkConstantRM_ACCESSTYPE);   // Specify the access type, HTTP, FTP, LSRM, etc.
        accessTypeDef->setAccessData(rmAccessData);
        accessTypeDef->setPortNumber(rmServerPort);              // Set the port at which the Resource Manager is running.
        rmDef->addAccessType(accessTypeDef);                     // Add the acess type to the definition.

        // Set Additional Information
        rmDef->setTokenDuration(rmTokenDuration);                // Set the token duration for the Resource Manager.
        rmDef->setRMUserid(rmUserName);                          // Specify the logon information for the Resource Manager. 
        rmDef->setRMPasswd(rmPassword);

        rmCfg->addResourceMgr(rmDef);                            // Add the Resource Manager Definition

        short rmCode = rmDef->getRMCode();                       // Get return code of new definition
        cout << "Added RM.  ID Code: '" << rmCode << "'." << endl;
        
        // Double Checking to Ensure Correct...
        DKResourceMgrDefICM* retrRM = rmCfg->retrieveResourceMgr(newUserDefinedReferenceName); // Retrieve the definition
        short retrRMCode = retrRM->getRMCode();                  // Get he ID Code
        if(rmCode!=retrRMCode) // if they aren't the same, there is a problem.
            throw DKException(DKString("Retrieved Resource Manager Definition's ID Code ('")+DKString(retrRMCode)+DKString("') doesn't match the one just added ('")+DKString(rmCode)+DKString("')."));
        
        delete rmDef;       // Free Memory
        delete(retrRM);     // Free Memory

        cout << "Setup Specified Resource Manager." << endl;
        return(rmCode);
    }//end setupResourceManager
  
