// ----------------------------------------------------------------------------
// @copyright(disclaimer) 
// Licensed Materials - Property of IBM
// IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
// (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
// 
// US Government Users Restricted Rights                                  
// Use, duplication or disclosure restricted by GSA ADP Schedule          
// Contract with IBM Corp.     
// 
// DISCLAIMER OF WARRANTIES :                                             
//                                                                        
// Permission is granted to copy and modify this  Sample code, and to
// distribute modified versions provided that both the copyright        notice,-
// and this permission notice and warranty disclaimer  appear in all copies and
// modified  versions. 
// 
// THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM  AND ITS SUPPLIERS AND LICENSORS
// DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE,
// INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
// IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
// OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
// SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN
// NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST
// REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
// CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
// REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR
// SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
// @endCopyright       
// ----------------------------------------------------------------------------

//******************************************************************************
//*
//* Source File Name = TEvaluatedKQDD.CPP
//*
//* TEvaluatedKQDD - Test datastore.evaluate
//*               for Domino.Doc connector
//*      
//*****************************************************************************
#include "stdafx.h"
#include "TEvaluatedKQDD.h"

#include "DKDatastoreDD.hpp"
#include "dkQueryableCollection.hpp"
#include "DKParametricQuery.hpp"
#include "dkResultSetCursor.hpp"
#include "DKAny.hpp"
#include "DKResults.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

void printDDO(DKDDO* ddo)
{
	if(ddo != NULL)
	{
		DKPid p = ddo->getPid();
		cout << "************PID************" << endl;
		cout << "PID ID: " << p.getId() << endl;
		cout << "PID DS: " << p.getDatastoreType() << endl;
		cout << "PID DS: " << p.getDatastoreName() << endl;
		cout << "PID ObjType: " << p.getObjectType() << endl;
		cout << "ID String Count: " << p.getIdStringCount() << endl;
		cout << "************DDO************" << endl;
		cout << "Object Type: " << ddo->getObjectType() << endl;
		cout << "Data Count: " << ddo->dataCount() << endl;
		for(int i = 1; i <= ddo->dataCount(); i++)
		{
			DKAny any = ddo->getData(i);
			DKString dataName = ddo->getDataName(i);
			cout << dataName << ":\t\t" << any << endl;
		}
	}
}


void printResults(DKResults* results)
{
	cout << "Cardinality: " << results->cardinality() << endl;
	
	DKSequentialIterator* iterator = NULL;
	dkIterator* it = NULL;
	it = results->createIterator();
	iterator = (DKSequentialIterator*)it;
	while(iterator->more())
	{
		DKAny* any = iterator->next();
		dkDataObjectBase* doB = (dkDataObjectBase*)any->value();
		DKDDO* doc = (DKDDO*)doB;
		if(doc != NULL)
		{
			printDDO(doc);
			delete doc;
		}
	}
	if(iterator != NULL)
		delete iterator;
}
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}

	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TListAttrsDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	if( nbrArgs < 4 )
	{
	    cout << "TListAttrsDD usage = <serverName> <userId> <pw>" << endl;
		return -1;
	}

	DKDatastoreDD* dd;
	try
	{
		DKString server = argv[1];
		DKString uid = argv[2];
		DKString pw = argv[3];
		dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{

			/**** QUERY 1 ****/

			DKString qString0("(([\"DocType\"]CONTAINS\"DWF*\")  AND  ([\"Title\"]CONTAINS\"H*\"))");
			DKString queryParms0("ENTITY=WebPublishing;QUERY=");
			DKString queryString0 = queryParms0 + qString0;
			cout << "Query: " << queryString0 << endl;
			
			DKParametricQuery* paramQuery = (DKParametricQuery*)((dkQuery*)(dd->createQuery(queryString0,DK_PARAMETRIC_QL_TYPE, NULL)));
			DKAny any = dd->evaluate(paramQuery);
			DKResults* pRCS = (DKResults*)any.value();
			printResults(pRCS);
			delete paramQuery;
			delete pRCS;			
			
	
			/**** QUERY 2 ****/

			DKString qString1("((NOT([\"DocType\"]CONTAINS\"DWF*\"))  AND  ([\"Title\"]CONTAINS\"G*\"))");
			DKString queryParms1("ENTITY=WebPublishing;QUERY=");
			DKString queryString1 = queryParms1 + qString1;
			cout << "Query: " << queryString1 << endl;

			paramQuery = (DKParametricQuery*)((dkQuery*)(dd->createQuery(queryString1,DK_PARAMETRIC_QL_TYPE, NULL)));
			any = dd->evaluate(paramQuery);
			pRCS = (DKResults*)any.value();
			cout << "Before calling printResult\n";
			printResults(pRCS);
			cout << "After calling printResult\n";

			delete paramQuery;
			delete pRCS;			
			cout << "Before disconnect\n";
			dd->disconnect();
			cout << "After called disconnect\n";
			
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
				cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TEvaluatedKQDD FAILURE! Caught unexpected exception." << endl;
	}

			cout << "Before calling delete\n";

	delete dd;
				cout << "After calling deleted\n";

	return nRetCode;

}

