/**************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/**************************************************************************/

// sample connect/execute using query string/disconnect to VI400
/*
 * Filename       : TExecuteV4.cpp
 * Description    : Connect to VI400 Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 15 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from VI400 Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreV4.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobV4.hpp>
#include <DKPidXDOV4.hpp>
#include <DKDate.hpp> 
#include <time.h>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
    
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 3)
    {
      cout<<"Invoke Syntax: "<<endl;
      cout<<"  TExecuteV4 <Library Server> <userid> <password>" << endl;
      return;
    }
    
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw     = DKString(argv[3]);
    DKDatastoreV4 dsV4;
  
    try 
    {
      dkResultSetCursor* pCur = 0;
      DKAny any;
      cout << "Datastore V4 created" << endl;
      cout << "connecting to datastore..." << endl;
      dsV4.connect(libsrv,userid,pw);
      cout << "  datastore connected, libsrv: " << libsrv 
           << ", userid: " << userid << endl << endl;
           
      DKString cmd = "SEARCH=(INDEX_CLASS=DATATYPE,";
      cmd += "MAX_RESULTS=15,";
      cmd += "COND=(Data_FString <> NULL));";
      cmd += "OPTION=(CONTENT=YES;";
      cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";

      cout << "query string: " << cmd << endl;

      cout << "executing query..." << endl;
      time_t ltime;

      time( &ltime );
      printf( "Starting time is %s", ctime( &ltime ) );

      pCur = dsV4.execute(cmd);
      cout << "  query executed" << endl;
      time( &ltime );
      printf( "Ending time is %s\n", ctime( &ltime ) );
      cout << "\n........ Displaying query results ......... \n\n";
    
      DKDDO *p = 0;
      dkDataObjectBase *pDOBase = 0;
      DKDDO *pDDO = 0;
      dkXDO *pXDO = 0;
      DKAny a;
      ushort j = 0;
      ushort k = 0;
      ushort val = 0;
      ushort cnt = 1;
      short sVal = 0;
      long lVal = 0;
      DKString strData = "";
      DKString strDataName = "";
      dkCollection* pdCol = 0;
      dkIterator*  pdIter = 0;
      ushort numDataItems = 0;
      DKPid pid;
    
      while (pCur->isValid())
      {
        p = pCur->fetchNext();
     
        if (p != 0)
        {
          cout << "==========> " << "Item " << cnt << " <=========" << endl;
          numDataItems = p->dataCount();
          pid = p->getPid();
          cout << "  Pid String: " << pid.pidString() << endl;
          k = p->propertyId(DK_PROPERTY_ITEM_TYPE);
      
          if (k > 0)
          {
            a = p->getProperty(k);
            val = a;
            cout << "  ******************************" << endl;
       
            switch (val)
            {
              case DK_DOCUMENT :
              {
                cout << "  Item is a document " << endl;
                break;
              }
              case DK_FOLDER :
              {
                cout << "  Item is a folder " << endl;
                break;
              }
            }
       
            cout << "  ******************************" << endl;
          }
      
          cout << "  Number of Data Items: " << numDataItems << endl;
      
          for (j = 1; j <= numDataItems; j++)
          {
            a = p->getData(j);
            strDataName = p->getDataName(j);
       
            switch (a.typeCode())
            {
              case DKAny::tc_string :
              {
                strData = a;
                cout << "  attribute name: " << strDataName 
                     << ", value: " << strData << endl;
                break;
              }
        
              case DKAny::tc_long :
              {
                lVal = a;
                cout << "  attribute name: " << strDataName 
                     << ", value: " << lVal << endl;
                break;
              }
        
              case DKAny::tc_null :
              {
                cout << "  attribute name: " << strDataName 
                     << ", value:  NULL " << endl;
                break;
              }
        
              case DKAny::tc_collection :
              {
                pdCol = a;
                cout << strDataName << " collection name: " << strDataName << endl;
                cout << "--------------------------------------" << endl;
                pdIter = pdCol->createIterator();
                ushort b = 0;
         
                while (pdIter->more() == TRUE)
                {
                  b++;
                  cout << "  -------------------------------------------------" << endl;
                  a = *(pdIter->next());
                  pDOBase = a;
            
                  if (pDOBase->protocol() == DK_PDDO)
                  {
                    pDDO = (DKDDO*)pDOBase;
                    cout << "  DKDDO object " << b << " in " << strDataName 
                         << " collection " << endl;
                    k = pDDO->propertyId(DK_PROPERTY_ITEM_TYPE);
             
                    if (k > 0)
                    {
                      a = pDDO->getProperty(k);
                      val = a;
                      cout << "  ******************************" << endl;
               
                      switch (val)
                      {
                        case DK_DOCUMENT :
                        {
                          cout << "  Item is a document " << endl;
                          break;
                        }
                        case DK_FOLDER :
                        {
                          cout << "  Item is a folder " << endl;
                          break;
                        }
                      }
                      cout << "  ******************************" << endl;
                    }
                  }
                  else if (pDOBase->protocol() == DK_XDO)
                  {
                    pXDO = (dkXDO*)pDOBase;
                    cout << "  dkXDO object " << b << " in " << strDataName 
                         << " collection " << endl;                  
                  }
                }
 
                if (pdIter != 0)
                {
                  delete pdIter;
                }
         
                if (b == 0)
                {
                  cout << strDataName << " collection has no elements " << endl;
                }
         
                cout << "  -------------------------------------------------" << endl;
                break;
              }
        
              default:
                cout << "Type is not supported\n";
            }
       
            cout << "    type: " 
                 << p->getDataPropertyByName(j,DK_PROPERTY_TYPE) << endl;
            cout << "    nullable: " 
                 << p->getDataPropertyByName(j,DK_PROPERTY_NULLABLE) << endl;
       
            if (strDataName != DKPARTS && strDataName != DKFOLDER)
            {
              cout << "    attribute id: " 
                   << p->getDataPropertyByName(j,DK_PROPERTY_ATTRIBUTE_ID) 
                   << endl;
            }
       
          }
      
          cnt++;
          delete p;
        }
      }
    
      cout << "Total Item count is " << cnt-1 << endl;
     
      if (pCur != 0)
        delete pCur;
       
      time( &ltime );
      printf( "Current time is %s\n", ctime( &ltime ) );

      dsV4.disconnect();
      cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id:             " << exc.errorId() << endl;
      cout << "Exception id          " << exc.exceptionId() << endl;
      cout << "Error code            " << exc.errorCode() << endl;
      cout << "Error state:          " << exc.errorState() << endl;
      
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text:          " << exc.text(i) << endl;
      }
      
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename:            " << p->fileName() << endl;
       cout << "Function:            " << p->functionName() << endl;
       cout << "LineNumber:          " << p->lineNumber() << endl;
      }
      
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    
    cout << "done!" << endl;
}

