/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Adds a link between one item and another item or items.  This sample works with CMv8 only.
 */
public class TAddLink {

	public static void main(String args[]) throws Exception {
		if (args.length < 8) {
			System.out.print(
				"Adds a link between two items.  CMv8 only. \n"
					+ "Syntax: \n"
					+ "  java TAddLink <dstype> <server> <userid> <password> <linktype> <source_entity> <source_condition> / <dest_entity> <dest_condition> \n"
					+ "Examples: \n"
					+ "  java TAddLink ICM icmnlsdb icmadmin password Contains Book @Title like \"Source Item\" / Book @Title like \"Dest Item\" \n"
					+ "");
			System.exit(0);
		}

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String linkType = args[4];
        String sourceEntity = args[5];
        String sourceCondition = "";
        int i;
        for (i = 6; i < args.length && !args[i].equals("/"); i++) {
            sourceCondition += " " + args[i];
        }
		String destEntity = args[++i];
		String destCondition = "";
		for (++i; i < args.length; i++) {
			destCondition += " " + args[i];
		}

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// Search for the source item
		TSearch.search(queryService, dstype, sourceEntity, sourceCondition, false);
		searchResults.newResults(queryService.getResults());
		if (searchResults.getCount() == 0) {
			throw new Exception("No items matched source entity/condition: "+sourceEntity+"/"+sourceCondition);
		}
		if (searchResults.getCount() > 1) {
			throw new Exception("More than one item matched source entity/condition: "+sourceEntity+"/"+sourceCondition);
		}
		CMBItem sourceItem = searchResults.getItem(0);
		System.out.println("Source item found");
		
		// Search for the destination item
		TSearch.search(queryService, dstype, destEntity, destCondition, false);
		searchResults.newResults(queryService.getResults());
		if (searchResults.getCount() == 0) {
			throw new Exception("No items matched destination entity/condition: "+destEntity+"/"+destCondition);
		}
		System.out.println("Destination item(s) found");
		for (i = 0; i < searchResults.getCount(); i++) {
			CMBItem destItem = searchResults.getItem(i);
		
			// Add a link from source to destination
			System.out.println("Adding link from source item to destination item");
			dataManagement.setDataObject(destItem);
			dataManagement.checkOut();
			dataManagement.setDataObject(sourceItem);
			dataManagement.checkOut();
			dataManagement.addLink(destItem,linkType);
			dataManagement.setDataObject(destItem);
			dataManagement.checkIn();
			dataManagement.setDataObject(sourceItem);
			dataManagement.checkIn();
		}

		System.out.println("Disconnecting from server");
		connection.disconnect();
	}

}
