/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import java.io.*;
import com.ibm.mm.beans.*;

/**
 * Write the contents of documents to files.  A search is first performed to find
 * the items to write.
 */
public class TExport {

    public static void main(String args[]) throws Exception {
        if (args.length < 6) {
            System.out.println(
                "Writes contents of documents matching a search pattern to files. \n"
                    + "Syntax: \n"
                    + "  java TExport <dstype> <server> <userid> <password> <entity> <prefix> <condition> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <prefix> a prefix to use for the names of files written. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
                    + "Examples: \n"
                    + "  java TExport DL libsrvrn frnadmin password NOINDEX noindexdoc Userid LIKE '%' \n"
                    + "  java TExport DB2 sample db2admin password EMP_PHOTO photo PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TExport OD milpitas edms1 edms1 Balance stmt CustName like '%BLONDIE%' \n"
                    + "  java TExport ICM icmnlsdb icmadmin password Journal01 journal @JTitle01 LIKE \"%\"");
            System.exit(0);
        }

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String prefix = args[5];
        String condition = "";
        for (int i = 6; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect to server
        TConnect.connect(connection, dstype, server, userid, password);

        // Search to find items to export
        TSearch.search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");

        // Export the items found
        if (searchResults.getCount() == 0) {
            System.out.println("No items deleted");
        } else {
            for (int i = 0; i < searchResults.getCount(); i++) {
                CMBItem item = searchResults.getItem(i);
                export(dataManagement, item, prefix + i);
            }
        }

        // Disconnect from server
        TConnect.disconnect(connection);

    }

    /**
     * Export the content of the item to a file.  This only handles one content part -- 
     * multi-part documents have only the first part written.
     * @param dataManagement an instance of CMBDataManagement.
     * @param item an instance of CMBItem for the document to be written.
     * @param prefix filename prefix for the file to write.  The extension is appended,
     *        and depends on the document type.
     */
    public static void export(CMBDataManagement dataManagement, CMBItem item, String prefix)
        throws CMBException, IOException, Exception {
        
        // Determine if the document has any content parts
        dataManagement.setDataObject(item);
        System.out.println("  Item = " + item.getName());
        if (dataManagement.getContentCount() == 0) {
            System.out.println("    Item contains no content.  Nothing exported.");
            return;
        }
        	
    	// Write the first content part to a file.
        String mimetype = dataManagement.getItemMimeType();
        String filename = prefix + "." + getExtension(mimetype);
        System.out.println("    Mimetype = " + mimetype);
        System.out.println("    Content size = " + dataManagement.getContent(0).getSize());
        System.out.println("    Exported as " + filename);
        FileOutputStream fileoutstream = new FileOutputStream(filename);
        fileoutstream.write(dataManagement.getContent(0).getData());
        fileoutstream.close();
    }

    /**
     * Returns file extensions for typical mimetypes.
     */
    public static String getExtension(String mimetype) {
        if (mimetype.equals("image/gif")) {
            return "gif";
        } else if (mimetype.equals("image/jpeg")) {
            return "jpeg";
        } else if (mimetype.equals("text/html")) {
            return "html";
        } else if (mimetype.equals("text/plain")) {
            return "txt";
        } else if (mimetype.equals("application/pdf")) {
            return "pdf";
        } else if (mimetype.equals("image/tiff")) {
            return "tif";
        } else if (mimetype.equals("application/afp")) {
            return "afp";
        } else if (mimetype.equals("application/vnd.ibm.modcap")) {
            return "mda";
        } else {
            return "bin";
        }
    }

}