/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import  com.ibm.mm.beans.*;

/**
 * Display the links to and from an item or items.  Links to an item are called
 * inbound links.  Links out of the item are known as outbound links.
 * This sample works with CMv8 only.
 */
public class TListLinks {
	
	public static void main(String args[]) throws Exception
	{
        if (args.length < 5) {
            System.out.println("List links to and from item(s).");
            System.out.println("Syntax:");
            System.out.println("  java TListLinks <dstype> <server> <userid> <password> <linktype> <entity> <condition>");
            System.out.println("Examples:");
            System.out.println("  java TListLinks ICM icmnlsdb icmadmin password Book @Title like \\\"%\\\"");
            return;
        }

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String linkType = args[4];
        String entity = args[5];
        String condition = "";
        for (int i = 6; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		TSearch.search(queryService, dstype, entity, condition, false);

        searchResults.newResults(queryService.getResults());
        System.out.println("There are "+searchResults.getCount()+" hits:");
        for (int i = 0; i < searchResults.getCount(); i++) {
            CMBItem item = searchResults.getItem(i);
            String names[] = item.getAttrNames();
            String values[] = item.getAttrValues();
            System.out.print("  ");
            for (int j = 0; j < names.length; j++) {
                System.out.print(names[j]+"='"+values[j]+"' ");
            }
            System.out.println("");
            
            // Retrieve the links.  These are not retrieved by default
            dataManagement.setDataObject(item);
            item.setRetrieved(false);
            dataManagement.setRetrieveLinks(true);
            dataManagement.retrieveItem();
            dataManagement.setRetrieveLinks(false);

            
            // Display outbound links
            CMBItem[] items = dataManagement.getLinks(linkType);
            System.out.println("    Outbound links:");
            for (int j = 0; j < items.length; j++) {
                item = items[j];
                System.out.print("      ");
                names = item.getAttrNames();
                values = item.getAttrValues();
                for (int k = 0; k < names.length; k++) {
                    System.out.print(names[k]+"='"+values[k]+"' ");
                }
                System.out.println("");
            }
            
            // Display inbound links
           	items = dataManagement.getInboundLinks(linkType);
            System.out.println("    Inbound links:");
            for (int j = 0; j < items.length; j++) {
                item = items[j];
                System.out.print("      ");
                names = item.getAttrNames();
                values = item.getAttrValues();
                for (int k = 0; k < names.length; k++) {
                    System.out.print(names[k]+"='"+values[k]+"' ");
                }
                System.out.println("");
            }
        }

        System.out.println("Disconnecting from server");
        connection.disconnect();
    }

}
