/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Reindex items using EIP JavaBeans.
 */
public class TReindex {
	
	public static void main(String args[]) throws Exception {
        if (args.length < 5) {
        	System.out.println(
                "Reindex (change item type) of an item. \n"
                + "Syntax: \n"
                + "  java TReindex <dstype> <server> <userid> <password> <entity> <condition> / <newentity> <newattributes>");
            System.exit(0);
        }

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        int i;
        for (i = 5; i < args.length && !args[i].equals("/"); i++) {
            condition += " " + args[i];
        }
		String newEntity = args[++i];
		String newAttributes = "";
		for (;i < args.length; i++) {
			newAttributes = newAttributes + " " + args[i];
		}

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// find items
		TSearch.search(queryService, dstype, entity, condition, false);
		searchResults.newResults(queryService.getResults());
		if (searchResults.getCount() > 3) {
			System.out.println("Condition matched greater than three items.  Reindex not performed!");
			return;
		}
		System.out.println("  There are " + searchResults.getCount() + " hits.");
		for (i = 0; i < searchResults.getCount(); i++) {
			CMBItem item = searchResults.getItem(i);
			
	        // Look up the entity, to make sure it exists and to get attribute metadata
	        CMBEntity entityBean = schemaManagement.getEntity(newEntity);
	        if (entityBean == null) {
	            throw new IllegalArgumentException("Entity not found: " + newEntity);
	        }	       

			// Reset the attributes to the new attribute values
			item.removeAllAttrs();
			item.setEntityName(newEntity);
			TCreateItem.setItemAttrs(entityBean,item,newAttributes);

			// Reindex the item
			System.out.println("  Reindexing "+item.getName());
			dataManagement.setDataObject(item);
			if (dataManagement.isSupported("checkOut")) { // some servers require checkout
				dataManagement.checkOut();
			}
			dataManagement.reindexItem(newEntity);
			if (dataManagement.isSupported("checkIn")) {
				dataManagement.checkIn();
			}
		}

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

    }

}