/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//*****************************************************************************
//
// Sample class to list all subEntities in a Domino.Doc datastore
// Library (a Domino.Doc Datastore)
// -- contains Rooms
// -- -- contains Cabinets (has a profile with attributes)
// -- -- -- contains Binders (has a profile with attributes)
// -- -- -- -- contains Documents (has a profile with attributes)
//
//*****************************************************************************

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;



 public class TClientListSubEntitiesDD implements DKConstantDD
 {

   public static String Me = new String("TClientListSubEntitiesDD");


   public void TClientListSubEntitiesDD(){}

   public static void main(String argv[])
   {
     System.out.println(" ");
     System.out.println(Me + ": Starting");
     String str;

     TClientListSubEntitiesDD thisTest = new TClientListSubEntitiesDD();

     // Input parameters
     String libURL = new String("");   // library server
     String userid = new String("");
     String pw     = new String("");

     // Indicate this is a Console Application on Windows NT/95 for error
     // message display in dos console code page. If this was a Windows
     // GUI (Graphic User Interface) Application this would not be necessary
     // because DK_CM_SS_WINDOWS is the default.
     DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

     //--------------------------------------------------------------
     // Checking for input parameters
     //--------------------------------------------------------------

     if (argv.length < 3)
     {
       System.out.println(Me + " Usage: " );
       System.out.println("  java " + Me +
       " <libURL> <userid> <pw>" );
       return;
     }
     else
     {
       libURL = argv[0];
       userid = argv[1];
       pw = argv[2];
       System.out.println(Me + ": Parameter1=" + argv[0] );
       System.out.println(Me + ": Parameter2=" + argv[1] );
       System.out.println(Me + ": Parameter3=" + argv[2] );
     }

     DKDatastoreDD dsDD = null;
     try
     {
       dsDD = new DKDatastoreDD();

       // get Connection
       System.out.println("\n" + Me + ": Connecting to datastore");
       dsDD.connect(libURL,userid,pw,"");

       // get a list of rooms
       dkCollection rooms = dsDD.listEntities();
       System.out.println("\n" + Me + ": Print rooms and their subEntities.");
       dkIterator itRooms = rooms.createIterator();
       itRooms.reset();
       while( itRooms.more() )
       { // print room titles and check for cabinets
         DKRoomDefDD aRoom = (DKRoomDefDD)itRooms.next();
         // check the room for cabinets
         System.out.println("Room: " + aRoom.getName() );
         if( aRoom.hasSubEntities() )
         {
           dkCollection cabinets = aRoom.listSubEntities();
           dkIterator itCabinets = cabinets.createIterator();
           while( itCabinets.more() )
           { // print cabinet titles and check for binders
             DKCabinetDefDD aCab = (DKCabinetDefDD)itCabinets.next();
             System.out.println("  Cabinet: " + aCab.getName() );

             if( aCab.hasSubEntities() )
             {
               dkCollection binders = aCab.listSubEntities();
               dkIterator itBinders = binders.createIterator();

               while( itBinders.more() )
               {
                 DKBinderDefDD aBinder = (DKBinderDefDD)itBinders.next();
                 System.out.println("    Binder: " + aBinder.getName() );

                 if( aBinder.hasSubEntities() )
                 {
                   dkCollection documents = aBinder.listSubEntities();
                   dkIterator itDocs = documents.createIterator();
                   itDocs.reset();

                   while( itDocs.more() )
                   { // print document titles
                     DKDocumentDefDD aDocument = (DKDocumentDefDD)itDocs.next();
                     System.out.println("      Document: " + aDocument.getName() );

                     // now, for extra credit, print the attributes of the document
                     DKAttrProfileDefDD profile = aDocument.getProfile();
                     dkCollection fields = profile.getFields();

                     if((fields != null) &&( fields.cardinality() > 0 ))
                     {
                       dkIterator itFields = fields.createIterator();
                       while( itFields.more() )
                       { // print name, keywords
                         DKAttrDefDD aField = (DKAttrDefDD)itFields.next();
                         System.out.println("        " + aField.getName() + ": " + aField.getValue() );
                         dkCollection keywords = ((DKAttrFieldDefDD)aField).getKeywords();
                         if( keywords != null )
                         {
                           if( keywords.cardinality() > 0 )
                           { // print name
                             dkIterator itKeywords = keywords.createIterator();
                             while( itKeywords.more() )
                             {
                               DKAttrDefDD aKeyword = (DKAttrDefDD)itKeywords.next();
                               System.out.println("          " + aKeyword.getName() + ": " + aKeyword.getValue() );
                             }
                           } // end keyword processing
                         }
                       } // end field processing
                     }
                   } // end document processing
                 }
               }  // end binder processing
             }
           } // end cabinet processing
         }
       } // end room processing

       // disconect
       System.out.println("\n");
       System.out.println(Me + ": Disconnecting from datastore");
       dsDD.disconnect();
       dsDD.destroy();
     }
     catch (DKException exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": DKException message " + exc.getMessage());
       exc.printStackTrace();
     }
     catch(Exception exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": Exception message " + exc.getMessage());
       exc.printStackTrace();
     }

     System.out.println(Me + ": Finished");
   }

  public void printList( dkCollection coll)
  throws DKException
  {
    if( coll == null )
    {
      System.out.println(Me + ":printList Input Collection object is null.");
      return;
    }
    int nbrElements = coll.cardinality();
    Class  collClass = coll.getClass();
    String collClassName = collClass.getName();
    System.out.println(Me + ":printList for a " + collClassName + ", nbrElements=" + nbrElements);
    int i = 1;
    if( nbrElements > 0 )
    {
      dkIterator it = coll.createIterator();
      while( it.more() )
      {
        Object anObject = it.next();
        Class objClass = anObject.getClass();
        String objClassName = objClass.getName();
        System.out.println("..toString[" + i++ + "]");
        String objString = anObject.toString();
        System.out.println(objString);
      }
    }
    return;
  }
 }


