/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TCQExecuteDL.java
 * Description    : Connect to DL Datastore.
 *                  Create DKCQExpr that contains the query.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TCQExecuteDL class
public class TCQExecuteDL implements DKConstant
{
  // Main method
  public static void main(String argv[])
  {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3) {
          System.out.println("Invoke Syntax: " );
          System.out.println("  java TCQExecuteDL <libSrv> <userid> <pw>" );
          return;
      }
      
      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];
       
      DKDatastoreDL dsDL = null;
      try {
           dsDL = new DKDatastoreDL();
           dkResultSetCursor pCur = null;
           DKNVPair soptions[] = new DKNVPair[1];
           DKNVPair qoptions[] = new DKNVPair[1];
           DKNVPair tnv = null;
           DKCQExpr cqe = new DKCQExpr();
           DKQExpr qe = new DKQExpr();
           DKQTerm qterm = new DKQTerm("DLSEARCH_DocType",DKOpCode.NEQ,"NULL");
           DKQSubExpr sqe = new DKQSubExpr();
           String strTmp = "";
           
           System.out.println("connecting to datastore"); 
           dsDL.connect(libSrv,userid,pw,"");
           System.out.println("datastore connected libSrv " + libSrv + " userid " + userid ); 
           
           cqe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
           qe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
           sqe.setTranslation(false);
           sqe.setEntityName("DLSAMPLE");
           sqe.setQueryTerm(qterm);
           strTmp = "5";
           tnv = new DKNVPair("MAX_RESULTS",strTmp);
           soptions[0] = tnv;
           sqe.setOptionList(soptions);
           qe.addSubQuery(sqe);
           strTmp = "YES";
           tnv = new DKNVPair("CONTENT",strTmp);
           qoptions[0] = tnv;
           qe.setOptionList(qoptions);
           cqe.setQueryExpr(qe);
           DKDDO p = null;
           DKDDO pDDO = null;
           dkXDO pXDO = null;
           int i = 0;
           int numDataItems = 0;
           short k = 0;
           short j = 0;
           String strDataName;
           dkCollection pCol = null;
           dkIterator pIter = null;
           Object a = null;
           dkDataObjectBase pDO = null;
           int cnt = 0;
           System.out.println("executing query");
           pCur = dsDL.execute(cqe);
           System.out.println("cardinality " + pCur.cardinality());
           System.out.println("datastore executed query");
           System.out.println("process query results");
           while (pCur.isValid()){
                p = pCur.fetchNext();
                if (p != null){
                   cnt++;
                   i = pCur.getPosition();
                   System.out.println("=====> Item " + i + " <=====");
                   numDataItems = p.dataCount();
                   DKPid pid = p.getPidObject();
                   System.out.println("pid string " + pid.pidString());
                   k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);
                   if (k > 0){
                       Short sVal = (Short)p.getProperty(k);
                       j = sVal.shortValue();
                       switch (j){
                           case DK_CM_DOCUMENT :
	      	                    System.out.println("Item is document ");
                                break;
	                   case DK_CM_FOLDER :
	                            System.out.println("Item is folder ");
                                break;
	                   }
                   }
                   System.out.println("Number of Data Items " + numDataItems);
                   for (j = 1; j <= numDataItems; j++){
                       a = p.getData(j);
                       strDataName = p.getDataName(j);
                       System.out.println(j + ": Attribute Name " + strDataName );
                       if (a instanceof dkCollection) {
	                       System.out.println("   Attribute Value is a collection");    
                           pCol = (dkCollection)a;
                           pIter = pCol.createIterator();
                           i = 0;
                           while (pIter.more() == true){
                                i++;
                                a = pIter.next();
                                pDO = (dkDataObjectBase)a;
                                if (pDO.protocol() == DK_PDDO){
                                    System.out.println("    DKDDO object " + i + " in collection");
                                    pDDO = (DKDDO)pDO;
	                            }
                                else if (pDO.protocol() == DK_XDO){
	                                System.out.println("    dkXDO object " + i + " in collection");
                                    pXDO = (dkXDO)pDO;
                                    DKPidXDO pid2 = (DKPidXDO)pXDO.getPidObject();
                                    System.out.println("XDO pid string " + pid2.pidString());
	                            }
	                       }
                       }
                       else if (a != null) 
	                       System.out.println("   Attribute Value " + a.toString());    
                       else 
                           System.out.println("   Attribute Value is null");    
                   }       
                }
           }
           pCur.destroy(); // Finished with the cursor
           System.out.println("query results processed");
           System.out.println("Number of items " + cnt );
           dsDL.disconnect();
           System.out.println("datastore disconnected"); 
           dsDL.destroy();
      }
      catch (DKException exc){
     	   try {
     	       dsDL.destroy();
     	   }
     	   catch (Exception e)
     	   {
     	    e.printStackTrace();
     	   }           
           System.out.println("Exception name " + exc.name());
           System.out.println("Exception message " + exc.getMessage());
           System.out.println("Exception error code " + exc.errorCode());
           System.out.println("Exception error state " + exc.errorState());
           exc.printStackTrace();
      }
      catch (Exception exc){
     	   try {
     	       dsDL.destroy();
     	   }
     	   catch (Exception e)
     	   {
     	    e.printStackTrace();
     	   }           
           System.out.println("Exception message " + exc.getMessage());
           exc.printStackTrace();
      }
   }
}

