/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TIndexingTS.java
 * Description    : Start the indexing process for a text search index
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TIndexingTS class
public class TIndexingTS implements DKConstantDL {

  // Main method
  public static void main(String args[]){
    
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (args.length < 2) {
          System.out.println("Invoke Syntax: " );
          System.out.println("  java TIndexingTS <srchSrv> <searchIndex> " );
          return;
      }
      
      String srchSrv   = args[0];
      String indexName = args[1];
       
      DKDatastoreTS dsTS = null;
      try {
         DKIndexFuncStatusTS pIndexFuncStatus = null;
         dsTS = new DKDatastoreTS();
         DKDatastoreDefTS dsDef = null;
         DKDatastoreAdminTS dsAdmin = null;
         
         System.out.println("connecting to datastore"); 
         dsTS.connect(srchSrv,"","","");
         System.out.println("datastore connected " + srchSrv); 
         dsDef = (DKDatastoreDefTS)dsTS.datastoreDef();
         dsAdmin = (DKDatastoreAdminTS)dsDef.datastoreAdmin();
    
         System.out.println("start updating index " + indexName); 
         dsAdmin.startUpdateIndex(indexName);
         System.out.println("update index completed"); 
         System.out.println("**** get index status ****"); 
         pIndexFuncStatus = dsAdmin.getIndexFunctionStatus(indexName, DK_TS_INDEX_FUNCID_INDEX_DOCUMENTS);
         System.out.println("****** index status ******"); 
         System.out.println("*isCompleted " + pIndexFuncStatus.isCompleted()); 
         System.out.println("*getEnabledId " + pIndexFuncStatus.getEnabledId()); 
         System.out.println("*getReasonCode " + pIndexFuncStatus.getReasonCode()); 
         System.out.println("*getFuncStopped " + pIndexFuncStatus.getFunctionStopped()); 
         System.out.println("*getStartedLast " + pIndexFuncStatus.getStartedLast()); 
         System.out.println("*getEndedLast " + pIndexFuncStatus.getEndedLast()); 
         System.out.println("*getStartedExecution " + pIndexFuncStatus.getStartedExecution()); 
         System.out.println("*getScheduledDocs " + pIndexFuncStatus.getScheduledDocs()); 
         System.out.println("*getDocsInPrimaryIndex " + pIndexFuncStatus.getDocsInPrimaryIndex()); 
         System.out.println("*getDocsInSecondaryIndex " + pIndexFuncStatus.getDocsInSecondaryIndex()); 
         System.out.println("*getDocMessages " + pIndexFuncStatus.getDocMessages()); 
         if (pIndexFuncStatus.isCompleted())
             System.out.println("***********indexing completed****************"); 
         
         if (pIndexFuncStatus.getReasonCode() != 0)
             dsAdmin.setIndexFunctionStatus(indexName, DK_TS_INDEX_FUNCID_INDEX_DOCUMENTS,DK_TS_INDEX_ACTID_RESET);
    
         dsTS.disconnect();
         System.out.println("datastore disconnected"); 
         dsTS.destroy();
      }
      catch (DKException exc){
     	 try {
     	     dsTS.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }         
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
     	 try {
     	     dsTS.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }         
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
  }
}

