/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TADLsmSample.java
 * Description    : test add, delete, list schema-mappings
 *                : This program will create a fed entity 'FedSample' which mapped to
 *                  DLSAMPLE on Talisman2 and Mufasa.
 *
 */

import java.sql.*;
import java.lang.*;
import java.util.*;
import com.ibm.mm.sdk.common.dkSchemaMapping;
import com.ibm.mm.sdk.common.dkAttrMapping;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

class TADLsmSample implements DKConstantFed
{
    public static void main(String argv[]) 
    { 
        //-------------------------------------------------------------- 
        // Checking for input parameters
        //-------------------------------------------------------------- 
        if (argv.length < 4) {
           System.out.println("Usage Syntax: ");
           System.out.println("\t java TADLsmSample <feddb> <userid> <pw> <add/del/list>" );
           return;
        }
        
        String libSrv = argv[0];
        String userid = argv[1];
        String pw     = argv[2];
        String cmd    = argv[3];

        try {
            // create federated datastore
            System.out.println(" *** connecting to datastore : " + libSrv);
            dkDatastore ds = new DKDatastoreFed();
            ds.connect(libSrv, userid, pw, "");
            System.out.println(" *** datastore connected *** "); 

            // get the Def
            dkDatastoreDef  defFed = ds.datastoreDef();

            // create a federated entity : FedSample
            DKEntityDefFed entityObj = new DKEntityDefFed(ds);        
            entityObj.setName("FedSample");
            entityObj.setDescription("FedSample to map to DLSAMPLE at mufasa & talisman2");
    
            // create a federated attributes :
            dkAttrDef fAuthor = new DKAttrDefFed();
            fAuthor.setEntityName("FedSample");
            fAuthor.setName("FedAuthor");
            fAuthor.setType(DK_CM_VSTRING);
            fAuthor.setId((short)95);
            fAuthor.setSize(50);
            fAuthor.setScale(1);
            fAuthor.setPrecision(50);
            fAuthor.setNullable(true);
            fAuthor.setStringType(DK_ATTR_ALPHA);
    
            dkAttrDef fTitle = new DKAttrDefFed();
            fTitle.setEntityName("FedSample");
            fTitle.setName("FedTitle");
            fTitle.setType(DK_CM_VSTRING);
            fTitle.setId((short)99);
            fTitle.setSize(100);
            fTitle.setScale(1);
            fTitle.setPrecision(100);
            fTitle.setNullable(true);
            fTitle.setStringType(DK_ATTR_ALPHA);
    
            dkAttrDef fSubject = new DKAttrDefFed();
            fSubject.setEntityName("FedSample");
            fSubject.setName("FedSubject");
            fSubject.setType(DK_CM_VSTRING);
            fSubject.setId((short)100);
            fSubject.setSize(100);
            fSubject.setScale(1);
            fSubject.setPrecision(100);
            fSubject.setNullable(true);
            fSubject.setStringType(DK_ATTR_ALPHA);

            // associate data sources to this entity:
            // get the Fed DS Def and Admin
            DKDatastoreAdminFed adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();

            // list back-end servers
            System.out.println(" *** list all back-end servers : ");
            String servers[] = adm.listServerNames();
            int len = servers.length;
            for (int i = 0; i < len; i++) {
                System.out.println("server [" + i + "] = " + servers[i] + " Strlen = " + servers[i].length());
            }  
              
            // retrieve the definition of the first server
            DKServerMapDefFed serverdat;
            String serverName = null;
            for (int i = 0; i < len; i ++) {
                serverdat = adm.getServer(servers[i]);
                serverName = serverdat.getName().trim();
                System.out.println(" DKServerMapDefFed = " + serverName );
                System.out.println("       name : " + serverName);
                System.out.println("       type : " + serverdat.getServerType());
                System.out.println(" Connect Str: " + serverdat.getConnectString().trim());
                System.out.println("       desc : " + serverdat.getDescription());
                System.out.println("    refTime : " + serverdat.getRefreshTime());
            }     

            // Now, interpret the command: add, del, or list
            if (cmd.equalsIgnoreCase("add")) {
                entityObj.add();        // adding an entity to the federated database
                entityObj.add(fAuthor);
                entityObj.add(fTitle);
                entityObj.add(fSubject);
                System.out.println(" *** federated entity and attributes created ");
            }

            // list federated entity:
            System.out.println(" *** federated entities :");
            DKSequentialCollection pCol = (DKSequentialCollection) ds.listEntities();
            DKSequentialCollection pCol2;
            dkIterator pIter = pCol.createIterator();
            dkIterator pIter2 = null;
            dkEntityDef enDef = null;
            dkAttrDef attrDef = null;
            String entName;
            int i = 0, j = 0;
            while (pIter.more()) {
                i++;
                enDef = (dkEntityDef) pIter.next();
                entName = enDef.getName();
                System.out.println(" Entity Def name [" + i + "] - " + entName);
                System.out.println("  list attributes for " + entName + " entity");
                pCol2 = (DKSequentialCollection) ds.listEntityAttrs(entName);
                pIter2 = pCol2.createIterator();
                j = 0;
                while (pIter2.more()) {
                   j++;
                   attrDef = (dkAttrDef) pIter2.next();
                   System.out.println("  Attribute name [" + j + "] - " + attrDef.getName());
                   System.out.println("    datastoreType : " + attrDef.datastoreType());
                   System.out.println("    attributeOf   : " + attrDef.getEntityName());
                   System.out.println("    type          : " + attrDef.getType());
                   System.out.println("    size          : " + attrDef.getSize());
                   System.out.println("    id            : " + attrDef.getId());
                   System.out.println("    nullable      : " + attrDef.isNullable());
                   System.out.println("    precision     : " + attrDef.getPrecision());
                   System.out.println("    scale         : " + attrDef.getScale());
                   System.out.println("    stringType    : " + attrDef.getStringType());
                }  
                System.out.println("  " + j + " attr listed for " + entName + " entity name");
            }
            System.out.println(i + " entity def listed");

            // create native attributes in-memory : DL index_class DLSAMPLE at mufasa
            dkAttrDef nmAuthor = new DKAttrDefDL();
                      nmAuthor.setEntityName("DLSAMPLE");
                      nmAuthor.setName("DLSEARCH_Author");
                      nmAuthor.setType(DK_CM_VSTRING);
                      nmAuthor.setId((short)95);
                      nmAuthor.setSize(50);
                      nmAuthor.setScale(1);
                      nmAuthor.setPrecision(50);
                      nmAuthor.setNullable(true);
                      nmAuthor.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmTitle = new DKAttrDefDL();
                      nmTitle.setEntityName("DLSAMPLE");
                      nmTitle.setName("DLSEARCH_Title");
                      nmTitle.setType(DK_CM_VSTRING);
                      nmTitle.setId((short)99);
                      nmTitle.setSize(100);
                      nmTitle.setScale(1);
                      nmTitle.setPrecision(100);
                      nmTitle.setNullable(true);
                      nmTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmSubject = new DKAttrDefDL();
                      nmSubject.setEntityName("DLSAMPLE");
                      nmSubject.setName("DLSEARCH_Subject");
                      nmSubject.setType(DK_CM_VSTRING);
                      nmSubject.setId((short)100);
                      nmSubject.setSize(100);
                      nmSubject.setScale(1);
                      nmSubject.setPrecision(100);
                      nmSubject.setNullable(true);
                      nmSubject.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSMUFASA, DLSAMPLE ");

            // create native attributes in-memory : DL index_class DLSAMPLE at talisman2
            dkAttrDef ncAuthor = new DKAttrDefDL();
                      ncAuthor.setEntityName("DLSAMPLE");
                      ncAuthor.setName("DLSEARCH_Author");
                      ncAuthor.setType(DK_CM_VSTRING);
                      ncAuthor.setId((short)95);
                      ncAuthor.setSize(50);
                      ncAuthor.setScale(1);
                      ncAuthor.setPrecision(50);
                      ncAuthor.setNullable(true);
                      ncAuthor.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncTitle = new DKAttrDefDL();
                      ncTitle.setEntityName("DLSAMPLE");
                      ncTitle.setName("DLSEARCH_Title");
                      ncTitle.setType(DK_CM_VSTRING);
                      ncTitle.setId((short)99);
                      ncTitle.setSize(100);
                      ncTitle.setScale(1);
                      ncTitle.setPrecision(100);
                      ncTitle.setNullable(true);
                      ncTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncSubject = new DKAttrDefDL();
                      ncSubject.setEntityName("DLSAMPLE");
                      ncSubject.setName("DLSEARCH_Subject");
                      ncSubject.setType(DK_CM_VSTRING);
                      ncSubject.setId((short)100);
                      ncSubject.setSize(100);
                      ncSubject.setScale(1);
                      ncSubject.setPrecision(100);
                      ncSubject.setNullable(true);
                      ncSubject.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSCHIUN, DLSAMPLE ");
     
            
            //  create schema mapping : entity and attr mappings 
            //   from FedSample to DLSAMPLE in LSMUFASA
            dkAttrMapping mapAuthor = new DKAttrMappingFed();
                          mapAuthor.addMappedAttr(fAuthor);
                          mapAuthor.addMapToAttr(nmAuthor);
     
            dkAttrMapping mapTitle = new DKAttrMappingFed();
                          mapTitle.addMappedAttr(fTitle);
                          mapTitle.addMapToAttr(nmTitle);
     
            dkAttrMapping mapSubject = new DKAttrMappingFed();
                          mapSubject.addMappedAttr(fSubject);
                          mapSubject.addMapToAttr(nmSubject);
     
            System.out.println(" ** attribute mapping to LSMUFASA defined ");
     
            dkSchemaMapping msm = new DKSchemaMappingFed(ds);
                            msm.setServerName("LSMUFASA");
                            msm.setServerType(DK_DSTYPE_DL);
                            msm.setEntityName("DLSAMPLE");
                            msm.setMappedEntityName("FedSample");
     
                            // add attribute mapping to this schema
                            msm.addAttrMapping(mapAuthor);
                            msm.addAttrMapping(mapTitle);
                            msm.addAttrMapping(mapSubject);
            
            try {
                if (cmd.equalsIgnoreCase("add")) {
                    msm.add();
                    System.out.println(" ** schema mapping for LSMUFASA added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    msm.del();
                    System.out.println(" ** schema mapping for LSMUFASA deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }
                 
            //  create schema mapping : entity and attr mappings 
            //   from FedSample to DLSAMPLE in LSCHIUN
            dkAttrMapping cmapAuthor = new DKAttrMappingFed();
                          cmapAuthor.addMappedAttr(fAuthor);
                          cmapAuthor.addMapToAttr(ncAuthor);
     
            dkAttrMapping cmapTitle = new DKAttrMappingFed();
                          cmapTitle.addMappedAttr(fTitle);
                          cmapTitle.addMapToAttr(ncTitle);
     
            dkAttrMapping cmapSubject = new DKAttrMappingFed();
                          cmapSubject.addMappedAttr(fSubject);
                          cmapSubject.addMapToAttr(ncSubject);
     
            System.out.println(" ** attribute mapping to LSCHIUN defined ");
     
            dkSchemaMapping csm = new DKSchemaMappingFed(ds);
                            csm.setServerName("LSCHIUN");
                            csm.setServerType(DK_DSTYPE_DL);
                            csm.setEntityName("DLSAMPLE");
                            csm.setMappedEntityName("FedSample");
     
            // add attribute mapping to this schema
            csm.addAttrMapping(cmapAuthor);
            csm.addAttrMapping(cmapTitle);
            csm.addAttrMapping(cmapSubject);

            try {
                if (cmd.equalsIgnoreCase("add")) {
                    csm.add();
                    System.out.println(" ** schema mapping for LSCHIUN added ");
                 }
                 else if (cmd.equalsIgnoreCase("del")) {
                    csm.del();
                    System.out.println(" ** schema mapping for LSCHIUN deleted ");
                 }
            }
            catch(Exception e) { e.printStackTrace(); }

            
            if (cmd.equalsIgnoreCase("del")) {
                entityObj.del(fSubject);
                entityObj.del(fTitle);
                entityObj.del(fAuthor);
                entityObj.del();        // deleting an entity from the federated database
                System.out.println(" *** federated entity and attributes deleted ");
            } 

            // list schema mappings in this entity:
            System.out.println(" list schema mappings in this entity : " + entityObj.getName());
            String[] smnames = entityObj.listSchemaMappingNames();
            if (smnames != null) {
                len = smnames.length;
                for (i = 0; i < len; i++) {
                    System.out.println(" SM name : " + smnames[i]);
                }
            } 
            else 
                System.out.println(" no sm defined in this entity ");
                

            // retrieve all schema mapping objects for this entity
            dkCollection coll = entityObj.listSchemaMappings();
            dkIterator iter = null;
            if (coll != null) {
                iter = coll.createIterator();
                System.out.println(" ");
                System.out.println("-- list AttrMapping -- ");
                dkAttrMapping _am;
                dkAttrDef     _ad;
                dkSchemaMapping _sm;
                while (iter.more()) {
                    _sm = (dkSchemaMapping) iter.next();
                    System.out.println("\n *** Schema Mapping Name : " + _sm.getName());
                    dkCollection amCol = _sm.listAttrMappings();
                    dkIterator amIter = amCol.createIterator();
                    while (amIter.more()) {
                        _am = (dkAttrMapping) amIter.next();
                        System.out.println(" ");
                        System.out.println("  ++ AttrMapping Name : " +_am.getName());
                        System.out.println("    -- To Function : " + _am.getToNative());
                        System.out.println("    -- From Function : " + _am.getFromNative());
                        System.out.println(" -- list Mapped Attr : ");
                        int num = _am.numberOfMappedAttr();
                        System.out.println("\t Mapped attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMappedAttr((short)(i+1));
                            System.out.println("  ++ Mapped Attr Name   : " +_ad.getName());
                            System.out.println("    -- Mapped Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size             : " +_ad.getSize());
                            System.out.println("    -- Precision        : " +_ad.getPrecision());
                            System.out.println("    -- Scale            : " +_ad.getScale());
                            System.out.println("    -- Max              : " +_ad.getMax());
                            System.out.println("    -- Min              : " +_ad.getMin());
                            System.out.println("    -- Nullable         : " +_ad.isNullable());
                        }
                        System.out.println(" -- list MapTo Attr -- ");
                        num = _am.numberOfMapToAttr();
                        System.out.println("\t MapTo attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMapToAttr((short)(i+1));
                            System.out.println("  ++ MapTo Attr Name   : " +_ad.getName());
                            System.out.println("    -- MapTo Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size            : " +_ad.getSize());
                            System.out.println("    -- Precision       : " +_ad.getPrecision());
                            System.out.println("    -- Scale           : " +_ad.getScale());
                            System.out.println("    -- Max             : " +_ad.getMax());
                            System.out.println("    -- Min             : " +_ad.getMin());
                            System.out.println("    -- Searchable      : " +_ad.isQueryable());
                        } 
                    }
                }
                
            } else
                System.out.println("Schema mapping is null");
                
        ds.disconnect();
        System.out.println(" ** disconnects... ");
    } 
    catch (java.lang.Exception ex) 
    { 
        ex.printStackTrace(); 
    } 
} 
}
