/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/* -------------------------------------------------------------------------
 * Class Name  = TdkDatastore.java
 * Description = FVT class for testing Queries.
 *                  - calls with Result set cursor, with callback, with 
 *                    DKResults collection
 *                  - Query types handled String, Object
 *                  - reads input from a file (see end of this program for
 *                    sample input file)
 * 
 *               This program uses TLogUtils.java, and TdkCallback.java
 *
 * Author      = Chandra S Potula
 * Create Date = 06/25/2000
 * 
 * Change History
 * -------------------------------------------------------------------------
 */

import com.ibm.mm.sdk.common.*;

import java.io.*;
import java.util.*;

public class TdkDatastore extends TLogUtils {
	public  static final short SH_PKG_UNKNOWN           =  0;
	public  static final short SH_PKG_CLIENT            =  1;
	public  static final short SH_PKG_SERVER            =  2;
	public  static final short SH_PKG_CS                =  3;

	public  static final short SH_DB_UNKNOWN            =  0;
	public  static final short SH_DB_DD                 =  1;
	public  static final short SH_DB_DES                =  2;
	public  static final short SH_DB_DL                 =  3;
	public  static final short SH_DB_FED                =  4;
	public  static final short SH_DB_IP                 =  5;
	public  static final short SH_DB_JDBC               =  6;
	public  static final short SH_DB_OD                 =  7;
	public  static final short SH_DB_QBIC               =  8;
	public  static final short SH_DB_TS                 =  9;
	public  static final short SH_DB_V4                 = 10;

	public  static final long  CALLBACK_WAIT_TIME_SECS  = 10;

public TdkDatastore() {
	// Indicate this is a Console Application on Windows NT/95 for 
	// error message display in dos console code page. If this was 
	// a Windows GUI (Graphic User Interface) Application this would 
	// not be necessary because DK_CM_SS_WINDOWS is the default.
	DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
}
/**
 * 1. Boths com.ibm.mm.sdk.common.*, and com.ibm.mm.sdk.server.* must be 
 *    accessible through CLASSPATH
 * 2. Datastore (e.g. feddb) must be accessible from your workstation OR
 *    created in DB2 on your workstation
 */
public void connect(String [] args) throws DKException, Exception {
	if( args == null || args.length < 4) {
		logmsg("usage :\n  connect DBName UID PWD command");
		logmsg(    "  e.g. connect feddb edms1 edms1 add");
	}
	else {
		String lCmd = (args.length > 4) ? args[4] : "";
		getdkDatastore().connect(args[1], args[2], args[3], lCmd);
		_strUid = args[2];
		_strPwd = args[3];

		logmsg("-->Connected to datastore successfully");
	}
}
public void disconnect() throws DKException, Exception {
	getdkDatastore().disconnect();
	logmsg("                 Disconnected successfully");
}
public void execute(String aQueryString, short nQLType) throws DKException, Exception {
	logmsg("-->execute(QueryString) called");
	dkResultSetCursor lCursor = getdkDatastore().execute(aQueryString, nQLType, null);
	logmsgdkResultSetCursor(lCursor);
}
public void execute(dkQuery aQuery) throws DKException, Exception {
	logmsg("-->execute(QueryObject) called");
	dkResultSetCursor lCursor = getdkDatastore().execute(aQuery);
	logmsgdkResultSetCursor(lCursor);
}
public void execute(DKCQExpr aCQExpr) throws DKException, Exception {
	logmsg("-->execute(QueryExpression) called");
	dkResultSetCursor lCursor = getdkDatastore().execute(aCQExpr);
	logmsgdkResultSetCursor(lCursor);
}
public void executeWithCallback(String aQueryString, short nQLType) throws DKException, Exception {
	logmsg("-->executeWithCallback(QueryString) called");
	TdkCallback lTdkCallback = getTdkCallback();
	getdkDatastore().executeWithCallback(aQueryString, nQLType, null, lTdkCallback);
	waitForTdkCallback(lTdkCallback);
}
public void executeWithCallback(dkQuery aQuery) throws DKException, Exception {
	logmsg("-->executeWithCallback(QueryObject) called");
	TdkCallback lTdkCallback = getTdkCallback();
	getdkDatastore().executeWithCallback(aQuery, lTdkCallback);
	waitForTdkCallback(lTdkCallback);
}
public void executeWithCallback(DKCQExpr aCQExpr) throws DKException, Exception {
	logmsg("-->executeWithCallback(QueryExpression) called");
	TdkCallback lTdkCallback = getTdkCallback();
	getdkDatastore().executeWithCallback(aCQExpr, lTdkCallback);
	waitForTdkCallback(lTdkCallback);
}
public void evaluate(String aQueryString, short nQLType) throws DKException, Exception {
	logmsg("-->evaluate(QueryString) called");
	dkCollection lCollection = (dkCollection)(getdkDatastore().evaluate(aQueryString, nQLType, null));
	logmsgdkCollection(lCollection);
}
public void evaluate(dkQuery aQuery) throws DKException, Exception {
	logmsg("-->evaluate(QueryObject) called");
	dkCollection lCollection = (dkCollection)(getdkDatastore().evaluate(aQuery));
	logmsgdkCollection(lCollection);
}
public void evaluate(DKCQExpr aCQExpr) throws DKException, Exception {
	logmsg("-->evaluate(QueryExpression) called");
	dkCollection lCollection = (dkCollection)(getdkDatastore().evaluate(aCQExpr));
	logmsgdkCollection(lCollection);
}
public void executeTemplates() {
	executeTemplates("RSC");
	executeTemplates("CBK");
	executeTemplates("RCL");
}
public boolean setCriteriaValues(dkSearchTemplate aST) {
	boolean rc = false;

	if(aST != null) {
		try {
			dkCollection lCollSC   = aST.listCriteria();
			Short        loptr     = null;
			String       lstrValue = null;
	
			if(lCollSC != null) {
				// put some criteria values for search
				dkIterator lIterSC = lCollSC.createIterator();
				while(lIterSC.more()) {
					DKSearchCriteria lSC = (DKSearchCriteria) lIterSC.next();
	
					loptr     = (Short) (_opTable.get(lSC.getName()));
					lstrValue = (String)(_valueTable.get(lSC.getName()));
	
					if(loptr == null || lstrValue == null) {
						continue;
					}
	
					lSC.setOperator(loptr.shortValue());
					lSC.setValue(lstrValue);
					aST.update(lSC);
	
					rc = true;
				}
			}
		}
		catch (Exception exc) {
			logmsgThrowable(exc);
		}
	}

	if(!rc) {
		logmsgError("unable to select proper criteria");
	}

	return rc;
}
public void executeTemplates(String aCallType) {
	try {
		logmsg("-->executeTemplates with callType : " + aCallType);
		dkCollection lSTCollection = null;
		if(     getdkDatastore() instanceof com.ibm.mm.sdk.server.DKDatastoreFed) {
			lSTCollection = ((com.ibm.mm.sdk.server.DKDatastoreFed)getdkDatastore()).
					listSearchTemplates();
		}
		else if(getdkDatastore() instanceof com.ibm.mm.sdk.client.DKDatastoreFed) {
			lSTCollection = ((com.ibm.mm.sdk.client.DKDatastoreFed)getdkDatastore()).
					listSearchTemplates();
		}
		else if(getdkDatastore() instanceof com.ibm.mm.sdk.cs.DKDatastoreFed) {
			lSTCollection = ((com.ibm.mm.sdk.cs.DKDatastoreFed)getdkDatastore()).
					listSearchTemplates();
		}
	
		if(aCallType != null && lSTCollection != null) {
			logmsg("-->Number of SearchTemplates   : " + lSTCollection.cardinality());

			dkIterator       lIter         = lSTCollection.createIterator();
			dkSearchTemplate lST           = null;
			DKCQExpr         lCQExpr       = null;

			for(short i = 1; lIter.more(); i++) {
				logmsg("-->Processing SearchTemplate # : " + i);
				lST = (dkSearchTemplate)lIter.next();

				try {
					if(setCriteriaValues(lST)) {
						lCQExpr = lST.createQueryExpr();
						logmsg("-->SQL : " + lCQExpr.toSQL());
	
						if(     aCallType.equalsIgnoreCase("RSC")) {
							execute(lCQExpr);
						}
						else if(aCallType.equalsIgnoreCase("CBK")) {
							executeWithCallback(lCQExpr);
						}
						else if(aCallType.equalsIgnoreCase("RCL")) {
							evaluate(lCQExpr);
						}
					}
				}
				catch (Exception exc) {
					logmsgThrowable(exc);
				}
			}
		}
		else {
			logmsgError("only DKDatastoreFed contains SearchTemplates");
		}
	}
	catch (Exception exc) {
		logmsgThrowable(exc);
	}
}
private void waitForTdkCallback(TdkCallback aTdkCallback) {
	// need to wait until it finishes
	long lStartTime = System.currentTimeMillis();
	boolean fDone   = false;
	while(!fDone) {
		try {
			Thread.currentThread().sleep(32);
			if( (System.currentTimeMillis() - lStartTime) / 1000 > CALLBACK_WAIT_TIME_SECS ) {
				logmsgError("callback timedout.  Waited for : " + 
						CALLBACK_WAIT_TIME_SECS  + " seconds");
				break;
			}
			fDone = aTdkCallback.isDone();
		}
		catch(InterruptedException e) {
		}
	}
}
private TdkCallback getTdkCallback() {
	return new TdkCallback(this);
}
protected void setdkDatastore(dkDatastore adkDatastore) {
	_dkDatastore = adkDatastore;
}
public dkDatastore getdkDatastore() {
	try {
		if(_dkDatastore == null) {
			processCommand("setdkDatastore server Fed");
		}
	}
	catch(Exception exp) {
		logmsgThrowable(exp);
	}

	dkDatastore rc = (_fQueryMapServer) ? _dsMapServer : _dkDatastore;
	return rc;
}
public void listDatastoreInfo() throws Exception {
	logmsg("-->Loaded      : " + getdkDatastore().getClass().getName());
	logmsg("-->DBName UID  : " + getdkDatastore().datastoreName() + " " + 
			getdkDatastore().userName());
	logmsg("-->Connected?  : " + getdkDatastore().isConnected());
}
public final short getPackageType() {
	return _nPkgType;
}
public final void setPackageType(String aPkgName) {
	_nPkgType = SH_PKG_UNKNOWN;
	if(     aPkgName.equalsIgnoreCase("client") ) {
		_nPkgType   = SH_PKG_CLIENT;
	}
	else if(aPkgName.equalsIgnoreCase("server") ) {
		_nPkgType   = SH_PKG_SERVER;
	}
	else if(aPkgName.equalsIgnoreCase("cs") ) {
		_nPkgType   = SH_PKG_CS;
	}

	_strPkgName = aPkgName.toUpperCase();
}
public final short getDBType() {
	return _nDBType;
}
public static final short getDBType(String aDBType) {
	short rc   = SH_DB_UNKNOWN;

	if(     aDBType.equalsIgnoreCase("DD"  ) ) {
		rc   = SH_DB_DD;
	}
	else if(aDBType.equalsIgnoreCase("DES" ) ) {
		rc   = SH_DB_DES;
	}
	else if(aDBType.equalsIgnoreCase("DL"  ) ) {
		rc   = SH_DB_DL;
	}
	else if(aDBType.equalsIgnoreCase("FED" ) ) {
		rc   = SH_DB_FED;
	}
	else if(aDBType.equalsIgnoreCase("IP"  ) ) {
		rc   = SH_DB_IP;
	}
	else if(aDBType.equalsIgnoreCase("JDBC") ) {
		rc   = SH_DB_JDBC;
	}
	else if(aDBType.equalsIgnoreCase("OD"  ) ) {
		rc   = SH_DB_OD;
	}
	else if(aDBType.equalsIgnoreCase("QBIC") ) {
		rc   = SH_DB_QBIC;
	}
	else if(aDBType.equalsIgnoreCase("TS"  ) ) {
		rc   = SH_DB_TS;
	}
	else if(aDBType.equalsIgnoreCase("V4"  ) ) {
		rc   = SH_DB_V4;
	}
	return rc;
}
public final void setDBType(String aDBType) {
	_nDBType   = getDBType(aDBType);
	if(_nDBType == SH_DB_FED) {
		_strDBName = "Fed";
	}
	else {
		_strDBName = aDBType.toUpperCase();
	}
}
public String getDBName() {
	return _strDBName;
}
public dkDatastore setdkDatastore(String aPackage, String aDBType) {
	dkDatastore ldkDatastore = null;

	setPackageType(aPackage);
	setDBType(aDBType);

	try {
		switch(_nDBType) {
			case SH_DB_FED :
				switch(_nPkgType) {
					case SH_PKG_CLIENT : ldkDatastore = new com.ibm.mm.sdk.client.DKDatastoreFed();  break;
					case SH_PKG_SERVER : ldkDatastore = new com.ibm.mm.sdk.server.DKDatastoreFed();  break;
					case SH_PKG_CS     : ldkDatastore = new com.ibm.mm.sdk.cs.DKDatastoreFed();      break;
				}
				break;

			case SH_DB_DL :
				switch(_nPkgType) {
					case SH_PKG_CLIENT : ldkDatastore = new com.ibm.mm.sdk.client.DKDatastoreDL();   break;
					case SH_PKG_SERVER : ldkDatastore = new com.ibm.mm.sdk.server.DKDatastoreDL();   break;
					case SH_PKG_CS     : ldkDatastore = new com.ibm.mm.sdk.cs.DKDatastoreDL();       break;
				}
				break;

			case SH_DB_TS :
				switch(_nPkgType) {
					case SH_PKG_CLIENT : ldkDatastore = new com.ibm.mm.sdk.client.DKDatastoreTS();   break;
					case SH_PKG_SERVER : ldkDatastore = new com.ibm.mm.sdk.server.DKDatastoreTS();   break;
					case SH_PKG_CS     : ldkDatastore = new com.ibm.mm.sdk.cs.DKDatastoreTS();       break;
				}
				break;

			case SH_DB_QBIC :
				switch(_nPkgType) {
					case SH_PKG_CLIENT : ldkDatastore = new com.ibm.mm.sdk.client.DKDatastoreQBIC(); break;
					case SH_PKG_SERVER : ldkDatastore = new com.ibm.mm.sdk.server.DKDatastoreQBIC(); break;
					case SH_PKG_CS     : ldkDatastore = new com.ibm.mm.sdk.cs.DKDatastoreQBIC();     break;
				}
				break;
		}
	}
	catch(Exception exp) {
		logmsgThrowable(exp);
	}

	setdkDatastore(ldkDatastore);

	return ldkDatastore;
}
public void processCommand(String aCmd) {
	try {
		String args[] = getParms(aCmd);

		if(args != null && args.length > 0) {
			if(     args[0].equalsIgnoreCase("listDatastoreInfo")) {
				listDatastoreInfo();
			}
			else if(args[0].equalsIgnoreCase("setdkDatastore")) {
				setdkDatastore(args[1], args[2]);

				if(getdkDatastore() != null) {
					logmsg("-->Loaded " + getdkDatastore().getClass().getName() + " successfully");
				}
				else {
					logmsgError("creating datastore");
				}
			}
			else if(args[0].equalsIgnoreCase("connect")) {
				connect(args);
			}
			else if(args[0].equalsIgnoreCase("disconnect")) {
				disconnect();
			}
			else if(args[0].equalsIgnoreCase("setMapServer")) {
				setMapServer(args[1]);
			}
			else if(args[0].equalsIgnoreCase("queryMapServer")) {
				if(args.length == 1) {
					queryMapServer(true);
				}
				else {
					queryMapServer(args[1]);
				}
			}
			else if(args[0].equalsIgnoreCase("executeTemplates")) {
				if(args.length == 1) {
					executeTemplates();
				}
				else {
					executeTemplates(args[1]);
				}
			}
			else if("executeX, executeWithCallbackX, evaluateX".indexOf(args[0]+"X") >= 0) {
				short nQLType = DK_UNKNOWN_QL_TYPE;

				if("STR, OBJ".indexOf(args[1]) >= 0) {
					strErrLoc   = args[0] + "(\"" + args[2] + "\", " + args[3] + ")";
					logmsg("                 " + strErrLoc);
				}

				if(args.length < 3) {
					logmsg("command usage:");
					logmsg("     " + args[0] + " ObjectType(STR/OBJ/EXP/TPL)  QueryString [QLType(PQ/TQ/PQorTQ/PQandTQ)]");
					logmsg("e.g. " + args[0] + " STR \"QueryString\" PQ");
				}
				if(args.length >= 3) {
					if(args.length > 3) {
						nQLType = getQLType(args[3]);
					}

					strErrLoc   = args[0] + "(" + args[1] + ")";
					strErrInput = args[2];

					if("STRx, OBJx, EXPx, TPLx".indexOf(args[1].toUpperCase() + "x") < 0) {
						logmsgError("unknown Object type!  Allowed values are STR/OBJ/EXP/TPL");
					}
					else if(args[0].equalsIgnoreCase("executeWithCallback")) {
						if(     args[1].equalsIgnoreCase("STR")) {
							executeWithCallback(args[2], nQLType);
						}
						else if(args[1].equalsIgnoreCase("OBJ")) {
							executeWithCallback((dkQuery)convertToQuery(args));
						}
						else if("EXPx, TPLx".indexOf(args[1].toUpperCase() + "x") >= 0) {
							executeWithCallback((DKCQExpr)convertToQuery(args));
						}
					}
					else if(args[0].equalsIgnoreCase("execute")) {
						if(     args[1].equalsIgnoreCase("STR")) {
							execute(args[2], nQLType);
						}
						else if(args[1].equalsIgnoreCase("OBJ")) {
							execute((dkQuery)convertToQuery(args));
						}
						else if("EXPx, TPLx".indexOf(args[1].toUpperCase() + "x") >= 0) {
							execute((DKCQExpr)convertToQuery(args));
						}
					}
					else if(args[0].equalsIgnoreCase("evaluate")) {
						if(     args[1].equalsIgnoreCase("STR")) {
							evaluate(args[2], nQLType);
						}
						else if(args[1].equalsIgnoreCase("OBJ")) {
							evaluate((dkQuery)convertToQuery(args));
						}
						else if("EXPx, TPLx".indexOf(args[1].toUpperCase() + "x") >= 0) {
							evaluate((DKCQExpr)convertToQuery(args));
						}
					}
				}
			}
			else {
				super.processCommand(aCmd);
			}
		}
	}
	catch(Exception exp) {
		logmsgThrowable(exp);
	}
}
private Object convertToQuery(String[] args) throws DKException, Exception {
	Object rc = null;

	if(args != null) {
		if(     args[1].equalsIgnoreCase("OBJ")) {
			if(args.length >= 4) {
				rc = getdkDatastore().createQuery(args[2], getQLType(args[3]), null);
			}
		}
		else if("EXPx, TPLx".indexOf(args[1].toUpperCase() + "x") >= 0) {
			if(args.length >= 3) {
				dkSearchTemplate lST = null;
				if(     getdkDatastore() instanceof com.ibm.mm.sdk.server.DKDatastoreFed) {
					lST = ((com.ibm.mm.sdk.server.DKDatastoreFed)getdkDatastore()).
							getSearchTemplate(args[2]);
				}
				else if(getdkDatastore() instanceof com.ibm.mm.sdk.client.DKDatastoreFed) {
					lST = ((com.ibm.mm.sdk.client.DKDatastoreFed)getdkDatastore()).
							getSearchTemplate(args[2]);
				}
				else if(getdkDatastore() instanceof com.ibm.mm.sdk.cs.DKDatastoreFed) {
					lST = ((com.ibm.mm.sdk.cs.DKDatastoreFed)getdkDatastore()).
							getSearchTemplate(args[2]);
				}

				if(lST != null) {
					lST.listCriteria();
					if(setCriteriaValues(lST)) {
						rc = lST.createQueryExpr();
					}
				}
				else {
					logmsgError("SearchTemplate not found");
				}
			}
		}
	}

	if(rc == null) {
		logmsgError("creating QueryObject");
	}

	return rc;
}
protected DKDatastoreAdminFed getDatastoreAdminFed() throws DKException, Exception {
	DKDatastoreAdminFed rc = null;
	dkDatastoreAdmin lDSAdmin = getdkDatastore().datastoreDef().datastoreAdmin();
	if(lDSAdmin instanceof DKDatastoreAdminFed) {
		rc = ((DKDatastoreAdminFed)lDSAdmin);
	}

	return rc;
}
private short getQLType(String astrQLType) {
	short shQLType = DK_UNKNOWN_QL_TYPE;
	if(getDBType() == SH_DB_FED) {
		shQLType = DK_CM_FEDERATED_QL_TYPE;
	}
	else if(astrQLType != null) {
		if(     astrQLType.equalsIgnoreCase("TQ")) {
			shQLType = DK_TEXT_QL_TYPE;
		}
		else if(astrQLType.equalsIgnoreCase("PQ")) {
			shQLType = DK_PARAMETRIC_QL_TYPE;
		}
		else if("PQORTQ, PQANDTQ, PQTQ, PQNTQ, TQORPQ, TQANDPQ, TQPQ, TQNPQ".
				indexOf(astrQLType.toUpperCase()) >= 0) {
			shQLType = DK_COMBINED_QL_TYPE;
		}
	}
	return shQLType;
}
public dkDatastore getMapServer() {
	return _dsMapServer;
}
public void setMapServer(String aName) throws DKException, Exception {
	if(getMapServer() == null || !(getMapServer().datastoreName().equals(aName))) {
	boolean fQueryMS = isQueryMapServer();
		queryMapServer(false);

		try {
			DKServerMapDefFed lServerMapDefFed = getDatastoreAdminFed().getServer(aName);
			String            lDSType          = lServerMapDefFed.datastoreType();

			if(_dsMapServer != null && _dsMapServer.isConnected() ) {
				_dsMapServer.disconnect();
				_dsMapServer = null;
			}
			try{
				logmsg("-->Connecting to native server");
				_dsMapServer = (dkDatastore)(Class.forName("com.ibm.mm.sdk.server.DKDatastore" + lDSType).newInstance());
			}
			catch(ClassNotFoundException cnfe) {
				_dsMapServer = (dkDatastore)(Class.forName("com.ibm.mm.sdk.client.DKDatastore" + lDSType).newInstance());
			}

			if(_dsMapServer != null) {
				if(!_dsMapServer.isConnected()) {
					_dsMapServer.connect(aName, _strUid, _strPwd, lServerMapDefFed.getConnectString());
					logmsg("Connected to " + aName);
				}
			}
			else {
				logmsg(STR_ERROR + "finding server with that name");
			}
		}
		finally {
			queryMapServer(fQueryMS);
		}
	}
}
public final void queryMapServer(String strEnable) {
	boolean fEnable = (strEnable != null && strEnable.equalsIgnoreCase("true"));
	queryMapServer(fEnable);
}
public final void queryMapServer(boolean fEnable) {
	_fQueryMapServer = fEnable;
}
public boolean isQueryMapServer() {
	return _fQueryMapServer;
}
protected void initVectVars() {
	if(!_isHelpInfoAdded_TdkDatastore) {
		super.initVectVars();
		//_vectMethods.addElement(new _MethodInfo("", "", "", "", ""));

		_vectMethods.addElement(new _MethodInfo("listDatastoreInfo",   "Lists datastore information.",             "", "", ""));
		_vectMethods.addElement(new _MethodInfo("setdkDatastore",      "Creates specified instance of Datastore.", "setdkDatastore {client|server|cs}, {Fed|DL|TS|DES|QBIC}", "", ""));
		_vectMethods.addElement(new _MethodInfo("connect",             "Connect to Datastore.",                    "connect _DBName_, _UID_, _PWD_ [, _connectString_]", "", ""));
		_vectMethods.addElement(new _MethodInfo("disconnect",          "Disconnect from Datastore.",               "", "", ""));
		_vectMethods.addElement(new _MethodInfo("setMapServer",        "Creates instance of native server datastore and connects to it.",   "setMapServer _NativeServerName_", "e.g. setMapServer LSMUFASA", "native, backend"));
		_vectMethods.addElement(new _MethodInfo("queryMapServer",      "set/unset current datastore as Native server datastore.", "queryMapServer {true|false}", "", "native, backend"));
		_vectMethods.addElement(new _MethodInfo("execute",             "Call execute method on current datastore.",               "execute {STR|OBJ}, {_QueryString_}, {PQ|TQ|PQorTQ|PQandTQ}", "", "runquerystring"));
		_vectMethods.addElement(new _MethodInfo("executeWithCallback", "Call executeWithCallback method on current datastore.",   "executeWithCallback {STR|OBJ}, {_QueryString_}, {PQ|TQ|PQorTQ|PQandTQ}", "", "runquerystring"));
		_vectMethods.addElement(new _MethodInfo("evaluate",            "Call evaluate method on current datastore.",              "evaluate {STR|OBJ}, {_QueryString_}, {PQ|TQ|PQorTQ|PQandTQ}", "", "runquerystring"));
		_vectMethods.addElement(new _MethodInfo("executeTemplates",    "Executes all templates that are in current datastore.",   "executeTemplates [RSC|CBK|RCL]", "", ""));

		_isHelpInfoAdded_TdkDatastore = true;
	}
}

public static void main(String argv[]) {
	try {
		TdkDatastore lTdkDatastore = new TdkDatastore();
		lTdkDatastore.processCommandLineArgs(argv);
	}
	catch(Exception exp) {
		exp.printStackTrace();
	}
}
	private static boolean     _isHelpInfoAdded_TdkDatastore;

	private        dkDatastore _dkDatastore     = null;
	// this is used to query MappedServer when connected as client
	// e.g. to see what are all available entities/attributes on the
	//      server so that we can create schema-mapping
	private        dkDatastore _dsMapServer     = null;

	// if enabled getdkDatastore will return value in _dsMapServer
	// instead of _dkDatastore
	private        boolean     _fQueryMapServer = false;

	private        short       _nPkgType        = 0;
	private        short       _nDBType         = 0;
	private        String      _strPkgName      = null;
	private        String      _strDBName       = null;
	private        String      _strUid          = "";
	private        String      _strPwd          = "";

	private static Hashtable   _opTable         = null;
	private static Hashtable   _valueTable      = null;

	static {
		_isHelpInfoAdded_TdkDatastore = false;

		_opTable    = new Hashtable();
		_opTable.put("Title",   (Object)(new Short(DKOpCode.LIKE)));
		_opTable.put("Subject", (Object)(new Short(DKOpCode.NEQ )));
		_opTable.put("Author",  (Object)(new Short(DKOpCode.LIKE)));
		_opTable.put("UserID",  (Object)(new Short(DKOpCode.NEQ )));
		_opTable.put("Date",    (Object)(new Short(DKOpCode.NEQ )));
		_opTable.put("XXX",     (Object)(new Short(DKOpCode.NEQ )));
		_opTable.put("YYY",     (Object)(new Short(DKOpCode.EQ  )));

		_valueTable = new Hashtable();
		_valueTable.put("Title",   "'Moving%'"  );
		_valueTable.put("Subject", "NULL"       );
		_valueTable.put("Author",  "'Randall%'" );
		_valueTable.put("UserID",  "'BASUKI'"   );
		_valueTable.put("Date",    "NULL"       );
		_valueTable.put("XXX",     "'xxx'"      );
		_valueTable.put("YYY",     "'yyy'"      );
	}

/*
 # ------------------------------------------------------------------------- 
 # Sample Inputfile
 # ------------------------------------------------------------------------- 
 #setdkDatastore client/server Fed/DL/QBIC
  setdkDatastore server DL  // will load com.ibm.mm.sdk.server.DKDatastoreDL
 
 #connect DBName    uid   pwd   command
  connect lsmufasa  edms1 edms1 add
 
 #execute             STR/OBJ "QueryString" PQ/TQ 
 #execute             TPL/EXP  SearchTemplateName
  execute             STR "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ
  execute             OBJ "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ
 
 #evaluate            STR/OBJ "QueryString" PQ/TQ 
 #evaluate            TPL/EXP  SearchTemplateName
  evaluate            STR "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ
  evaluate            OBJ "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ
 
 #executeWithCallback STR/OBJ "QueryString" PQ/TQ 
 #executeWithCallback TPL/EXP  SearchTemplateName
  executeWithCallback OBJ "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ
  executeWithCallback STR "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=1, COND=((DLSEARCH_DocType <> NULL) AND (DLSEARCH_Date >= 1995))); OPTION=(CONTENT=YES; TYPE_QUERY=DYNAMIC; TYPE_FILTER=FOLDERDOC)"  PQ

 #executeTemplates HowToCall(RSC/CBK/RCL)
  executeTemplates RSC      // will loop through all SearchTemplates in current Datastore
                            // and execute them with ResultSetCursor after converting them to
                            // DKCQExpr
  executeTemplates          // will execute all SearchTemplates in all three ways
 
  disconnect()
 # ------------------------------------------------------------------------- 
*/
}
