/* -------------------------------------------------------------------
 * @copyright(disclaimer) 
 * IBM Enterprise Information Portal for Multiplatforms V7.1  (program number 5697-G29) 
 * (c ) Copyright IBM Corp. 1998, 2000. All Rights Reserved. 
 * IBM Content Manager for Multiplatforms V6.1  (program number 5648-D03) 
 * (c ) Copyright IBM Corp. 1994, 2000. All Rights Reserved. 
 * 
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.  Licensed Materials - Property of IBM 
 * 
 * DISCLAIMER OF WARRANTIES : 
 * 
 * Permission is granted to copy and modify this code, and to 
 * distribute modified versions provided that both the copyright 
 * notice and this permission notice appear in all copies and modified 
 * versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS 
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM 
 * WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE 
 * OF THE CODE.
 * 
 * @endCopyright
 */

/*
 * Filename       : TMappingIC.java
 * Description    : Connect to IC Datastore.
 *                  Run samples for mapping methods.
 *                  Disconnect from IC Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

// TMappingIC class
public class TMappingIC implements DKConstantIC
{


  // Main method
  public static void main(String argv[])
  {
          
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
      //*** for input parameters
      String db = new String("");   // database
      String userid = new String("");   // userid
      String pw = new String("");   // password

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TMappingIC " +
        "<db> <userid> <pw>" );
       return;
      }
      else
      {
       db = argv[0];
       userid = argv[1];
       pw = argv[2];
      } 

    DKDatastoreIC dsIC = new DKDatastoreIC();
    dkResultSetCursor pCur = null;
    DKNVPair parms[] = null;
    String strMappedEntityName = "docs";
    String strEntityName = "Documents";
    DKCQExpr cqe = new DKCQExpr();
    DKQExpr qe = new DKQExpr();
    DKQSubExpr sqe = new DKQSubExpr();

    dsIC.connect(db, userid, pw,"");

// Map info to DOCS table of IC
// DOCS - documents  (IC Name - Documents)
// SHRTDESC - short desc  (IC Name - Short description)
// LONGDESC - Long desc  (IC Name - Long description)
// AUTHOR - Writer  (IC Name - Document author)
// LOCATION - Site  (IC Name - Location)


    String qry = "SEARCH=(ENTITY=" + strMappedEntityName + "," +
         "MAX_RESULTS=10,COND=(( 'short desc' LIKE '%information%')" +
         " AND ('writer' <> 'IBM') OR ('writer' IN 'IBM')));" +
         " OPTION = (CONTENT=YES)";

    DKQExpr dkq = com.ibm.mm.sdk.server.DKDatastoreIC.parseParametricQuery(qry);
    dkq.getSubQuery(0).setMappingName(strMappedEntityName);
    dkq.getSubQuery(0).setTranslation(true);


    DKSchemaMappingFed mapFed = new DKSchemaMappingFed(strMappedEntityName);
    mapFed.setServerName(db);
    mapFed.setServerType("IC");
    mapFed.setEntityName(strEntityName);
    mapFed.setMappedEntityName(strMappedEntityName);
 

    DKDDO p = null;
    int i = 0;
    int numDataItems = 0;
    short j = 0;
    String strDataName;
    Object a = null;

    dkIterator itr = dsIC.listEntityAttrs("Documents").createIterator();
    while ( itr.more() ) {
       dkAttrDef attr = (dkAttrDef)itr.next();
//Set mapping for "SHRTDESC"
       String attrName = attr.getName();
       if (attrName.equals("Short description")) {
          DKAttrMappingFed attrMap = new DKAttrMappingFed( "short desc");
          DKAttrDefIC attrF = new DKAttrDefIC();
          attrF.setName( "short desc");
          attrMap.addMappedAttr(attrF);
          attrMap.addMapToAttr(attr);
          mapFed.addAttrMapping(attrMap);
        }
//Set mapping for "AUTHOR"
        else if (attrName.equals("Document author")) {
          DKAttrMappingFed attrMap = new DKAttrMappingFed("writer");
          DKAttrDefIC attrF = new DKAttrDefIC();
          attrF.setName( "writer");
          attrMap.addMappedAttr(attrF);
          attrMap.addMapToAttr(attr);
          mapFed.addAttrMapping(attrMap);
        }
     }


    DKNVPair sourceMap = new DKNVPair("SCHEMA",mapFed);
    String mapping = dsIC.registerMapping(sourceMap);
    cqe = new DKCQExpr(dkq);
    pCur = dsIC.execute(cqe);
    while (pCur.isValid())
    {
     p = pCur.fetchNext();
     if (p != null)
     {
      i = pCur.getPosition();
      System.out.println("=====> Item " + i + " <=====");
      numDataItems = p.dataCount();
      System.out.println("Number of Data Items " + numDataItems);
      for (j = 1; j <= numDataItems; j++)
      {
       a = p.getData(j);
       strDataName = p.getDataName(j);
       System.out.println("Attribute Name: " + strDataName );
       if (a instanceof String)
       {
        System.out.println("Attribute Value: " + a);    
       }
       else if (a instanceof Integer)
       {
        System.out.println("Attribute Value: " + a);    
       }
       else if (a instanceof Short)
       {
        System.out.println("Attribute Value: " + a);    
       }
      }
     }
    }
    pCur.destroy(); // Finished with the cursor

// getMapping

    dkSchemaMapping sm = dsIC.getMapping(mapping);
    System.out.println("get mapping returns: " + sm.getName());

// listMappingNames
// add an additional mapping for APPROACH and listMappingNames

    DKSchemaMappingFed mapFed2 = new DKSchemaMappingFed("spreadsheets");
    mapFed2.setServerName(db);
    mapFed2.setServerType("IC");
    strMappedEntityName = "spreadsheets";
    strEntityName = "APPROACH";
    mapFed2.setEntityName(strEntityName);
    mapFed2.setMappedEntityName(strMappedEntityName);
    sourceMap = new DKNVPair("SCHEMA",mapFed2);
    mapping = dsIC.registerMapping(sourceMap);

    String[] mNames = dsIC.listMappingNames();
    for (int x= 0; x < mNames.length; x++) {
        System.out.println("Mapping name: " + mNames[x]);
    }

//
// unRegisterMapping
//

    dsIC.unRegisterMapping(mapping);

    mNames = dsIC.listMappingNames();
    for (int x= 0; x < mNames.length; x++) {
        System.out.println("Mapping name: " + mNames[x]);
    }



    dsIC.disconnect();
    System.out.println("--- datastore disconnected ---");
    dsIC.destroy();
    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

