/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating an Attribute Definition
    Attribute Definitions exist in two forms, the general Attribute Definition and the
    tailored Attribute Definition for a specific Component Type.  Modifications of the
    general Attribute Definition will not affect those already added to Item or Component
    Types.  This prevents breaking, destroying, or corrupting existing Data Models and 
    existing Items of those Data Models.
    
    As explained in both SAttributeDefinitionCreationICM & SItemTypeCreationICM, Attributes
    may be tailored for a particular situation applicable to the specific Item/Component
    Type that it belongs to.
    
    Datastore Definition's Attributes
        To modify the general characteristics of an Attribute Definition, such as name or
        description, and will be applicable only to new Item or Component Types created
        using this Attribute, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To modify any property of an Attribute Definition already added to an Item or 
        Component Type, the definition object must be retrieved from the Component Type
        Definition.  Modifications will apply only to this particular Component Type.
        
        Hint:  Use DKComponentTypeDef.retrieveAttr().  Please refer to SItemTypeRetrievalICM
               for more information on how to retrieve an existing Component Type Definition.

This Sample will update the Datastore Definition's Attributes.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Saving Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Attribute Definitions in Memory
 *                    Saving / Updating Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeDefinitionUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes must already be defined.  If not, please run Sample
 *                    SAttributeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SAttributeDefinitionRetrievalICM.java
 ************************************************************************************************/
public class SAttributeDefinitionUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeDefinitionUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("==============================================");
        System.out.println("IBM Information Integrator for Content    v"+ver);
        System.out.println("Sample Program:  SAttributeDefinitionUpdateICM");
        System.out.println("----------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("==============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve & Print the S_varchar Attribute Definition
            //----------------------------------------------------------------
            // Retrieving the Datastore Definition's Attribute Definition.  Please
            // refer to the header documentation for an explanation of the difference
            // between that defined within the Datastore Definition compared to the 
            // Component Type Definition.
            //
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            System.out.println("Retrieving the Attribute Definition of 'S_varchar'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrDefICM attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_varchar"); // Retrieve the definition.
                SAttributeDefinitionRetrievalICM.printAttrDef(attrDef); // Use wrapper funciton from this Sample.
                
            System.out.println("Retrieved the Attribute Definition of 'S_varchar'.");

            //----------------------------------------------------------------
            // Modify the S_varchar Attribute Definition in Memory
            //----------------------------------------------------------------
            // Change any of the characteristics of an attribute definition.  This example
            // will change just one characteristic, the description.
            //
            // Updating the Datastore Definition's Attribute Definition.  Please
            // refer to the header documentation for an explanation of the difference
            // between that defined in the Datastore Definition and the Component Type
            // Definition.
            System.out.println("Modify the Attribute Definition 'S_varchar' in Memory...");
            
                attrDef.setDescription("My modified varchar attribute description.");

            System.out.println("Modified the Attribute Definition of 'S_varchar' in Memory.");

            //----------------------------------------------------------------
            // Update the S_varchar Attribute Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The attribute definition can be updated through its own update() method
            // or by using the Datastore Definition's update() method, passing the attribute
            // definition as the parameter.
            System.out.println("Updating the Attribute Definition 'S_varchar' in Datastore Definition...");

                attrDef.update(); // save the attribute definition changes.

            System.out.println("Modified the Attribute Definition of 'S_varchar' in Datastore Definition.");

            //----------------------------------------------------------------
            // Print the Attribute Definition with changes.
            //----------------------------------------------------------------
            System.out.println("Printing Attribute Definition...");
                SAttributeDefinitionRetrievalICM.printAttrDef(attrDef); // Sample's Wrapper            
            System.out.println("Printed Attribute Definition.");

            //----------------------------------------------------------------
            // Retrieve & Print the S_integer Attribute Definition
            //----------------------------------------------------------------
            System.out.println("Retrieving the Attribute Definition of 'S_integer'...");

                attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_integer"); // Retrieve the definition.
                SAttributeDefinitionRetrievalICM.printAttrDef(attrDef); // Use wrapper funciton from this Sample.
                
            System.out.println("Retrieved the Attribute Definition of 'S_integer'.");

            //----------------------------------------------------------------
            // Modify the S_integer Attribute Definition in Memory
            //----------------------------------------------------------------
            // Change any of the characteristics of an attribute definition.
            System.out.println("Modify the Attribute Definition 'S_integer' in Memory...");
            
                attrDef.setDescription("My modified Integer attribute description.");

            System.out.println("Modified the Attribute Definition of 'S_integer' in Memory.");

            //----------------------------------------------------------------
            // Update the S_integer Attribute Definition using an Alternative Method.
            //----------------------------------------------------------------
            System.out.println("Updating the Attribute Definition 'S_integer' Using an Alternative Method...");

                dsDefICM.update(attrDef); // save the attribute definition changes.

            System.out.println("Modified the Attribute Definition of 'S_integer' Using an Alternative Method.");

            //----------------------------------------------------------------
            // Print the Attribute Definition with changes.
            //----------------------------------------------------------------
            System.out.println("Printing Attribute Definition...");
                SAttributeDefinitionRetrievalICM.printAttrDef(attrDef); // Sample's Wrapper            
            System.out.println("Printed Attribute Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
    
}//end class SAttributeDefinitionUpdateICM
              
