/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Attribute Group Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute Group Definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Attribute Group Definitions, Insurance Model Attribute Group
 *                    Definitions, and Other Sample Attribute Groups
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Attribute Group Definitions
 *                    Deleting Insurance Model Attribute Group Definitions
 *                    Deletes all Attribute Groups Defined in SAttributeGroupDefCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeGroupDefDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attribute Groups used here should be already defined.  If not, Sample
 *                    SAttributeGroupDefCreationICM may be run.  Sample will complete whether
 *                    the attribute groups exist or not.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SAttributeGroupDefDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeGroupDefUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SAttributeGroupDefDeletionICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Retrieving an Attribute Group Definition
            //--------------------------------------------------------------
            // Retrieving the "S_carAccident" attribute Group.
            //
            // For more information on Attribute Group Retrieval, please consult
            // the Sample SAttributeGroupDefRetrievalICM.
            System.out.println("Retrieving Attribute Group 'S_carAccident'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_carAccident"); // Retrieve the definition.

            System.out.println("Retrieved Attribute Group 'S_carAccident'.");

            //--------------------------------------------------------------
            // Delete Attribute Group Definition in Datastore Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's del() method can be called
            // as well as the Datastore Definition's del(attrGroupDef) method.
            //
            // This example will use the Attribute Group's del() method.
            System.out.println("Deleting Attribute Group Definition 'S_carAccident' in Datastore...");

                if(attrGroupDef!=null) // if it is not already deleted...
                    attrGroupDef.del();  // Delete Attribute Group Definition in Datastore Definition

            System.out.println("Deleted Attribute Group Definition 'S_carAccident' in Datastore.");

            //-------------------------------------------------------------
            // Retrieve & Delete "Address"
            //-------------------------------------------------------------
            // This method uses the Datastore Definition's del(attrGroup) method.
            System.out.println("Retrieving & Deleting Attribute Group Definition 'S_address'...");

                attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_address"); // Retrieve the definition.

                if(attrGroupDef!=null) // if it is not already deleted...
                    dsDefICM.del(attrGroupDef);  // Delete Attribute Group Definition in Datastore Definition

            System.out.println("Deleted Attribute Group Definition 'S_address'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Attribute Group Definition
    * @param dsICM          Connected DKDatastoreICM object.
    * @param attrGroupName  Attribute Group Name for the Attribute Group Definition to Delete.
    **/
    public static void deleteAttrGroupDef(DKDatastoreICM dsICM, String attrGroupName) throws DKException, Exception{
        System.out.println("Deleting the Attribute Group Definition '"+attrGroupName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup(attrGroupName); // Retrieve the definition.
            if(attrGroupDef!=null) // if it is not already deleted...
                attrGroupDef.del();   // Delete the Attribute Group Definition

        System.out.println("Deleted the Attribute Group Definition '"+attrGroupName+"'.");
    }
    
}//end class SAttributeGroupDefDeletionICM
              
