/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Item Types can be deleted by using either their own del() functions or by the Datastore 
Definition's del() function.

Depending on the product configuration, subEntities / child components / multivalue attributes
may be required to be deleted first, before the root Item Type can be deleted.  If this is 
the case, please refer to Sample SItemTypeRetrievalICM for learning how to access the
Sub-Entities (See Listing Sub-Entitiesin SItemTYpeRetrievalICM).  Once a reference to the 
entitiy definition is objetained, the del() method may be called to deleted it.

NOTE:  When retrieving Item Type Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Item Type definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemTypeDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Item Type Definitions Using its del() Method
 *                    Deleting Item Type Definitions Using the Datastore Definition's del() Method
 *                    Deletes all Insurance Model Item Types
 *                    Deletes all Item Types Defined in Sample SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemTypeDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types should be defined.  If not, you may run Sample 
 *                    SItemTypeCreationICM first, otherwise sample will complete, but since
 *                    none of the Item Types will be found, no action will take place.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SItemTypeDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemTypeDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemTypeDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------------
            // Deleting Item Type Definition using it's del() method
            //-------------------------------------------------------------------
            // Item Types can be deleted by using either their own del() functions
            // or by the Datastore Definition's del() function.
            System.out.println("Retrieving & Deleting Item Type Definition of 'S_simple'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM itemTypeDef = (DKItemTypeDefICM) dsDefICM.retrieveEntity("S_simple"); // Retrieve the Item Type Definition.

                if(itemTypeDef!=null) // if it is not already deleted...
                    itemTypeDef.del(); // Delete the Item Type Definition

            System.out.println("Deleted Item Type Definition of 'S_simple'.");

            //----------------------------------------------------------------------------
            // Deleting Item Type Definition using the Datastore Definition's del() method
            //----------------------------------------------------------------------------
            System.out.println("Retrieving & Deleting Item Type Definition of 'S_withMultAttrs'...");

                itemTypeDef = (DKItemTypeDefICM) dsDefICM.retrieveEntity("S_withMultAttrs"); // Retrieve the Item Type Definition.

                if(itemTypeDef!=null) // if it is not already deleted...
                    dsDefICM.del(itemTypeDef); // Delete the Item Type Definition

            System.out.println("Deleted Item Type Definition of 'S_withMultAttrs'.");

            //----------------------------------------------------------------
            // Delete the rest of the Item Types created in the creation sample
            //----------------------------------------------------------------
            // Delete all Item Types created in the SItemTypeCreationICM
            // sample, including the Insurance Model.
            deleteItemTypeDef(dsICM,"S_autoClaim");
            deleteItemTypeDef(dsICM,"S_adjustrReport");
            deleteItemTypeDef(dsICM,"S_policeReport");
            deleteItemTypeDef(dsICM,"S_insurePolicy");
            deleteItemTypeDef(dsICM,"S_autoPhoto");
            deleteItemTypeDef(dsICM,"S_docModel");
            deleteItemTypeDef(dsICM,"S_lob");
            deleteItemTypeDef(dsICM,"S_text");
            deleteItemTypeDef(dsICM,"S_image");
            deleteItemTypeDef(dsICM,"S_withChild");
            
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Item Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  Item Type Name for the Item Type Definition to Delete.
    **/
    public static void deleteItemTypeDef(DKDatastoreICM dsICM, String itemTypeName) throws DKException, Exception{
        System.out.println("Deleting the Item Type Definition '"+itemTypeName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKItemTypeDefICM itemTypeDef = (DKItemTypeDefICM) dsDefICM.retrieveEntity(itemTypeName); // Retrieve the definition.
            if(itemTypeDef!=null){ // if it is not already deleted...
                // If it is a CM Doc Model Item Type, delete an Item Type Relations
                if(itemTypeDef.getClassification()==DKConstantICM.DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                    System.out.println("Deleting Item Type Relations...");
                    dkCollection coll = dsDefICM.retrieveItemTypeRelations(itemTypeDef.getItemTypeId());
                    dkIterator iter = coll.createIterator();
                    while(iter.more()){
                        DKItemTypeRelationDefICM itemTypeRel = (DKItemTypeRelationDefICM) iter.next();
                        itemTypeRel.del();
                    }
                }
                itemTypeDef.del();   // Delete the Item Type Definition
            }

        System.out.println("Deleted the Item Type Definition '"+itemTypeName+"'.");
    }

}//end class SItemTypeDeletionICM
              
