/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Link Type Definitions, if it does not exist, null is returned.  
       Usage of the null object will result in a NullPointerException.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving, Printing, & Listing Link Type Definitions in the System.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Link Type Definitions
 *                    Printing Link Type Definitions
 *                    Listing All Link Types Available in the System
 *                    Obtaining All Link Type Definitions in the System
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SLinkTypeDefinitionRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types must already be defined.  If not, please run Sample
 *                    SLinkTypeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SLinkTypeDefinitionRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SLinkTypeDefinitionRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SLinkTypeDefinitionRetrievalICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Obtaining all Link Type Definitions in the System
            //----------------------------------------------------------------
            System.out.println("Obtaining All Link Type Definitions in the System...");
        
        	    DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        	    
        	    dkCollection linkTypeColl = dsDefICM.listLinkTypes(); // Obtain all of the Link Type Definition objects in the system.
        	    // NOTE: dsDefICM.listLinkTypeName() could also be used to just obtain the names.

            System.out.println("Obtained All Link Type Definitions in the System.");

            //----------------------------------------------------------------
            // List all Link Types Available in the System
            //----------------------------------------------------------------
            // Uses the link type definitions retrieved above.
            System.out.println("Listing All Link Types Available In System...");

        	    // Print each in the list retrieved above.
        	    System.out.println("All Link Types in System:");
        	    dkIterator iter = linkTypeColl.createIterator(); // create an iterator go iterate through collection.
        	    while(iter.more()){                              // while there still more items in the colleciton, continue
        	        DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) iter.next(); // Move pointer to next element & return that object.
        	        System.out.println("     - "+linkTypeDef.getName()+":  "+linkTypeDef.getDescription());
        	    }

            System.out.println("Listed All Link Types Available In System.");

            //----------------------------------------------------------------
            // Retrieve the S_policeReports Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined Link Types, please consult
            // the SLinkTypeDefinitionCreationICM sample.  For more information
            // on linking, please consult the SLinksICM Sample.
            //
            // NOTE:  When retrieving Link Type Definitions, if it does not exist, null is 
            //        returned.  Usage of the null object will result in a NullPointerException.
            System.out.println("Retrieving the Link Type Definition of 'S_policeReports'...");

                DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType("S_policeReports"); // Retrieve the definition.
                
            System.out.println("Retrieved the Link Type Definition of 'S_policeReports'.");
            
            //---------------------------------------------------------------
            // Print the Link Type Definition
            //---------------------------------------------------------------
            System.out.println("Printing the Link Type Definition of 'S_policeReports'...");
                printLinkTypeDef(linkTypeDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Link Type Definition of 'S_policeReports'.");

            //----------------------------------------------------------------
            // Retrieve the S_otherPolicies Link Type Definition
            //----------------------------------------------------------------
            System.out.println("Retrieving the Link Type Definition of 'S_otherPolicies'...");

                linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType("S_otherPolicies"); // Retrieve the definition.
                
            System.out.println("Retrieved the Link Type Definition of 'S_otherPolicies'.");

            //---------------------------------------------------------------
            // Print the Link Type Definition
            //---------------------------------------------------------------
            System.out.println("Printing the Link Type Definition of 'S_otherPolicies'...");
                printLinkTypeDef(linkTypeDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Link Type Definition of 'S_otherPolicies'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * List all Link Types available in the system.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void listLinkTypes(DKDatastoreICM dsICM) throws Exception{

        // Obtaining all Link Type Definitions in the System
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        dkCollection linkTypeColl = dsDefICM.listLinkTypes(); // Obtain all of the Link Type Definition objects in the system.

        // List all Link Types Available in the System
        System.out.println("All Link Types in System:");
        dkIterator iter = linkTypeColl.createIterator(); // create an iterator go iterate through collection.
        while(iter.more()){                              // while there still more items in the colleciton, continue
        	DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) iter.next(); // Move pointer to next element & return that object.
        	System.out.println("     - "+linkTypeDef.getName()+":  "+linkTypeDef.getDescription());
        }
    }//end listLinkTypes

   /**
    * Print the specified Link Type Definition
    * @param linkTypeDef  Link Type Definition, DKLinkTypeDefICM.
    **/
    public static void printLinkTypeDef(DKLinkTypeDefICM linkTypeDef) throws Exception{
        System.out.println("--------------------------------------------");
        System.out.println("            Link Type Definition");
        System.out.println("---------------------------------------------");
        System.out.println("           Name:  "+linkTypeDef.getName());
        System.out.println("    Description:  "+linkTypeDef.getDescription());
        System.out.println("        ID Code:  "+linkTypeDef.getCode());
        System.out.println("---------------------------------------------");
    }

   /**
    * Print the specified Link Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param linkTypeName  Link Type Name.
    **/
    public static void printLinkTypeDef(DKDatastoreICM dsICM, String linkTypeName) throws Exception{
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType(linkTypeName); // Retrieve the definition.
        printLinkTypeDef(linkTypeDef);   
    }

}//end class SLinkTypeDefinitionRetrievalICM
              
