/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Sample Data Model - Business Model Example, Insurance Model
    For a complete explanation of the Sample Data Model, please refer to the Sample
    SItemTypeCreationICM.

Complete Documentation
    Please refer to the Application Programming Guide (APG) and the Applicaiton 
    Programming Reference (APR) (ex: Javadoc) for complete and latest information.
    This sample will roughly cover the key concepts to get up and running with 
    Query / Search.

Search
    Searches may be performed based on virtually any of piece of an Item or Component, 
    text within an Item or Component, or text within Resource Content.
    
           Terminology      Description
        -----------------   ---------------------------------------------------------------
              Query         "Query" is the same as "Search".
        ...................................................................................
        Parametric Search   Searching based on Item & Component Properties, Attributes, 
                            References, Links, Folder Contents, etc.
        ...................................................................................
           Text Search      Searching within text and based on text contained in anything
                            marked as textSearchable(true) using a Text Search engine such
                            as IBM DB2 Text Information Extender (TIE) for DB2 v7 or IBM
                            DB2 Net Search Extender (NSE) for DB2 v8.
        ...................................................................................
         Combined Search    Searching using both Parametric & Text Search.
        -----------------------------------------------------------------------------------
        
    Execution Methods
        There is one easy-to-use search engine and three choices for how and when searches 
        should be executed and results returned.
        
        Execution  Method      Description
        ---------  ----------  -----------------------------------------------------------------
         Execute   execute()   Returns a Cursor that may be used to iterate over a collection 
                               of results.  Each result is retrieved in blocks, managed by the
                               system as the User-Program explicitly iterates through.  This
                               is especially beneficial for large numbers of results.  The 
                               User-Program is responsible for explicitly determining when to
                               spend the time retrieving blocks of results.
        ........................................................................................
        Evaluate   evaluate()  Returns all results as a collection.  All items are retrieved 
                               at once during this operation.  This is especially beneficial
                               for small numbers of results.  Large numbers of results may 
                               require a long time since all results will be retrieved at once,
                               if a maximum limit option is not specified.  If all are not
                               needed right away, Execute is the best alternative.
        ........................................................................................
         Execute   executeWit  Spawns a separate thread to execute the query & retrieve results.  
          With     hCallback() The thread will iterate over the Result Set, calling a "callback"
        Callback               object for each block of results.  A callback object is any 
                               object that implements the dkCallback interface.  This object 
                               is defined & created by the User-Program.  The 
                               SSearchCallbackObjectICM is an example of one such object.
        ----------------------------------------------------------------------------------------
        Note:  Funcitons Defined in com.ibm.mm.sdk.server.DKDatastoreICM (or other package).

    Accessing & Using Results
        Depending on the execution method chosen above, results are either returned as a 
        collection or as a Result Set Cursor.
        
        Results Collection
            The DKResults collection extends DKSequentialCollection and may be accessed in the 
            same way the DKFolder & DKLinkColleciton objects are accessed.  Please refer to
            the reference section at the bottom of the Samples readme for reference to
            samples demonstrating use of the DKSequentialCollection.
            
            In short, create a dkIterator to iterate through the collection.  A dkIterator
            is created by the DKSequentialCollection.createIterator() function.  The 
            dkIterator.more() & fecthNext() function may be used to access each DDO.
        
        Result Set Cursor
            The Result Set Cursor, dkResultSetCursor, returned is different from the 
            Results Collection.  The contents are retrieved in blocks by the system
            as needed.  The User-Program must make the explicit choice to iterate to 
            the next element or next N elements when desired.  The fetchNext() operation
            will return null when no more elemnts are to be found.
            
            IMPORTANT:  When finished using the Result Set Cursor, it *MUST* be destroyed
                        using the dkResultSetCursor.destroy() function in order for its
                        resources to be released on your system while your application is
                        connected.  All open cursors will be closed when the
                        DKDatastoreICM.disconnect() or destroy() methods are called.
                        However, it is best to destroy the cursor when you are done
                        using it in order to release resources that are limited in
                        number.  It is necessary if an application would otherwise
                        reach the limit on those resources such as open connections.

            Determining End of Results
                The cardinality(), number of results, is not supported in Content Manager
                v8.2 in order to maximize overall performance.  The fetchNext() operation
                will return null when no more elements are to be found.  

    Search Options
        Options are specified as NVPairs (Name-Value Pairs).

        Option Name  Constant                  Description
        -----------  ------------------------  -------------------------------------------------
        Max Results  DK_CM_PARM_MAX_RESULTS    Accompanied with an Integer value for the maximum
                                               number of results to return in the search.  Set 
                                               to '0' for no maximum.
        ........................................................................................
         Prepare     DK_CM_PREPARE_QUERY       Only prepare the query.  Do not open the Result
        Query Only                             Set Cursor.  The User-Program must explicitly
                                               open the cursor / execute the query at later.
                                               The value specified is ignored.
        ........................................................................................
         Prefetch    DK_CM_PARM_PREFETCH_SIZE  The block size in which results are retrieved
                                               from the datastore (if not using default).  A
                                               small value will allow for faster display of the
                                               first N results, but will increase overall 
                                               retrieval time for the whole result set.
        ........................................................................................
         Retrieval   DK_CM_PARM_RETRIEVE       Retrieval options to apply for retrieving each
          Options                              result.  For more information on Retrieval 
                                               Options, please refer to SItemRetrievalICM.
                                               Overrides Content Options below.
        ........................................................................................
          Content    DK_CM_PARM_CONTENT        In the IBM Content Manager (ICM) connector, 
          Options                              Retrieval Options listed above should be used
                                               instead.  Please refer to the APR (javadoc)
                                               for more information if desired.
        ........................................................................................
        Projection  DK_CM_PARM_PROJECTION_LIST Allows the user to apply a projection such that
           List                                the returned DDOs are only set with the desired
                                               attributes.
        ----------------------------------------------------------------------------------------
        Note: Constants defined in com.ibm.mm.sdk.common.DKConstant

    Search / Query String
        For complete documentation please refer to the Application Programming Guide.  The 
        section below includes the core elements to building Query Strings.  
        
        Core Symbols
            The construct that all queries fit into star with the base constructs explained
            below.

            Symbol  Description
            ------  ------------------------------------------------------------------------------
              /     Indicates a root Item or a direct child relationship of a Component.
              //    Indicates any descendent of an Item (child, grandchild, great grandchild, etc.)
              .     Represents the current Component in the hierarchy.
              ..    Represents the parent of the current Component.
              @     Denotes an Attribute.  Should be followed by an Attribute Name.
              [ ]   Denote a conditional statement or a list.
              ,     Separates Elements in a List of Values.
              =>    Dereference / Represents Linking or Referencing Action.
              *     Denotes All (Wildcard).
              %     Used with LIKE operator and Denotes any Sequence of Characters (Wildcard).
              _     Used with LIKE operator and Denotes any Single Character (Wildcard).

        Starting Point
            The entire Datastore is the starting point for any query.  The search must be 
            narrowed down by criteria specified.  Root Components / Items are the direct
            descendents or next level.

        Items / Item Types
            Search criteria that specifies a particular Item begins with '/' to indicate a
            direct descendent of the datastore.
        
                    Example              Syntax
            -------------------------    ----------------------------------------------
            All Items in Datastore       /*     
            ...........................................................................
            All Items of an Item Type    /<Item Type Name>
            ...........................................................................
            Multiple Item Types          (/<Item Type Name> | /<Item Type Name> | ...)
                                     OR  [/<Item Type 1>, /<Item Type 2>, ...]
            ...........................................................................
            All Insurance Policies       /S_insurePolicy
            ...........................................................................
            All Claims & Adjustments     (/S_autoClaim | /S_adjustrReport)
                                     OR  [/S_autoClaim, /S_adjustrReport]
            ---------------------------------------------------------------------------
            Note: All Queries are actually performed on Item / Component Type Views.
            
        Direct Descendents / Children
            To access direct descendents / direct children on the next sub-level of the component
            described, use '/' to indicate a direct descendent relationship.
            
                    Example             Syntax
            -------------------------   ---------------------------------------------------
            All Direct Children under   /<Item Type Name>/*
            particular Item Type        
            ...............................................................................
            All Child Components of     /<Item Type Name>/<Component Type Name>
            a particular Type of a   
            parent                   
            ...............................................................................
            All Grandchildren of a      /<Item Type Name>/<Comp Type Name>/<Comp Type Name>
            particular Child Type     
            ...............................................................................
            All Insured People of a     /S_insurePolicy/S_insured
            policy.
            -------------------------------------------------------------------------------
        
        Any Descendent / Sub-Component
            To access any descendent or component from any level under the parent, use '//' to
            indicate any descendent (child, grand child, great grand child, etc.).

                    Example             Syntax
            -------------------------   ------------------------------------------
            All Components System on    //*
            any level
            ......................................................................
            Any Component of a          //<Component Type Name>
            particular type
            ......................................................................
            All Descendents under       /<Item Type Name>//*
            particular Item Type
            ......................................................................
            All Descendents of a        /<Item Type Name>//<Component Type Name>
            particular Child Type
            ......................................................................
            All Insured People          //S_insured
            ----------------------------------------------------------------------

        Attribute Criteria
            To limit search based on an attribute and value, use the conditional statement
            with the following format:  "[@<Attr Name>=<Value>]".  
            
            NOTE:
                - Don't forget to use escape characters when specifying string values.  
                - Values may contain arithmetic operations that will be computed by the Query 
                  Engine before processing.
                - Functions such as latestVersion() or DB2 functions may be used as values.
                - If versioning is enabled, all versions will be returned unless latest
                  version is specified.

            Latest Version Performance Note
                - The most efficient way to specify latest version is to send the request
                  as an option for the query, using latest version retrieval option
                  documented in SItemRetrievalICM, rather than using the latest version
                  function in the query string.  The retrieval option will be equal or
                  faster than using the latest version function within a query string.

                    Example             Syntax
            -------------------------   ----------------------------------------------------------
            All Items with specific     /<Item Type Name>[@<Attr Name>=<Value> AND @<Attr Name>=<Value> AND ...]
            attribute and value          
            ......................................................................................
            Any Component with          //*[@<Attr Name>=<Value> AND @<Attr Name>=<Value> AND ...]
            Attribute & Value                             
            ......................................................................................
            All Children of an Item     /<Item Type Name>/<Comp Type Name>[@<Attr Name>=<Value>]
            & Child Type with the
            attribute and value
            ......................................................................................
            Any Component with Attr     //*[@<Attr Name>=[<Value1>,<Value2>,<Value3>,...]]
            Any Value Listed
            ......................................................................................
            Obtain the Item with        /*[@ITEMID = "A1001001A01J09B00241C95000"]
            the known Item ID     
            ......................................................................................
            Obtain any pictures         /S_autoPhoto[@S_vin="9GOAL56H3LI552314"]
            relating to this car
            ......................................................................................
            Obtain all Adjustments      /S_adjustrReport[@S_adjustmntDate BETWEEN "2000-12-31" AND "2002-01-01"]
            in the year 2001
            ......................................................................................
            Obtain the latest version   /S_insurePolicy[@VERSIONID = latest-version(.)]
            of an Insurance Policy
            --------------------------------------------------------------------------------------

        Attribute Existence
            Searches may be limited to only those Items that have a particular Attribute, 
            regardless of value.
            
                    Example             Syntax
            -------------------------   --------------------------------------------------------
            All Items that have the     /*[@<Attr Name>]
            specified Attr.
            ....................................................................................
            All Components that have    //*[@<Attr Name>] 
            the specified Attr.
            ....................................................................................
            Return any Item that has    /*[@S_claimNumber]
            a claim number in the 
            top level of it data tree.
            ....................................................................................
            Return any Item that has    /*[.//@S_claimNumber]
            a claim number anywhere 
            in its data tree.
            ------------------------------------------------------------------------------------

        Attribute Group Criteria
            To limit search based on an attribute beloging to an attribute group, use the
            conditional statement with the following format: 
            "[@<Attr Group Name>.<Attr Name>=<Value>]".

                    Example             Syntax
            -------------------------   -------------------------------------------------------------
            All Items with specific     /*[@<Attr Group Name>.<Attr Name>=<Value>]
            attribute and value
            .........................................................................................
            Return any Adjustments      /S_adjustrReport[@S_carAccident.S_licensePlate = "6USA676"]
            for the specified car
            -----------------------------------------------------------------------------------------

        Reference Attribute Criteria & Traversing
            To limit a search based on a reference attribute or traverse through References,
            the following examples explain how.

                    Example             Syntax
            -------------------------   -------------------------------------------------------------------------
            Limiting a search based     /<Item Type Name>[@<Ref Attr Name> => <Item Type Name>/[<some condition>]]
            on a Reference Attribute
            .....................................................................................................
            Traverse a Reference        /<Item Type Name>[<some condition]/@<Ref Attr Name> => <* or Item Type Name>[<some condition>]
            in the Forward Direction
            .....................................................................................................
            Traverse a Reference in     /<Item Type Name>/REFERENCEDBY/@REFERENCER => *
            the Reverse Direction
            .....................................................................................................
            Return all Adjustments      /S_adjustrReport[@S_autoClaim => S_autoClaim]
            that reference a Claim      NOTE: Don't forget that the reference attr name & Item Type name
                                              are in separate name spaces.  That is why it appears that
                                              the same name is listed twice, which is not the case.
            .....................................................................................................
            Return all Claims           /S_adjustrReport/@S_autoClaim => S_autoClaim
            referenced by any           NOTE: The first S_autoClaim refers to the Reference Attr
            adjustment report.                name and the second is an Item Type Name.
            .....................................................................................................
            Get all Components that     /S_autoClaim/REFERNCEDBY/@REFERENCER => *
            have references pointing 
            to any Claims               NOTE: S_autoClaim here is referring to the Item Type Name.
            -----------------------------------------------------------------------------------------------------

        Link Criteria & Traversing
            To limit a search based on a Link or traverse through Links, the following examples
            explain how.

                    Example             Syntax
            -------------------------   -------------------------------------------------------------------------
            Limiting a search based     /<Item Type Name>[./OUTBOUNDLINK[@LINKTYPE = "<Link Type Name>"]/@TARGETITEMREF => <Item Type>[<conditions...>]]
            on Outbound Link Target
            .....................................................................................................
            Traverse an Outbound Link   /<Item Type Name>[<conditions...>]/OUTBOUNDLINK[@LINKTYPE = "<Link Type Name>"]/@TARGETITEMREF => <Item Type>[<conditions...>]
            .....................................................................................................
            Traverse an Inbound Link    /<Item Type Name>[<conditions...>]/INBOUNDLINK[@LINKTYPE = "<Link Type Name>"]/@SOURCEITEMREF => <Item Type>[<conditions...>]
            .....................................................................................................
            Traverse a Link Item        /<Item Type Name>[<conditions...>]/OUTBOUNDLINK[@LINKTYPE = "<Link Type Name>"]/@LINKITEMREF => <Item Type>[<conditions...>]
            .....................................................................................................
            Return all Policies that    /S_insurePolicy[./OUTBOUNDLINK[@LINKTYPE="S_policeReports"]/@TARGETITEMREF => S_policeReport]
            have Police Reports
            .....................................................................................................
            Return all Police Reports   /S_insurePolicy[@S_policyNumber="10452136412"]/OUTBOUNDLINK[@LINKTYPE = "S_policeReports"]/@TARGETITEMREF => S_policeReport
            for an Insurance Policy
            .....................................................................................................
            Return all Policies         /S_policeReport[@S_reportNumber="01202001-30"]/INBOUNDLINK[@LINKTYPE = "S_policeReports"]/@SOURCEITEMREF => S_insurePolicy
            related to a Police Report
            -----------------------------------------------------------------------------------------------------

        Folder Criteria & Traversing
            To limit a search based on Folder contents or traverse Folder contents, the Outbound
            Link must be used.  As explained in SFolderICM, the DKFolder is implemented using Links.
            
            Note:  The official Folder Link Type Name may be found in 
                   DKConstant.DK_ICM_LINKTYPENAME_DKFOLDER

                    Example             Syntax
            -------------------------   -------------------------------------------------------------------------
            Limiting a search based     /<Item Type Name>[./OUTBOUNDLINK[@LINKTYPE = "DKFolder"]/@TARGETITEMREF => <Item Type>[<conditions...>]]
            on Folder Contents          
            .....................................................................................................
            Traverse Folder Contents    /<Item Type Name>[<conditions...>]/OUTBOUNDLINK[@LINKTYPE = "DKFolder"]/@TARGETITEMREF => <Item Type>[<conditions...>]
            .....................................................................................................
            Traverse an Backwards       /<Item Type Name>[<conditions...>]/INBOUNDLINK[@LINKTYPE = "DKFolder"]/@SOURCEITEMREF => <Item Type>[<conditions...>]
            (Folders Containing Item)
            .....................................................................................................
            Return all Adjustments      /S_adjustrReport[./OUTBOUNDLINK[@LINKTYPE="DKFolder"]/@TARGETITEMREF => S_autoPhoto]
            that have photos  
            .....................................................................................................
            Return all Photos in an     /S_adjustrReport[@S_policyNumber="10452136412"]/OUTBOUNDLINK[@LINKTYPE = "DKFolder"]/@TARGETITEMREF => S_autoPhoto
            Adjustment Report       
            .....................................................................................................
            Return all Adjustments      /S_autoPhoto[S_vin="1HOME10R4KL987258"]/INBOUNDLINK[@LINKTYPE = "DKFolder"]/@SOURCEITEMREF => S_adjustrReport
            that use this photo
            -----------------------------------------------------------------------------------------------------

        Text Search
            Searching within text and based on text contained in anything marked as 
            textSearchable(true) using a Text Search engine such as IBM DB2 Text
            Information Extender (TIE) for DB2 v7 or IBM DB2 Net Search Extender
            (NSE).  Individual Attributes and Item Type Definitions (for Resource
            Content) may be marked as textSearchable(true).

            Text Indexes are updated in a queue for processing.  Please refer to 
            STextIndexUpdateICM for more information on Text Indexes and updating
            Text Indexes.

            Resource Content
                In order to search Resource Content, use the @TIEREF attribute.
                The same search string is used as explained below.

            Text Search Basic
                The easiest, user-friendly Text Search method is Text Search Basic.  For 
                comprehensive, complete control, Text Search Advanced may be used, which 
                directly uses the DB2 Text Information Extender functions.  Using Text
                Search Basic also allows the underlying Text Search engine to be changed
                without affecting existing User-Programs.
                
                Syntax
                     Use a '+' to indicate the words or phrases that should be present in
                     the attribute specified.  Similarly, use a '-' to exclude other words
                     or phrases.  Note that the function should be equated with 1 to be true
                     and with 0 to be false.  Single terms do not need single quotes surrounding
                     them.  Phrases do require single quotes.
                     
                Sorting Results
                    The score-basic utilizes the ranking information returned by TIE or NSE,
                    which is used in this case to sort the results through SORTBY.
            
                        Example             Syntax
                -------------------------   ------------------------------------------------------------------------------------------------------------------------------------------
                Return Components that      //<Comp Type Name>[contains-text-basic(@<Attr Name>,"<Search Aguments / Text>")=1] SORTBY(score-basic(@<Attr Name>, "<Search Arguments / Text>"))
                contain specific text
                .....................................................................................................
                Return Resource Items       /<Item Type Name>[contains-text-basic(@TIEREF,"<Search Aguments / Text>")=1]
                containing specific text
                in the Resource Content.
                .....................................................................................................
                Return all Police Reports   /S_policeReport[./S_victimInfo[contains-text-basic(@S_licensePlate,"+089"])=1] SORTBY (@S_reportNumber ASCENDING)
                with specified partial
                license plate.
                & sort by attribute.
                .....................................................................................................
                Return all Policies for     /S_insurePolicy[contains-text-basic(@S_street,"+First +St -Ave")=1] SORTBY(score-basic(@S_street, "+First +St -Ave"))
                Street "First" and sort
                by same criteria.
                .....................................................................................................
                Return all Policies for     /S_insurePolicy[contains-text-basic(@S_street,"+'258 Green Rd.'")=1] SORTBY(score-basic(@S_street, "+'258 Green Rd.'"))
                Street "258 Green Rd." 
                & sort by same criteria.
                ----------------------------------------------------------------------------------------------------------------------------------------------------------------------
            
            Text Search Advanced
                Syntax
                    For the syntax supported by this function, please refer to the
                    documentation for IBM DB2 Text Information Extender (TIE) for
                    DB2 v7 or IBM DB2 Net Search Extender (NSE) for DB2 v8.  Note
                    that the contains-text() function should be equated with 1 to
                    be true and with 0 to be false.  Both single words and phrases
                    require single quotes around them.

                    In General, the Reference Material may be found in:
                        IBM DB2 Text Information Extender (TIE) for DB2 v7:
                            AIX: /usr/lpp/db2_07_01/doc/db2ext/cteu0mst.pdf
                            Win: %DB2HOME%\db2ext\doc\cteu0mst.pdf
                                 (Ex: DB2HOME = C:\Program Files\SQLLIB\)
                        IBM DB2 Net Search Extender (NSE) for DB2 v8:
                            Win: %DB2HOME%\doc\cteu9e80.pdf
                                 (Ex: DB2HOME = C:\Program Files\SQLLIB\)

                Sorting Results
                    The score function utilizes the ranking information returned
                    by TIE or NSE, which is used in this case to sort the results
                    through SORTBY.                

                        Example             Syntax
                -------------------------   ------------------------------------------------------------------------------------------------------------------------------------------
                Return Components that      //<Comp Type Name>[contains-text(@<Attr Name>,"<Search Aguments / Text>")=1] SORTBY(score(@<Attr Name>, "<Search Arguments / Text>"))
                contain specific text
                .....................................................................................................
                Return Resource Items       /<Item Type Name>[contains-text(@TIEREF,"<Search Aguments / Text>")=1]
                containing specific text
                in the Resource Content.
                .....................................................................................................
                Return all Police Reports   /S_policeReport[./S_victimInfo[contains-text(@S_licensePlate,"'089'"])=1] SORTBY (@S_reportNumber DESCENDING)
                with specified partial
                license plate.
                & sort by attribute.
                .....................................................................................................
                Return all Policies for     /S_insurePolicy[contains-text(@S_street,"'First' | 'Green'")=1] SORTBY(score(@S_street, "'First' | 'Green'"))
                Streets "First" or "Green"
                & sort by same criteria.
                ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        Sorting
            Results may be sorted by attribute sorting or by Text Search score function.

            Attribute Sorting
                Results may be sorted by attributes in either "ASCENDING" or "DESCENDING" order.
                by default, they are sorted by "ASCENDING" order.  Please refer to example 2 of
                the Text Search Basic section and example 2 from the Test Search Advanced
                section above for an example.

            Text Search Basic Sorting
                Results may be sorted by the text search score-basic function.  Please refer to 
                examples 1, 3, & 4 of the Text Search Basic section above for examples.
                
            Text Search Advanced Sorting
                Results may be sorted by the text search score function.  Please refer to 
                examples 1 & 3 of the Text Search Advanced section above for examples.
            
        Document Model

                    Example             Syntax
            -------------------------   -------------------------------------------------------------------------
            Return all documents        /Doc[contains-text(.//ICMPARTS/@TIEREF, " 'XML' ")=1]
            containing "XML" in any 
            of its parts.
            .....................................................................................................
            Return all parts with       /Doc[@ArchiveID = 555]/ICMPARTS/@SYSREFERENCEATTRS => *
            storage ID of 555
            .....................................................................................................
            Return all Parts of all     //ICMPARTS/@SYSREFERENCEATTRS => *
            Documents in system.
            -----------------------------------------------------------------------------------------------------

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSearchICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Searching / Querying Stored Content, Parametric Search, Text Search, 
 *                    Combined Search, Execution Methods, Execute, Evaluate, Execute with 
 *                    Callback, Callback Obects, Accessing & Using Results, Results Collection,
 *                    Result Set Cursor, Search Options, Search / Query String Syntax, Core
 *                    Symbols, +++
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Searching / Querying Stored Content.
 *                    Search / Query
 *                    Parametric Search
 *                    Text Search Basic
 *                    Text Search Advanced
 *                    Combined Search
 *                    Execution Methods
 *                    Execute
 *                    Evaluate
 *                    Execute With Callback
 *                    Accessing & Using Results
 *                    Results Collection
 *                    Result Set Cursor
 *                    Search Options
 *                    Search / Query String Syntax
 *                    Core Symbols
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SSearchICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: One set of the Insurance Model Items must exist in the datastore.  To 
 *                    remove duplicates and create a fresh, new single set of Items, run:
 *                         - SSampleModelItemDeletionICM (Optional)
 *                         - SInsuranceModelItemCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SSearchCallbackObjectICM.java
 ************************************************************************************************/
public class SSearchICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SSearchICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SSearchICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Execute
            //-------------------------------------------------------------
            // There is one easy to use search engine and three choices for
            // how and when searches should be executed and results returned.
            //
            // As explained in the header documentation, execute() returns
            // a cursor and is best for searches that allow the User-
            // Program to determine explicitly when to spend time retrieving.
            //
            // Example: Obtain all Insurance Claims & Adjustments in System.
            System.out.println("Executing:  Obtain all Insurance Claims & Adjustments in System...");

                String query = "(/S_autoClaim | /S_adjustrReport)";
                System.out.println("    Query:  "+query);
                                
                // Specify Search / Query Options
                DKNVPair options[] = new DKNVPair[3];
                options[0] = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "0"); // No Maximum (Default)                    // Specify max using a string value.
                options[1] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_ATTRONLY)); // Specify any Retrieval Options desired.  Default is ATTRONLY.
                options[2] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                           // Must mark the end of the NVPair

                dkResultSetCursor cursor = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);

            System.out.println("Executed:  Obtain all Insurance Claims & Adjustments in System.");

            //-------------------------------------------------------------
            // Using Result Set Cursor
            //-------------------------------------------------------------
            // The Cursor must be accessed using the fetchNext() for fetchNextN()
            // operations.  When null is returned, it has reached the end of the
            // results.  Please refer to the Application Programming Reference
            // for complete method listing and explanation.
            //
            // IMPORTANT:  When finished using the Result Set Cursor, it *MUST* be destroyed
            //             using the dkResultSetCursor.destroy() function in order for its
            //             resources to be released on your system while your application is
            //             connected.  All open cursors will be closed when the
            //             DKDatastoreICM.disconnect() or destroy() methods are called.
            //             However, it is best to destroy the cursor when you are done
            //             using it in order to release resources that are limited in
            //             number.  It is necessary if an application would otherwise
            //             reach the limit on those resources such as open connections.
            //
            // Determining End of Results
            //    The cardinality(), number of results, is not supported in Content Manager
            //    v8.2 in order to maximize overall performance.  The fetchNext() operation
            //    will return null when no more elements are to be found.  
            System.out.println("Accessing Result Set Cursor...");            

                  DKDDO  ddo;
                  String itemId = "";
                  
                  while((ddo = cursor.fetchNext())!=null){ // Get the next ddo & stop when ddo == null.
                       itemId = ((DKPidICM)ddo.getPidObject()).getItemId();
                       System.out.println("     - Item ID:  "+itemId+"  ("+ddo.getPidObject().getObjectType()+")");  // Print Item ID & Object Type
                  }
                  
                  // IMPORTANT STEP:
                  cursor.destroy();      // Close & Destroy Cursor, Ending Implied Transaction.
           
            System.out.println("Accessed Result Set Cursor.");            

            //-------------------------------------------------------------
            // Evaluate
            //-------------------------------------------------------------
            // As explained in the header documentation, evaluate() returns
            // a collection retrieved at once and is beneficial for small 
            // numbers of results.            
            //
            // Example: Return the item matching the given Item ID.
            System.out.println("Evaluating:  Return the item matching the given Item ID...");            
                                                       // Using Item ID from last DDO found in the Result Set above.
                query = "/*[@ITEMID=\""+itemId+"\"]";  // Looks like: /*[@ITEMID=\"A1001001A01J09B00241C95000\"];
                System.out.println("     Query:  "+query);
                
                // Specify Search / Query Options
                options    = new DKNVPair[3];
                options[0] = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "1");                                            // Specify max using a string value.
                options[1] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_ATTRONLY)); // Specify any Retrieval Options desired.  Default is ATTRONLY.
                options[2] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                           // Must mark the end of the NVPair

                DKResults results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);

            System.out.println("Evaluated:  Return the item matching the given Item ID.");            

            //-------------------------------------------------------------
            // Using Result Collection
            //-------------------------------------------------------------
            // The DKResult collection extends DKSequentialCollection.  It
            // is accessed using an iterator.
            System.out.println("Accessing Results Collection...");
                
                dkIterator iter = results.createIterator();
                
                // Print Number of Results
                System.out.println("  Number of Results:  "+results.cardinality());
                  
                while(iter.more()){   // While there are still items to iterate through, continue.                                        
                    ddo = (DKDDO) iter.next();  // Move pointer to next element and obtain that object.
                    System.out.println("     - Item ID:  "+((DKPidICM)ddo.getPidObject()).getItemId() + "  ("+ddo.getPidObject().getObjectType()+")");  // Print Item ID & Object Type
                }

            System.out.println("Accessed Results Collection.");

            //-------------------------------------------------------------
            // Execute With Callback
            //-------------------------------------------------------------
            // As explained in the header documentation, executeWithCallback()
            // spawns a separate thread to execute the query & retrieve
            // results.
            //
            // A callback object is any object that implements the dkCallback interface.
            // The SSearchCallBackObjectICM sample is an object that implements the dkCallback
            // interface.  Please refer to the Application Programming Reference for complete
            // information.
            //
            // Example: Obtain all Adjustments in the year 2001.
            System.out.println("Executing With Callback:  Return all Adjustments in the year 2001...");

                query = "/S_adjustrReport[@S_adjustmntDate BETWEEN \"2000-12-31\" AND \"2002-01-01\"]";
                System.out.println("                  Query:  "+query);
                
                // Specify Search / Query Options
                options    = new DKNVPair[3];
                options[0] = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "0"); // No Maximum                              // Specify max using a string value.
                options[1] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_ATTRONLY)); // Specify any Retrieval Options desired.  Default is ATTRONLY.
                options[2] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                           // Must mark the end of the NVPair

                // Create Callback Object (Any object implementing the dkCallback interface).
                SSearchCallbackObjectICM callbackObject = new SSearchCallbackObjectICM();

                dsICM.executeWithCallback(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options, callbackObject);

            System.out.println("Executed With Callback:  Return all Adjustments in the year 2001...");

            //-------------------------------------------------------------
            // Wait for Callback Object to receive all results
            //-------------------------------------------------------------
            // The callback object will receive results in increments.  This
            // example will simply wait until it is complete.
            System.out.println("Waiting for Callback Object to Receive All Results...");
            
                boolean isDone = false;               // Check to see if done.

                while(isDone == false){               // While not done, wait.
                    System.out.println("Waiting...");
                    java.lang.Thread.sleep(100);      // Wait
                    isDone = callbackObject.isDone(); // Check to see if done
                    if(callbackObject.error())        // if an error was detected, throw exception
                        throw callbackObject.getFirstException();
                }
                
                System.out.println("Done!");

            System.out.println("Waited for Callback Object to Receive All Results.");

            //-------------------------------------------------------------
            // Using Results Held in Our Sample Callback Object
            //-------------------------------------------------------------
            // Note that the callback object you design might not store 
            // results in it, but instead process the results directly.
            // Our sample process the results by storing them in a vector.
            // The code below will access those stored results.
            System.out.println("Accessing Our Callback Object's Results...");
            
                java.util.Vector resultsVector = callbackObject.getResults();
                
                for(int i=0; i<resultsVector.size(); i++){  // Go through all results, printing them
                    ddo = (DKDDO) resultsVector.elementAt(i);
                    System.out.println("     - Item ID:  "+((DKPidICM)ddo.getPidObject()).getItemId() + "  ("+ddo.getPidObject().getObjectType()+")");  // Print Item ID & Object Type
                }
            
            System.out.println("Accessed Our Callback Object's Results.");

            //-------------------------------------------------------------
            // More Examples
            //-------------------------------------------------------------
            // The same methods above may be applied to any query string.  
            // A number of example query strings relating to the Insurance
            // Model, Business Model Example, have been provided within 
            // the header documentation.  Some of those will be executed
            // below.
            System.out.println("Starting More Examples..."); 

                // Specify Search / Query Options
                options    = new DKNVPair[3];
                options[0] = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "0");  // No Maximum                             // Specify max using a string value.
                options[1] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_ATTRONLY)); // Specify any Retrieval Options desired.  Default is ATTRONLY.
                options[2] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                           // Must mark the end of the NVPair

                // 4
                query   = "/S_insurePolicy/S_insured";
                System.out.println("Evaluate:  All Insured People of Insurance Policies");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 5
                query   = "//S_insured";
                System.out.println(" Execute:  All Insured People");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 6
                query   = "/S_autoPhoto[@S_vin=\"9GOAL56H3LI552314\"]";
                System.out.println("Evaluate:  Find any pictures relating to this car");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 7
                query   = "/S_insurePolicy[@VERSIONID = latest-version(.)]";
                System.out.println("Evaluate:  Obtain the latest version of an Insurance Policy");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 8
                query   = "/*[.//@S_claimNumber]";
                System.out.println(" Execute:  Any Item that has claim number anywhere in its data tree");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 9
                query   = "/S_adjustrReport[@S_carAccident.S_licensePlate = \"6USA676\"]";
                System.out.println("Evaluate:  Find any Adjustments for the specified car");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 10
                query   = "/S_adjustrReport[@S_autoClaim => S_autoClaim]";
                System.out.println(" Execute:  Return all Adjustments that reference a Claim");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 11
                query   = "/S_adjustrReport/@S_autoClaim => S_autoClaim";
                System.out.println("Evaluate:  Return all Claims referenced by any adjustment report");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 12
                query   = "/S_insurePolicy[./OUTBOUNDLINK[@LINKTYPE=\"S_policeReports\"]/@TARGETITEMREF => S_policeReport]";
                System.out.println(" Execute:  Return all Insurance Policies that have Police Reports");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 13
                query   = "/S_policeReport[@S_reportNumber=\"01202001-30\"]/INBOUNDLINK[@LINKTYPE = \"S_policeReports\"]/@SOURCEITEMREF => S_insurePolicy";
                System.out.println("Evaluate:  Return all Policies related to a particular Police Report");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 14
                query   = "/S_adjustrReport[./OUTBOUNDLINK[@LINKTYPE=\"DKFolder\"]/@TARGETITEMREF => S_autoPhoto]";
                System.out.println(" Execute:  Return all Adjustments tat have photos");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 15
                query   = "/S_adjustrReport[@S_policyNumber=\"10452136412\"]/OUTBOUNDLINK[@LINKTYPE = \"DKFolder\"]/@TARGETITEMREF => S_autoPhoto";
                System.out.println("Evaluate:  Return all Photos in a particular Adjustment Report");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 16
                query   = "/S_policeReport[./S_victimInfo[contains-text-basic(@S_licensePlate,\"+089\")=1]] SORTBY (@S_reportNumber ASCENDING)";
                System.out.println(" Execute:  Return all Police Reports with the specified partial License Plate.");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.

                // 17
                query   = "/S_insurePolicy[contains-text-basic(@S_street,\"+First +St -Ave\")=1] SORTBY(score-basic(@S_street, \"+First +St -Ave\"))";
                System.out.println("Evaluate:  Return all Policies with a street with First & St, but not Ave.");
                System.out.println("   Query:  "+query);
                results = (DKResults)dsICM.evaluate(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(results);   // Use Wrapper Provided.

                // 18
                query   = "/S_insurePolicy[contains-text(@S_street,\"'First' | 'Green'\")=1] SORTBY(score(@S_street, \"'First' | 'Green'\"))";
                System.out.println(" Execute:  Return all Policies for Streets 'First' or 'Green'.");
                System.out.println("   Query:  "+query);
                cursor  = dsICM.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
                printResults(cursor);    // Use Wrapper Provided.
           
            System.out.println("Completed More Examples."); 

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print Item IDs & Object Type for Result Set Cursor.  Destory cursor when done.
    * @param cursor  Result Set Cursor obtained from the DKDatastoreICM.eexecute() function.
    **/
    public static void printResults(dkResultSetCursor cursor) throws DKException, Exception{

        DKDDO ddo;
        System.out.println("Results:");

        while((ddo = cursor.fetchNext())!=null){ // Get the next ddo & stop when ddo == null.
            System.out.println("     - Item ID:  "+((DKPidICM)ddo.getPidObject()).getItemId() + "  ("+ddo.getPidObject().getObjectType()+")");
        }
                  
        // VERY IMPORTANT STEP:
        cursor.destroy();      // Close & Destroy Cursor, Ending Implied Transaction.

        System.out.println("");  // For cleaner output, add two blank lines.
        System.out.println("");
    }

   /**
    * Print Item IDs & Object Type for Results Collection.
    * @param results  DKResults collection obtained from the DKDatastoreICM.evaluate() function.
    **/
    public static void printResults(DKResults results) throws DKException{
        
        dkIterator iter = results.createIterator(); // Create an iterator to go through Results collection.
                
        System.out.println("Results:");
        System.out.println("     - Total:  "+results.cardinality());
                
        while(iter.more()){   // While there are still items to iterate through, continue.                                        
            DKDDO ddo = (DKDDO) iter.next();  // Move pointer to next element and obtain that object.
            System.out.println("     - Item ID:  "+((DKPidICM)ddo.getPidObject()).getItemId() + "  ("+ddo.getPidObject().getObjectType()+")");  // Print Item ID & Object Type
        }
        
        System.out.println("");  // For cleaner output, add two blank lines.
        System.out.println("");
    }

}//end class SSearchICM
              
