
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample submit a parametric query to IP datastore

/*
 * Filename       : TSamplePQryIP.java
 * Description    : Connect to IP Datastore
 *                  Create query string
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  recursively display folder and its documents
 *                  Disconnect from IP Datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.Date;

// TSamplePQryIP class
public class TSamplePQryIP implements DKConstantIP {

  // Main method
  public static void main(String argv[]){

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
         pw = new String(""); // pw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TSamplePQryIP " +
        "<libSrv> <userid> <pw>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      }

    DKDatastoreIP dsIP = null;
    try {
       dsIP = new DKDatastoreIP();
       dkQuery pQry = null;
       DKResults pResults = null;
       DKNVPair parms[] = null;

       // Annotation support
       String s = new String( "APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;"
                              + "IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61" );

       System.out.println("connecting to ImagePlus datastore");
       dsIP.connect(libSrv,userid,pw,s);
       System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );

       // prepare the parametric query string
       String pcmd = "SEARCH=(ENTITY=FOLDER," +
                     "MAX_RESULTS=5," +
                     "COND=( FolderId==" +
                     "IPSAMP" +
                     " ) );" +
                     "OPTION=(CONTENT=YES)";

       System.out.println("query string " + pcmd);
       System.out.println("create query");
       pQry = dsIP.createQuery(pcmd,DK_PARAMETRIC_QL_TYPE,null);
       System.out.println("execute query");
       pQry.execute(parms);
       System.out.println("query executed");

       System.out.println("number of results " + pQry.numberOfResults());

       System.out.println("process query results");
       pResults = (DKResults)pQry.result();
       processResults((dkCollection)pResults);
       System.out.println("query results processed");

       dsIP.disconnect();
       System.out.println("datastore disconnected");
       dsIP.destroy();
    }
    catch (DKException exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception name " + exc.name());
       System.out.println("Exception message " + exc.getMessage());
       System.out.println("Exception error code " + exc.errorCode());
       System.out.println("Exception error state " + exc.errorState());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
  }

  public static void processResults(dkCollection results)
                throws DKException, Exception
  {
    int      numDataItems = 0;
    Short    sVal;
    short    itemType = 0;
    short    i = 0;
    short    k = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    DKDDO p = null;
    DKDDO item = null;
    dkXDO pXDO = null;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;
    int resultCounter = 0;
    DKPidXDO pidXDO = null;
    String pidString;
    System.out.println( "number of items " + results.cardinality());

    dkIterator iter = results.createIterator();

    while(iter.more())
    {
       item = (DKDDO) iter.next();
       if (item != null)
       {
          //item.retrieve()      // used when CONTENT=NO

          System.out.println("=====> Item " + ++resultCounter + " <=====");
          i++;
          numDataItems = item.dataCount();
          k = item.propertyId(DK_PROPERTY_ITEM_TYPE);
          if (k > 0)
            {
               sVal = (Short)item.getProperty(k);
               j = sVal.shortValue();
               switch (j)
               {
                  case DK_DOCUMENT :
                  {
                     System.out.println("Item is document ");
                     break;
                  }
                  case DK_FOLDER :
                  {
                     System.out.println("Item is folder ");
                     break;
                  }
               }  // end switch...
            }  // end valid property

            System.out.println("Pid                  : " + item.getPid().pidString());
            System.out.println("Number of Data Items : " + numDataItems);
            for (j = 1; j <= numDataItems; j++)
            {
               a = item.getData(j);
               strDataName = item.getDataName(j);
               System.out.println(j + ": Attribute Name   : " + strDataName );
               if (a instanceof Integer)
               {
                  System.out.println("   Integer Value    : " + a);
               }
               else if (a instanceof Short)
               {
                  System.out.println("   Short Value      : " + a);
               }
               else if (a instanceof DKTimestamp)
               {
                  System.out.println("   Timestamp value  : " + a);
               }
               else if (a instanceof DKDate)
               {
                  System.out.println("   Date value       : " + a);
               }
               else if (a instanceof String)
               {
                  System.out.println("   String Value     : " + a);
               }
               else if (a instanceof dkCollection)
               {
                  System.out.println("   Attribute Value is a collection");
                  pCol = (dkCollection)a;
                  pIter = pCol.createIterator();
                  i = 0;
                  while (pIter.more() == true)
                  {
                     i++;
                     a = pIter.next();
                     pDO = (dkDataObjectBase)a;
                     if (pDO.protocol() == DK_PDDO)
                     {
                        System.out.println("    DKDDO object " + i + " in collection");
                        System.out.println("      object's pid is " +
                              ((DKDDO)pDO).getPid().pidString());
                     }
                  } // end while
               } // end if collection
               else
               {
                  System.out.println("   Attribute Value " + a);
               }
            }  // endfor
       }
    }
   }
}
