/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

/*
 * Filename       : TAddViewOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 1 items.
 *                  Get the result in a resultset cursor.
 *                  Display documents found and their parts and attributes,
 *                  Add a view to the found document.
 *                  Display documents found and their parts and attributes,
 *                  Disconnect from OD Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TAddAnnotationOD class
public class TAddViewOD implements DKConstantOD
{
  static String appgrp = new String("");
  static String cmd = new String("");

  // Main method
  public static void main(String argv[])
  {
    DKDatastoreOD dsOD = null;
    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw     = new String(""),     // pw
             connect_string = new String("");

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TAddViewOD " + "<libSrv> <userid> <pw>, <connect string>" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
      }


      dsOD = new DKDatastoreOD();

      dkResultSetCursor pCur = null;

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Select a testcase by changing this number!!!
      int testcase = 10;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setAGQueryCriteria(testcase, libSrv);

      DKNVPair[] parms = new DKNVPair[3];
      parms[0] = new DKNVPair("APPL_GROUP", new String(appgrp));
      parms[1] = new DKNVPair("MAX_RESULTS", "1"); //new Integer(1));
      parms[2] = new DKNVPair("CONTENT", "YES");

      System.out.println("\nApplication group =>" + appgrp + "<=");
      System.out.println("Query string =>" + cmd + "<=\n");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " +
                         libSrv + ", userid: " + userid );

      DKDDO ddo = null;
      System.out.println("executing query");

      Date date;
      pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
      System.out.println("number of hits: " + pCur.cardinality());

      if (pCur == null || pCur.isValid() == false)
      {
        System.out.println("cursor is null or invalid!!!");
        return;
      }

      ddo = pCur.fetchNext();
      if (ddo != null)
      {
        displayViews(ddo);
        addView(dsOD, ddo);
      }
      pCur.destroy(); // Finished with the cursor
      pCur = null;

      // find out if the view is indeed added
      System.out.println("\nexecuting query 2nd time...");
      pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
      System.out.println("number of hits: " + pCur.cardinality());

      ddo = null;
      ddo = pCur.fetchNext();
      if (ddo != null)
      {
        displayViews(ddo);
      }
      pCur.destroy(); // Finished with the cursor

      date  = new Date();
      System.out.println("\nAfter results processed " + date.toString());
      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }

  }
  // --------------------------------------------------------------------
  // Convert regular month, day, year values to an OnDemand integer date
  // Asumming (and hopefully) the year is in the yyyy format
  // --------------------------------------------------------------------
  public static String ODdate( int month, int day, int year )
  {
    int[] days = {31,28,31,30,31,30,31,31,30,31,30,31};
    int OD_days = 0;
    String od_days_str;
    Integer OD_DAY = new Integer(0);

    // Just in case someone entered a 2 digit year
    if ( year < 100 )
    {
      // 21st century
      if (year < 70)
        year += 2000;
      else
        year += 1900;
    }

    // Get the numbers of days since 1/1/1970 to 1/1/yyyy
    OD_days = ( year - 1970 ) * 365;

    // Add days from leap years
    for ( int yr = 1970; yr < year; yr++ )
    {
      if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
      {
        OD_days++;
      }
    }

    // Add days sicne 1/1 of this year
    for ( int mon = 0; mon < month-1; mon++ )
    {
      OD_days += days[mon];
    }

    // Add the current day
    OD_days += day;

    // Additional Leap year adjustment
    if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
      OD_days++;

    od_days_str = OD_DAY.toString(OD_days);
    return od_days_str;
  }

  //--------------------------------------------------------------------
  // Add a view
  //--------------------------------------------------------------------
  public static void addView(DKDatastoreOD dsOD, DKDDO ddo)
  {
    try
    {
      DKPid pid = ddo.getPidObject();
      System.out.println("document pid string: " + pid.pidString());

      DKPidXDOOD newPid = new DKPidXDOOD();
      newPid.setDatastoreType(DK_OD_DSTYPE);
      newPid.setDatastoreName(dsOD.datastoreName());
      System.out.println("primaryid for xdo is "+pid.getPrimaryId());
      newPid.setPrimaryId(pid.getPrimaryId());
      newPid.setObjectType("DKBlobOD");
      newPid.setType(DK_OD_VIEW);
      DKBlobOD newBlob = new DKBlobOD(dsOD);
      newBlob.setPidObject(newPid);
      DKViewDataDefOD newView = new DKViewDataDefOD("Zoom 51%",
                                                    false,
                                                    50,
                                                    0,
                                                    50,
                                                    50,
                                                    DK_OD_GRAYSCALE_SCALE,
                                                    DK_OD_COLOR_YELLOW,
                                                    DK_OD_COLOR_RED,
                                                    DK_OD_COLOR_GREY,
                                                    DK_OD_COLOR_BLUE,
                                                    DK_OD_IMAGE_INTENSITY_NORMAL,
                                                    DK_OD_TEXT_FIDELITY_LINE,
                                                    DK_OD_OVERSTRIKE_MERGE,
                                                    false,
                                                    0,
                                                    0);
      newBlob.addExtension("DKViewDataDefOD", (dkExtension)newView);
      newBlob.add();  // this adds it to the database
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  //--------------------------------------------------------------------
  // Display views
  //--------------------------------------------------------------------
  public static void displayViews(DKDDO ddo)
  {
    int i = 0;
    dkXDO pXDO = null;
    String MimeType = new String("");

    try
    {
      short propId = ddo.dataId("DKViews");
      if (propId == 0)
      {
        System.out.println("\nNo logical views for document!");
      }
      else
      {
        DKViewDataOD views = (DKViewDataOD)ddo.getData(propId);
        DKViewDataDefOD view;

        int numViews = views.cardinality();
        if (numViews > 1)
          System.out.println("\nFound "+numViews+" views.");
        else
          System.out.println("\nFound "+numViews+" view.");

        dkIterator iter = views.createIterator();
        while (iter.more())
        {
          i += 1;
          pXDO = (dkXDO)iter.next();

          System.out.println("\n>>VIEW #" + i + "<<");

          // Retrieve the content for XDO
          pXDO.retrieve();
          MimeType = pXDO.getMimeType();
          System.out.println("  MimeType = " + MimeType);

          view = (DKViewDataDefOD)pXDO.getExtension("DKViewDataDefOD");

          System.out.println("  Name: " + view.getName());
          System.out.println(view.isPublic() ? "  Public" : "  Private");
          System.out.println("  Zoom: " + view.getZoom());
          System.out.println("  Copy Group: " + view.getCopyGroup());
          System.out.println("  Contrast: " + view.getContrast());
          System.out.println("  Brightness: " + view.getBrightness());
          System.out.println("  Grayscale: " + view.getGrayscale());
          System.out.println("  Background color: " + view.getBackgroundColor());
          System.out.println("  Heading color: " + view.getHeadingColor());
          System.out.println("  Selected area color: " + view.getSelectedAreaColor());
          System.out.println("  Image color: " + view.getImageColor());
          System.out.println("  Image intensity: " + view.getImageIntensity());
          System.out.println("  Text fidelity: " + view.getTextFidelity());
          System.out.println("\n");

        } //while (iter.more())

      } //if (propId != 0)

    } //try

    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // set application group and query SQL
  // --------------------------------------------------------------------
  public static void setAGQueryCriteria(int testcase, String libSrv)
  {
    int month = 6;
    int day   = 2;
    int year  = 1997;
    Integer temp_int = new Integer(0);
    String  mon_str, day_str, yr_str, date_str;
    String od_date = "";

    // Application Group and SQL string
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          appgrp = "INSURE  01 ";
          cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
          break;
        case 2:
          appgrp = "CHKSTMN301 ";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line/large object
          appgrp = "DELINQCY02 ";
          cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
          break;
        case 4:  // pdf
          appgrp = "KEYBANK 01 ";
          cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
          break;
      }
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          appgrp = "Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          appgrp = "CHKSTMNT";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line data
          appgrp = "Delinqcy-AG";
          cmd = "WHERE ( pagenum != 0 )";
          break;
        case 4:  // line data large object
          appgrp = "LOAN LO";
          od_date = ODdate( 10, 1, 1994 );
          cmd = "WHERE ( load_date = " + od_date + " )";
          //od_date = ODdate( 2, 2, 1970 );
          //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 5:  // pdf
          appgrp = "PDF landscape";
          cmd = "WHERE ( region LIKE '%' )";
        case 6:  // AFP
          appgrp = "afpdata";
          cmd = "WHERE (TaxYear = 1995)";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // Large object AFP - gunnar
          appgrp = "OnDemand Publications";
          cmd = "where bookname LIKE '%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest"))
    {
      switch (testcase)
      {
        default:
        case 1:
          // AFP data
          appgrp = "CRD";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          // Large Object line data
          appgrp = "sp-ebcdic-lo";
          //od_date = ODdate( 1, 1, 1990 );
          //od_date2 = ODdate( 12, 31, 2001 );
          //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
          od_date = ODdate( 12, 15, 2000 );
          cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
          break;
        case 3: // Loan Delinquency Reports - Large Objects
          appgrp = "LOAN LO";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 4:
          // Large Object line data
          appgrp = "eliz-largeline";
          od_date = ODdate( 3, 19, 1999 );
          cmd = "WHERE ( rptdate = " + od_date + " )";
          break;
      case 5:
        // Large Object line data
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CHECKSTMTS ";
          cmd = "WHERE ( KY1VAL LIKE '%' )";
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CheckStatements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          appgrp = "Few Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2: // Line - Loan Delinquency Reports
          appgrp = "Loan Delinquency Reports";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
      }
    }
    else // can be any server!
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          mon_str = temp_int.toString( month );
          day_str = temp_int.toString( day );
          yr_str  = temp_int.toString( year );
          date_str = mon_str + '/' + day_str + '/' + yr_str;
          // native OD integer date
          od_date = ODdate( month, day, year );
          System.out.println("The date '"+ date_str +"' equals to the OnDemand integer date '"+ od_date+"'" );

          appgrp = "CRD";
          //cmd = "WHERE ( crd_date = " + od_date + " )";
          cmd = "WHERE ( name LIKE '%')";
          break;
        case 2:  // line
          appgrp = "Balance";
          cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
          break;
      }
    }
  }
}

