/****************************************************************************************
@copyright(disclaimer)
Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and
modified versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR
ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION
OF THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT
SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,
HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS
OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
 ********************************************************************/


import  com.ibm.mm.beans.*;
import  com.ibm.mm.servlets.*;
import  com.ibm.mm.beans.util.CMBTraceLog;
import 	java.io.IOException;
import  javax.servlet.*;
import 	javax.servlet.http.*;
import  java.util.*;

/**
 * EIP v8 Servlet Toolkit action
 * <p>
 * This action mimics the searchQuery action (com.ibm.mm.servlets.CMBSearchQueryServlet)
 * <p>
 * The servlet extends the abstract class com.ibm.mm.servlets.CMBServletAction
 * <pre>
 * public abstract class CMBServletAction {
 *    public abstract void perform(ServletContext app, HttpSession session, HttpServletRequest request, HttpServletResponse res, Hashtable params) throws IOException, ServletException, CMBException, Exception;
 * }
 * </pre>
 * @since  v8.1
 */

public class CMBSampleActionServlet extends com.ibm.mm.servlets.CMBServletAction implements com.ibm.mm.servlets.CMBServletConstants {
    /**
     * @param application ServletContext
     * @param session HttpSession
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @param params Hashtable of parameter names and values in the client request
     * <p>
     **/
    public void perform(ServletContext application, HttpSession session, HttpServletRequest request, HttpServletResponse response, Hashtable params)
    throws IOException, ServletException, CMBException, Exception {

        // get beans from session
        // the naming convention is described in the cmbservlet.properties file
        CMBConnection connBean = (CMBConnection) session.getAttribute(BEAN_CMBConnection);
        CMBQueryService queryBean = (CMBQueryService) session.getAttribute(BEAN_CMBQueryService);
        if (queryBean == null)  queryBean = connBean.getQueryService();

        // retrieve the parameter queryString from the Hashtable
        String queryString = (String)ServletUtils.getRequiredParameterValue(params, request, PARM_QueryString);

        // OD only
        CMBQueryService.QueryParameter[] queryParams;

        // query syntax depends on the server type
        String dsType = connBean.getDsType();
        short queryType = 0;
        if (dsType.equals(CMB_DSTYPE_ICM)) {
            queryType = CMBBaseConstant.CMB_QS_TYPE_XPATH;
            queryParams = null;
        } else if (dsType.equals(CMB_DSTYPE_FED)) {
            queryType = CMB_QS_TYPE_FEDERATED;
            queryParams = null;
        } else if (dsType.equals(CMB_DSTYPE_DL) || dsType.equals(CMB_DSTYPE_IP390) || dsType.equals(CMB_DSTYPE_VI400) || dsType.equals(CMB_DSTYPE_FN)) {
            queryType = CMB_QS_TYPE_PARAMETRIC;
            queryParams = null;
        } else if (dsType.equals(CMB_DSTYPE_OD)) {
            queryType = CMB_QS_TYPE_SQL;
            queryParams = getQueryParameters(params,queryBean);
        } else if (dsType.equals(CMB_DSTYPE_DES)) {
            queryType = CMB_QS_TYPE_DES;
            queryParams = null;
        } else {
            queryType = CMB_QS_TYPE_SQL;
            queryParams = null;
        }

        queryBean.setAsynchSearch(false);

        // retrieve the property maxresults
        String maxResults = ServletUtils.getPropertyValue(application,"maxresults");
        // Set the max # of hits returned in a search
        queryBean.setMaxResults(Integer.parseInt(maxResults));

        if (queryParams != null) {
            queryBean.setQueryString(queryString, queryType, queryParams);
        } else {
            queryBean.setQueryString(queryString, queryType);
        }

        // run query
        queryBean.runQuery();

        // get search results
        CMBSearchResults resultBean = new CMBSearchResults();
        resultBean.setConnection(connBean);
        resultBean.newResults(queryBean.getResults());

        // keep bean in request scope (following the convention)
        request.setAttribute(BEAN_CMBSearchResults, resultBean);

    }

    private CMBQueryService.QueryParameter[] getQueryParameters(Hashtable params, CMBQueryService queryBean) {

        // get the number of query paramters.
        // syntax: {queryParameter.<parametername>=<parametervalue>}
        int count=0;
        Enumeration enum = params.keys();
        Vector parameters = new Vector(); // Vector of <parametername>
        while (enum.hasMoreElements()) {
            String paramname = (String) enum.nextElement();
            if (paramname.startsWith("queryParameter.") && params.get(paramname) != null) {
                parameters.add(paramname);
                count++;
            }
        }

        int index = 14; //index of "." in "queryParameter.<parametername>"

        // create CMBQueryService.QueryParameter object
        if (count > 0) {
            CMBQueryService.QueryParameter[] queryParams = new CMBQueryService.QueryParameter[count];
            for (int i=0; i<count; i++) {
                String name = (String) parameters.elementAt(i);  // <parametername>
                String value = (String)params.get(name); // <parametervalue>
                queryParams[i] = queryBean.new QueryParameter(name.substring(index+1), value);
            }
            return queryParams;
        }

        return null;
    }
}
